
//============================================================================
//				HEADERS
//============================================================================

#include <avr/io.h>
#include "Buttons.h"


//============================================================================
//				GLOBAL VARIABLES
//============================================================================

volatile unsigned short Buttons_FlagByte;


//============================================================================
//				FUNCTION PROTOTYPES
//============================================================================

void Buttons_Check(void);


//============================================================================
//				FUNCTIONS
//============================================================================

void Buttons_Check(void)
{
	static unsigned char ButtonCounter[4];
	unsigned char Button;

	for(Button=0; Button<4; Button++)
	{
// --- Button Pin Check ------------------------------------------------------

		// Button pressed (Pin --> GND)
		if( !(BUTTONS_PORT & (BUTTON_OFFSET<<Button)) )					
	    {
			if(ButtonCounter[Button] == 0)
			{
				// Button marked 'pressed'
				Buttons_FlagByte |=  (1<<(4*Button+PRESSED));	
			}
			if(ButtonCounter[Button] != LIMIT_HOLD)
			{
				// Button count
				ButtonCounter[Button]++;						
			}
		}
		// Button marked 'not pressed' (Pin --> 5V)
		else Buttons_FlagByte &= ~(1<<(4*Button+PRESSED));		


// --- ButtonCounter Check ----------------------------------------------------

		if(ButtonCounter[Button] == LIMIT_HOLD)// Button pressed hold
        {
			// Button pressed finished
			if( !( Buttons_FlagByte & (1<<(4*Button+PRESSED)) ) )		
			{
				ButtonCounter[Button] = 0;	
			}
			else
			{
				// Button marked 'pressed hold'
	            Buttons_FlagByte &= ~(1<<(4*Button+PRESSED_SHORT));
	            Buttons_FlagByte &= ~(1<<(4*Button+PRESSED_LONG));
	            Buttons_FlagByte |=  (1<<(4*Button+PRESSED_HOLD));	
			}
	    }
		// Button not pressed
		else if(ButtonCounter[Button] == 0)						
		{
            Buttons_FlagByte &= ~(1<<(4*Button+PRESSED_SHORT));
            Buttons_FlagByte &= ~(1<<(4*Button+PRESSED_LONG));
	        Buttons_FlagByte &= ~(1<<(4*Button+PRESSED_HOLD));
		}
		// Button pressed
		else if(ButtonCounter[Button] > 0)						
        {
	        Buttons_FlagByte &= ~(1<<(4*Button+PRESSED_HOLD));

			// Button pressed finished
			if( !( Buttons_FlagByte & (1<<(4*Button+PRESSED)) ) )		
			{
				// Button pressed too short
	            if(ButtonCounter[Button] < LIMIT_TOO_SHORT_AND_SHORT)
	    		{
	                Buttons_FlagByte &= ~(1<<(4*Button+PRESSED_SHORT));
	                Buttons_FlagByte &= ~(1<<(4*Button+PRESSED_LONG));
					ButtonCounter[Button] = 0;
	    		}
				// Button pressed short
	    		else if(ButtonCounter[Button] < LIMIT_SHORT_AND_LONG)
	            {
					// Button marked 'pressed short'
	                Buttons_FlagByte |=  (1<<(4*Button+PRESSED_SHORT));	
	                Buttons_FlagByte &= ~(1<<(4*Button+PRESSED_LONG));
					ButtonCounter[Button] = 0;
	            }
				// Button pressed long
	     		else if(ButtonCounter[Button] < LIMIT_HOLD)
	            {
					// Button marked 'pressed long'
	                Buttons_FlagByte &= ~(1<<(4*Button+PRESSED_SHORT));
	                Buttons_FlagByte |=  (1<<(4*Button+PRESSED_LONG));	
					ButtonCounter[Button] = 0;
				}
			}
		}
	}
}	

