
//============================================================================
//				HEADERS
//============================================================================

#include <avr/eeprom.h>


//============================================================================
//				DEFINITIONS
//============================================================================

#define Eeprom_WriteChar(pAddress,pValue)	_EEPUT(pAddress,pValue); \
											eeprom_busy_wait()

#define Eeprom_ReadChar(pVar,pAddress)		_EEGET(pVar,pAddress); \
											eeprom_busy_wait()

#define Eeprom_WriteNumber(pAddress,pValue)	Eeprom_WriteChar(pAddress+1,pValue%256); \
											Eeprom_WriteChar(pAddress,pValue/256)

#define Eeprom_ReadNumber(pValue,pAddress)	Eeprom_ReadChar(EepromH,pAddress);\
											Eeprom_ReadChar(EepromL,pAddress+1);\
											(pValue = 256*EepromH + EepromL)


#define EEP_ADR_MOTOR_RUNTIME_TOTAL			0x0000
#define EEP_ADR_MOTOR_RPM_MAX				0x0010
#define EEP_ADR_2ND_BYTE_1					0x0011			
#define EEP_ADR_MOTOR_TORQUE_MAX			0x0012
#define EEP_ADR_2ND_BYTE					0x0013			
#define EEP_ADR_MOTOR_POWER_MAX				0x0014
#define EEP_ADR_2ND_BYTE_2					0x0015			
#define EEP_ADR_MOTOR_LOAD_MAX				0x0016
#define EEP_ADR_COOLANT_TEMP_MIN			0x0017
#define EEP_ADR_COOLANT_TEMP_MAX			0x0018
#define EEP_ADR_MOTOR_RESETTIME				0x0019
#define EEP_ADR_2ND_BYTE_3					0x001A			

#define EEP_ADR_AIRINTAKE_TEMP_MIN			0x0020
#define EEP_ADR_AIRINTAKE_TEMP_MAX			0x0021
#define EEP_ADR_O2SENSOR1_MIN				0x0022
#define EEP_ADR_O2SENSOR1_MAX				0x0023
#define EEP_ADR_O2SENSOR2_MIN				0x0024
#define EEP_ADR_O2SENSOR2_MAX				0x0025
#define EEP_ADR_LUFT_RESETTIME				0x0026
#define EEP_ADR_2ND_BYTE_4					0x0027			

#define EEP_ADR_SPEED_MAX					0x0030
#define EEP_ADR_SPEED_AVG_HIGH				0x0031
#define EEP_ADR_2ND_BYTE_13					0x0032
#define EEP_ADR_SPEED_AVG_LOW				0x0033
#define EEP_ADR_2ND_BYTE_14					0x0034
#define EEP_ADR_VERBRAUCH_AVG_HIGH			0x0035
#define EEP_ADR_2ND_BYTE_15					0x0036
#define EEP_ADR_VERBRAUCH_AVG_LOW			0x0037
#define EEP_ADR_2ND_BYTE_16					0x0038
#define EEP_ADR_SPRINT_RECORD				0x0039

#define EEP_ADR_VOLTAGE_MIN					0x0040
#define EEP_ADR_VOLTAGE_MAX					0x0041
#define EEP_ADR_VOLTAGE0RPM_MIN				0x0042
#define EEP_ADR_VOLTAGE0RPM_MAX				0x0043
#define EEP_ADR_VOLTAGE_RESETTIME			0x0044
#define EEP_ADR_2ND_BYTE_5					0x0045			

#define EEP_ADR_SENSOR_DATE_DAY				0x0050
#define EEP_ADR_SENSOR_DATE_MONTH			0x0051
#define EEP_ADR_SENSOR_DATE_YEAR			0x0052
#define EEP_ADR_SENSOR_TIME_HOUR			0x0053
#define EEP_ADR_SENSOR_TIME_MIN				0x0054

#define EEP_ADR_LCD_BRIGHTNESS				0x0060
#define EEP_ADR_LED_COLOR					0x0061

#define EEP_ADR_OBD2_DELAY					0x0070
#define EEP_ADR_MENU_POINT_STATUS			0x0071

#define EEP_ADR_ERROR_COUNTER				0x0080
#define EEP_ADR_2ND_BYTE_6					0x0081			
#define EEP_ADR_RESET_JTAG					0x0082
#define EEP_ADR_2ND_BYTE_7					0x0083			
#define EEP_ADR_RESET_WATCHDOG				0x0084
#define EEP_ADR_2ND_BYTE_8					0x0085			
#define EEP_ADR_RESET_BROWNOUT				0x0086
#define EEP_ADR_2ND_BYTE_9					0x0087			
#define EEP_ADR_RESET_EXTERN				0x0088
#define EEP_ADR_2ND_BYTE_10					0x0089			
#define EEP_ADR_RESET_POWERON				0x008A
#define EEP_ADR_2ND_BYTE_11					0x008B			
#define EEP_ADR_RESET_BAD_INT				0x008C
#define EEP_ADR_2ND_BYTE_12					0x008D
#define EEP_ADR_ERROR_NULL_COUNTER			0x008E
#define EEP_ADR_ERROR_COMPARE_COUNTER		0x008F

#define EEP_ADR_STATISTIK_START				0x07F0
#define EEP_OFFSET_STATISTIK_POINTER		0x00
#define EEP_OFFSET_STATISTIK_DATE_DAY		0x01
#define EEP_OFFSET_STATISTIK_DATE_MONTH		0x02
#define EEP_OFFSET_STATISTIK_DATE_YEAR		0x03
#define EEP_OFFSET_STATISTIK_RPM_MAX		0x04
#define EEP_OFFSET_STATISTIK_LOAD_MAX		0x06
#define EEP_OFFSET_STATISTIK_POWER_MAX		0x08
#define EEP_OFFSET_STATISTIK_SPEED_MAX		0x0A
#define EEP_OFFSET_STATISTIK_SPEED_AVG		0x0B
#define EEP_OFFSET_STATISTIK_VERBRAUCH_AVG	0x0C
#define EEP_OFFSET_STATISTIK_RUNTIME		0x0E


//============================================================================
//				GLOBAL VARIABLES
//============================================================================

extern volatile unsigned char EepromH,EepromL;


//============================================================================
//				FUNCTION PROTOTYPES
//============================================================================

//extern void Eeprom_WriteNumber(unsigned long pHighAddress,unsigned short pValue);
//extern void Eeprom_ReadNumber(char* pDest,unsigned long pHighAddress);


