
// Elm327.c

//============================================================================
//				HEADERS
//============================================================================

#include <stdlib.h>
#include <string.h>
#include <avr/pgmspace.h>

#include "EvoBox.h"
#include "LCD_KS0108.h"
#include "Elm327.h"
#include "Sensors.h"
#include "uart.h"
#include "led.h"
#include "Eeprom.h"
#include "OBD.h"


//============================================================================
//				DEFINITIONS
//============================================================================


//============================================================================
//				GLOBAL VARIABLES
//============================================================================

char* Elm_Cmds[ELM_CMD_SIZE] = {
"AT",						// 0

"ELM327 v1.3a",				// 1

"<CR>",						// 2
"BRD",						// 3
"BRT",						// 4
"D",						// 5
"E0","E1",					// 6
"I",						// 8
"L0","L1",					// 9
"M0","M1",					// 11
"WS",						// 13
"Z",						// 14
"@1","@2",					// 15

"PP",						// 17
"PP",						// 18
"PPS",						// 19

"RV",						// 20
"CV",						// 21
"AL",						// 22
"AR",						// 23
"AT0","AT1","AT2",			// 24
"DP",						// 27
"DPN",						// 28
"MA",						// 29
"NL",						// 30
"PC",						// 31
"S0","S1",					// 32
"SP",						// 34
"SPA",						// 35

"CS",						// 36
"D0","D1",					// 37
"RTR",						// 39
"V0","V1",					// 40
};

unsigned char ErrorNoDataCounter;
unsigned short ErrorNullCounterCpy,ErrorNullCounterCat,ErrorNullCounter1,ErrorNullCounter2,ErrorNullCounter3,ErrorNullCounter4,ErrorNullCounter5,ErrorNullCounter6,ErrorNullCounter7;
unsigned short ErrorATRV;
unsigned short ErrorATI;

volatile unsigned char Elm_GlobalRequestCmd;
volatile unsigned char Elm_GlobalRequestMode;
volatile unsigned char Elm_GlobalRequestPID;

char UartTxBuffer[10];
char* pToCmd;
char* pToData;
char* pToResp;
char* pToRetVal;
volatile unsigned char  CmdLen, DataLen, RespLen;


//============================================================================
//				FUNCTION PROTOTYPES
//============================================================================

void Elm_Init(void);
void Elm_SendATCmd(unsigned char pCmd);
void Elm_SendATCmdPlusString(unsigned char pCmd, char* pString);
void Elm_SendOBDCmd(unsigned char pCmd, unsigned char pMode, unsigned pPID);
void Elm_SendOBDString(char* pString);
void Elm_AddHexDigit(unsigned char pDigit);
void Elm_AddHexNumber(unsigned char pNumber);
unsigned char Elm_ReadHexDigit(char* pPointer,unsigned char pIndex);
void Elm_SearchData(void);
void Elm_AnalyseATCmd(void);


//============================================================================
//				FUNCTIONS
//============================================================================

void Elm_Init(void)
{
	// Headline
	GLCD_FillScreen(0x00);
	GLCD_SetFontBold(1);
	GLCD_GoTo(0,0);
	GLCD_WriteStringM("EvoBox");

	GLCD_SetFontBold(0);
	GLCD_GoTo(0,1);
	GLCD_WriteStringM("(c)2009 Danilo Mueller");

	// Search Hello
	Elm_FreeSearchData();
	Elm_GlobalRequestCmd = ELM_ELM327_VERSION_HELLO;
	Elm_SearchData();

	GLCD_SetFontBold(0);
	GLCD_GoTo(0,3);
	GLCD_WriteStringM("OBD2-Chip: ");
	if(Do_FlagByte & DO_ELM_HELLO_OK)
	{
		GLCD_WriteStringM(Elm_Cmds[ELM_ELM327_VERSION_HELLO]);
		GLCD_GoTo(0,4);
		GLCD_WriteStringM("UART: 19200 Baud");
	}
	else
	{
		GLCD_WriteStringM("nicht erkannt!");
	}

	// Spaces OFF
	Elm_SendATCmd(ELM_OBD_PRINTING_SPACES_OFF);
	WaitInMs(300);	// 300 min
	Elm_SearchData();

	// Set Protocol
	Elm_SendATCmdPlusString(ELM_OBD_SET_PROTOCOL_AND_AUTO,ELM_PROTOCOL_ISO9141_2);
	WaitInMs(300);	// 300 min
	Elm_SearchData();

	GLCD_GoTo(0,5);
	GLCD_WriteStringM("Protokoll: ");
	if(Do_FlagByte & DO_ELM_SET_PROTOCOL_OK)
	{
		GLCD_WriteStringM("ISO-9141-2");
	}
	else
	{
		GLCD_WriteStringM("nicht gefunden!");
	}

	// Positiven OBD response erhalten
	Elm_SendOBDCmd(ELM_SEND_OBD,MODE_1_SHOW_CURRENT_DATA,PID_00_SUPPORTED_PIDS);
	WaitInMs(3000);	// Init 3000 min
	Elm_SearchData();

	GLCD_GoTo(0,6);
	GLCD_WriteStringM("OBD2: ");
	if(Do_FlagByte & DO_OBD_CMD_OK)
	{
		GLCD_WriteStringM("verbunden");
		Led_SetRGBLedColor(GREEN);
		Led_RefreshRGBLedColor();
		WaitInMs(300);
	}
	else
	{
		GLCD_WriteStringM("nicht verbunden!");
	}
}

void Elm_SendATCmd(unsigned char pCmd)
{
	if(Elm_GetSendCmdStatus() != ELM_LOCKED)
	{
		Elm_LockSendCmd();

		Elm_GlobalRequestCmd = pCmd;	

		Elm_StartString(Elm_Cmds[ELM_AT_CMD]);
		Elm_AddString(Elm_Cmds[pCmd]);
		Elm_AddString("\r");
		Elm_SendUart();
	//	uart_puts_P("ATRV\r");
		
		Elm_FreeSearchData();
	}
}

void Elm_SendATCmdPlusString(unsigned char pCmd, char* pString)
{
	if(Elm_GetSendCmdStatus() != ELM_LOCKED)
	{
		Elm_LockSendCmd();

		Elm_GlobalRequestCmd = pCmd;	

		Elm_StartString(Elm_Cmds[ELM_AT_CMD]);
		Elm_AddString(Elm_Cmds[pCmd]);
		Elm_AddString(pString);
		Elm_AddString("\r");
		Elm_SendUart();

		Elm_FreeSearchData();
	}
}

void Elm_SendOBDCmd(unsigned char pCmd, unsigned char pMode, unsigned pPID)
{
	if(Elm_GetSendCmdStatus() != ELM_LOCKED)
	{
		Elm_LockSendCmd();

		Elm_GlobalRequestCmd = pCmd;	
		Elm_GlobalRequestMode = pMode;
		Elm_GlobalRequestPID = 0;

		Elm_StartString("");
		Elm_AddHexNumber(pMode);
		if(pPID != 0xFF)
		{
			Elm_AddHexNumber(pPID);
			Elm_AddHexDigit(1);	// Only 1 Response
			Elm_GlobalRequestPID = pPID;
		}
		Elm_AddString("\r");
		Elm_SendUart();
	//	uart_puts_P("010C1\r");
		
		Elm_FreeSearchData();
	}
}

void Elm_SendOBDString(char* pString)
{
	char UartTxBuffer[10];

	if(Elm_GetSendCmdStatus() != ELM_LOCKED)
	{
		Elm_LockSendCmd();

		Elm_GlobalRequestCmd = ELM_SEND_OBD;	

		Elm_StartString(pString);
		Elm_AddString("\r");
		Elm_SendUart();

		Elm_FreeSearchData();
	}
}

void Elm_AddHexDigit(unsigned char pDigit)
{
	if(pDigit < 0x0A)
	{
		itoa(pDigit,NumberBuffer,10);
	}
	else
	{
		NumberBuffer[0] = pDigit - 0x0A + 'A' ;
		NumberBuffer[1] = 0;		
	}

	Elm_AddString(NumberBuffer);
}

void Elm_AddHexNumber(unsigned char pNumber)
{
	Elm_AddHexDigit(pNumber/16);
	Elm_AddHexDigit(pNumber%16);
}

unsigned char Elm_ReadHexDigit(char* pPointer,unsigned char pIndex)
{
	unsigned char Char,RetVal;

	Char = *(pPointer + pIndex);

	if(Char >= '0' && Char <= '9')
	{
		RetVal = Char - '0';
	}
	else if(Char >= 'A' && Char <= 'F') 
	{
		RetVal = Char - 'A' + 10;
	}
	else if(Char >= 'a' && Char <= 'f') 
	{
		RetVal = Char - 'a' + 10;
	}
	else RetVal = 0;

	return RetVal;
}

void Elm_SearchData(void)
{
	unsigned char  i;
    unsigned short Data=0;
	unsigned char  Char=0;
	         char* pToCR;
	unsigned char  BufferIndex=0;
	         char  UartRxBuffer[128],SearchCmd[8],ReadCmd[8];


//	static unsigned short SearchDataCounter;

	if(Elm_GetSearchDataStatus() != ELM_LOCKED)
	{
		Elm_LockSearchData();

		for(i=0; i<128; i++) UartRxBuffer[i] = 0;
		for(i=0; i<8; i++) SearchCmd[i] = 0;
		for(i=0; i<8; i++) ReadCmd[i] = 0;

/*		SearchDataCounter++;
		GLCD_GoTo(60,5);
		GLCD_WriteNumberM(SearchDataCounter);
*/
		while(1)
		{

			// Get received character from ringbuffer
			// uart_getc() returns in the lower byte the received character and 
			// in the higher byte (bitmask) the last receive error

			Data = uart_getc();
			Char = (unsigned char)(Data);

			// quit, if there is no data
			if (Data & UART_NO_DATA)
			{
				// no data available from UART 
				break;
			}
			else
			{
				// new data available from UART
				// check for Frame or Overrun error
				if (Data & UART_FRAME_ERROR)
				{


					// Framing Error detected, i.e no stop bit detected 
					uart_puts_P("UARTE1");
				}

				if (Data & UART_OVERRUN_ERROR)
				{
					// Overrun, a character already present in the UART UDR register was 
					// not read by the interrupt handler before the next character arrived,
					// one or more received characters have been dropped
					uart_puts_P("UARTE2");
				}

				if (Data & UART_BUFFER_OVERFLOW)
				{
					// We are not reading the receive buffer fast enough,
					// one or more received character have been dropped 
					uart_puts_P("UARTE3");
				}

				// quit, if complete ELM message is received
				if(Char == ELM_ELM_READY)
				{
					UartRxBuffer[BufferIndex] = 0;
					break;
				}

				UartRxBuffer[BufferIndex] = Char;

				if(BufferIndex < 127) BufferIndex++;
			}
		} // while(1)

	//	GLCD_GoTo(0,6);
	//	strncpy(StringBuffer,UartRxBuffer,25);
	//	GLCD_WriteStringM(StringBuffer);
	
	//	GLCD_GoTo(0,4);

		// Check, if data available
		if(!(Data & (UART_NO_DATA | UART_FRAME_ERROR | UART_OVERRUN_ERROR | UART_BUFFER_OVERFLOW) ) )
		{
			// Check last character from ELM is Elm_ELM_READY '>' (transmitting data from ELM327 finished)
			if(Char == ELM_ELM_READY)
			{
				///////////////////////////////////////////////////////////////
				// Check for reading an OBD-Response
				if(Elm_GlobalRequestCmd == ELM_SEND_OBD)
				{
					// Simulate UartRxBuffer
					#ifdef CFG_OBD_SIMULATION_ENABLE
					     if(Elm_GlobalRequestPID == PID_00_SUPPORTED_PIDS)
						pToRetVal = strcpy(UartRxBuffer,"01011\r41018207E504\r");
					else if(Elm_GlobalRequestPID == PID_01_DTC_STATUS)
						pToRetVal = strcpy(UartRxBuffer,"01011\r41018207E504\r");
					else if(Elm_GlobalRequestPID == PID_03_FUEL_SYSTEM_STATUS)
						pToRetVal = strcpy(UartRxBuffer,"01031\r41030400\r");
					else if(Elm_GlobalRequestPID == PID_04_CALC_ENGINE_LOAD)
						pToRetVal = strcpy(UartRxBuffer,"01041\r41043F\r");
					else if(Elm_GlobalRequestPID == PID_05_ENGINE_COOLANT_TEMP)
						pToRetVal = strcpy(UartRxBuffer,"01051\r410532\r");
					else if(Elm_GlobalRequestPID == PID_06_FUEL_SHORT_TRIM_BANK1)
						pToRetVal = strcpy(UartRxBuffer,"01061\r410680\r");
					else if(Elm_GlobalRequestPID == PID_07_FUEL_LONG_TRIM_BANK1)
						pToRetVal = strcpy(UartRxBuffer,"01071\r410787\r");
					else if(Elm_GlobalRequestPID == PID_0B_INTAKE_PRESS)
						pToRetVal = strcpy(UartRxBuffer,"010B1\r410B2A\r");
					else if(Elm_GlobalRequestPID == PID_0C_ENGINE_RPM)
						pToRetVal = strcpy(UartRxBuffer,"010C1\r410C16C4\r");
					else if(Elm_GlobalRequestPID == PID_0D_VEHICLE_SPEED)
						pToRetVal = strcpy(UartRxBuffer,"010D1\r410D0B\r");
					else if(Elm_GlobalRequestPID == PID_0E_TIMING_ADVANCE)
						pToRetVal = strcpy(UartRxBuffer,"010E1\r410E9E\r");
					else if(Elm_GlobalRequestPID == PID_0F_INTAKE_TEMP)
						pToRetVal = strcpy(UartRxBuffer,"010F1\r410F4B\r");
					else if(Elm_GlobalRequestPID == PID_10_MASS_AIR_FLOW)
						pToRetVal = strcpy(UartRxBuffer,"01101\r4110036F\r");
					else if(Elm_GlobalRequestPID == PID_11_THROTTLE_POS)
						pToRetVal = strcpy(UartRxBuffer,"01111\r411104\r");
					else if(Elm_GlobalRequestPID == PID_13_O2_SENSORS_PRESENT)
						pToRetVal = strcpy(UartRxBuffer,"01131\r411303\r");
					else if(Elm_GlobalRequestPID == PID_14_O2_BANK1_SENSOR1)
						pToRetVal = strcpy(UartRxBuffer,"01141\r4114A476\r");
					else if(Elm_GlobalRequestPID == PID_15_O2_BANK1_SENSOR2)
						pToRetVal = strcpy(UartRxBuffer,"01151\r41150074\r");
					else if(Elm_GlobalRequestPID == PID_1C_OBD_STANDARDS)
						pToRetVal = strcpy(UartRxBuffer,"011C1\r411C01\r");
					#endif

					if(pToRetVal == 0) ErrorNullCounterCpy++;

					// Build searched command
					pToRetVal = strncpy(SearchCmd,UartTxBuffer,strlen(UartTxBuffer)-1);

					if(pToRetVal == 0) ErrorNullCounterCpy++;

/*					GLCD_GoTo(0,0);
					GLCD_WriteStringM(UartRxBuffer);
					GLCD_GoTo(0,1);
					GLCD_WriteStringM(SearchCmd);
*/
					// Calculate start of received command
					pToCmd = strstr(UartRxBuffer,SearchCmd);
					if(pToCmd != 0)
					{
						// Search for next CR between command and data
						pToCR = strchr(pToCmd,ELM_ELM_RETURN);
						if(pToCR != 0)					
						{
							// Calculate length of received command
							CmdLen = pToCR - pToCmd;
					
							// Calculate start of received response
							pToResp = pToCR + 1;

							// Read command
							pToRetVal = strncpy(ReadCmd,pToCmd,CmdLen);

							if(pToRetVal == 0) ErrorNullCounterCpy++;

							if(CmdLen < 7)
							{
								// Check, if received command is identical to transmitted command (without "AT"-expression)
								if(strcmp(ReadCmd,SearchCmd) == 0)
								{
									// Search for next CR after response
									pToCR = strchr(pToResp,ELM_ELM_RETURN);
									if(pToCR != 0) 
									{
										// Calculate lenght of received response
										RespLen = pToCR - pToResp;
					
										// Calculate start of received data
										pToData = pToResp + 4;

										// Calculate lenght of received data
										DataLen = RespLen - 4;

										// Check positive Response
										if(Elm_ReadHexNumber(pToResp,0)  == Elm_ReadHexNumber(pToCmd,0) + MODE_POSITIVE_RESPONSE)
										{	
											// Analyse OBD commands
											OBD_AnalyseOBDCmd();
										}
										else
										{
											//GLCD_GoTo(0,7);
											//GLCD_WriteStringM("NegResp");
										}

									} // pToCR != 0
									else
									{
										ErrorNullCounter1++;
									}
											
								} // strcmp(ReadCmd,Elm_Cmds[Elm_GlobalRequestCmd]) == 0
								
							} // CmdLen < 7

						} // pToCR != 0
						else
						{
							ErrorNullCounter2++;
						}
						
					} // pToCmd != 0
					else
					{
						ErrorNullCounter3++;
					}
				}
				///////////////////////////////////////////////////////////////
				// Check for reading first Hello-Message from ELM327
				else if(Elm_GlobalRequestCmd == ELM_ELM327_VERSION_HELLO)
				{
					pToCmd = strstr(UartRxBuffer,"\r\r");
					if(pToCmd != 0)
					{
						if(strncmp(pToCmd+2,Elm_Cmds[ELM_ELM327_VERSION_HELLO],12) == 0)
						{
							// equal
							Do_FlagByte |= DO_ELM_HELLO_OK;
						}					
					}
					else
					{
						ErrorNullCounter4++;
					}
				}
				///////////////////////////////////////////////////////////////
				// Check for reading AT commands
				else
				{
					// Build searched command
					pToRetVal = strncpy(SearchCmd,UartTxBuffer,strlen(UartTxBuffer)-1);

					if(pToRetVal == 0) ErrorNullCounterCpy++;

				/*	GLCD_GoTo(0,0);
					GLCD_WriteStringM(UartRxBuffer);
					GLCD_GoTo(0,1);
					GLCD_WriteStringM(SearchCmd);
				*/
					// Calculate start of received command
					pToCmd = strstr(UartRxBuffer,SearchCmd);
					if(pToCmd != 0)
					{
						// Search for next CR between command and data
						pToCR = strchr(pToCmd,ELM_ELM_RETURN);
						if(pToCR != 0)					
						{
							// Calculate length of received command
							CmdLen = pToCR - pToCmd;
					
							// Calculate start of received data
							pToData = pToCR + 1;

							// Read command
							pToRetVal = strncpy(ReadCmd,pToCmd,CmdLen);
						
							if(pToRetVal == 0) ErrorNullCounterCpy++;

							if(CmdLen < 7)
							{
								// Check, if received command is identical to transmitted command (without "AT"-expression)
								if(strcmp(ReadCmd,SearchCmd) == 0)
								{
									// Search for next CR after data
									pToCR = strchr(pToData,ELM_ELM_RETURN);
									if(pToCR != 0) 
									{
										// Calculate lenght of received data
										DataLen = pToCR - pToData;

										// Analyse AT commands
										Elm_AnalyseATCmd();

									} // pToCR != 0
									else
									{
										ErrorNullCounter5++;
									}
											
								} // strcmp(ReadCmd,Elm_Cmds[Elm_GlobalRequestCmd]) == 0

							} // CmdLen < 5
							
						} // pToCR != 0
						else
						{
							ErrorNullCounter6++;
						}

					} // pToCmd != 0
					else
					{
						ErrorNullCounter7++;
					}

				} // Elm_GlobalRequestCmd == Elm_ELM_VERSION
				
				Elm_FreeSendCmd();

			} // Char == Elm_ELM_READY

			// No Elm_ELM_READY received
			else
			{
			}
			
		} // !(Data & UART_NO_DATA) && !(Data & UART_BUFFER_OVERFLOW)
		else
		{
			Elm_FreeSendCmd();

			ErrorNoDataCounter++;
		}
		
	} // Do_FlagByte & DO_ELM327_CMD_SENT

}

void Elm_AnalyseATCmd(void)
{
	switch(Elm_GlobalRequestCmd)
	{
/*		case ELM_GENERAL_PRINT_VERSION_ID:
			if(strncmp(pToData,Elm_Cmds[ELM_ELM327_VERSION_HELLO],12) == 0)
			{
				// eaual
				//GLCD_GoTo(0,4);
				//GLCD_WriteStringM("Hello OK!");
				//ErrorATI++;
			}
			break;

		case ELM_OBD_PRINTING_SPACES_OFF:
			if(strncmp(pToData,"OK",2) == 0)
			{
				// eaual
				GLCD_GoTo(0,4);
				GLCD_WriteStringM("Space OK!");
			}
			break;

		case ELM_OBD_DESCRIBE_CURR_PROTOCOL:
			break;
*/
		case ELM_OBD_SET_PROTOCOL_AND_AUTO:
			if(strncmp(pToData,"OK",2) == 0)
			{
				// eaual
				Do_FlagByte |= DO_ELM_SET_PROTOCOL_OK;
			}												
			break;

		case ELM_READ_VOLTAGE:
			if(DataLen == 4)
			{
				Sensor_Voltage[0] = 10*Elm_ReadDecDigitData(0) + Elm_ReadDecDigitData(2);
			}
			else if(DataLen == 5)
			{
				Sensor_Voltage[0] = 100*Elm_ReadDecDigitData(0) + 10*Elm_ReadDecDigitData(1) + Elm_ReadDecDigitData(3);
			}

			if(Sensor_Voltage[0] < SensorCalc_VoltageMin[0]) SensorCalc_VoltageMin[0] = Sensor_Voltage[0];
			else if(Sensor_Voltage[0] > SensorCalc_VoltageMax[0]) SensorCalc_VoltageMax[0] = Sensor_Voltage[0];

			if(Sensor_MotorRpm[0] < 250)
			{
				if(DataLen == 4)
				{
					Sensor_Voltage0Rpm[0] = 10*Elm_ReadDecDigitData(0) + Elm_ReadDecDigitData(2);
				}
				else if(DataLen == 5)
				{
					Sensor_Voltage0Rpm[0] = 100*Elm_ReadDecDigitData(0) + 10*Elm_ReadDecDigitData(1) + Elm_ReadDecDigitData(3);
				}

				if(Sensor_Voltage0Rpm[0] < SensorCalc_Voltage0RpmMin[0]) SensorCalc_Voltage0RpmMin[0] = Sensor_Voltage0Rpm[0];
				else if(Sensor_Voltage0Rpm[0] > SensorCalc_Voltage0RpmMax[0]) SensorCalc_Voltage0RpmMax[0] = Sensor_Voltage0Rpm[0];
			}

			ErrorATRV++;
			break;

		default:
			break;
	} // switch(Elm_GlobalRequestCmd)
}

