
// Elm327.h

//============================================================================
//				HEADERS
//============================================================================


//============================================================================
//				DEFINITIONS
//============================================================================

#define Elm_StartString(pString)			strcpy(UartTxBuffer,pString);
#define Elm_AddString(pString)				pToRetVal = strcat(UartTxBuffer,pString);\
											if(pToRetVal == 0) ErrorNullCounterCat++

#define Elm_StartNumber(pNumber)			itoa(pNumber,NumberBuffer,10);\
											strcpy(UartTxBuffer,NumberBuffer)

#define Elm_AddNumber(pNumber)				itoa(pNumber,NumberBuffer,10);\
											pToRetVal = strcat(UartTxBuffer,NumberBuffer);\
											if(pToRetVal == 0) ErrorNullCounterCat++

#define Elm_SendUart()						uart_puts(UartTxBuffer);

#define Elm_ReadDecDigitData(pIndex)		(*(pToData + pIndex) - '0')
#define Elm_ReadHexNumber(pPointer,pIndex2)	16*Elm_ReadHexDigit(pPointer,pIndex2) + Elm_ReadHexDigit(pPointer,pIndex2+1)

#define Elm_LockSendCmd()					(Do_FlagByte &= ~DO_ELM_READY)
#define Elm_FreeSendCmd()					(Do_FlagByte |=  DO_ELM_READY)
#define Elm_GetSendCmdStatus()				(Do_FlagByte &   DO_ELM_READY)

#define Elm_LockSearchData()				(Do_FlagByte &= ~DO_ELM_CMD_SENT)
#define Elm_FreeSearchData()				(Do_FlagByte |=  DO_ELM_CMD_SENT)
#define Elm_GetSearchDataStatus()			(Do_FlagByte &   DO_ELM_CMD_SENT)

#define ELM_LOCKED							0


#define ELM_ELM_RETURN						0x0D
#define ELM_ELM_READY						'>'

#define ELM_AT_CMD							0

#define ELM_ELM327_VERSION_HELLO			1						

#define ELM_GENERAL_REPEAT_LAST_CMD			2
#define ELM_GENERAL_TRY_BAUDRATE_DIVISOR	3
#define ELM_GENERAL_SET_BAUDRATE_TIMEOUT	4
#define ELM_GENERAL_SET_ALL_TO_DEFAULT		5
#define ELM_GENERAL_ECHO_OFF				6
#define ELM_GENERAL_ECHO_ON					7
#define ELM_GENERAL_PRINT_VERSION_ID		8
#define ELM_GENERAL_LINEFEEDS_OFF			9
#define ELM_GENERAL_LINEFEEDS_ON			10
#define ELM_GENERAL_MEMORY_OFF				11
#define ELM_GENERAL_MEMORY_ON				12
#define ELM_GENERAL_WARM_START				13
#define ELM_GENERAL_RESET_ALL				14
#define ELM_GENERAL_DEVICE_DESCRIPTION		15
#define ELM_GENERAL_DEVICE_IDENTIFIER		16

#define ELM_PROGPAR_SET_ENABLE				17
#define ELM_PROGPAR_SET_VALUE				18
#define ELM_PROGPAR_PRINT_SUMMARY			19

#define ELM_READ_VOLTAGE					20	// OK
#define ELM_VOLTAGE_CALIBRATE				21

#define ELM_OBD_ALLOW_LONG_MESSAGES			22
#define ELM_OBD_AUTOMATICALLY_RECEIVE		23
#define ELM_OBD_ADAPTIVE_TIMING_OFF			24
#define ELM_OBD_ADAPTIVE_TIMING_AUTO1		25
#define ELM_OBD_ADAPTIVE_TIMING_AUTO2		26
#define ELM_OBD_DESCRIBE_CURR_PROTOCOL		27
#define ELM_OBD_DESCRIBE_PROTOCOL_NR		28
#define ELM_OBD_MONITOR_ALL					29
#define ELM_OBD_NORMAL_LENGTH_MESSAGES		30
#define ELM_OBD_PROTOCOL_CLOSE				31
#define ELM_OBD_PRINTING_SPACES_OFF			32
#define ELM_OBD_PRINTING_SPACES_ON			33
#define ELM_OBD_SET_PROTOCOL				34
#define ELM_OBD_SET_PROTOCOL_AND_AUTO		35

#define ELM_CAN_SHOW_CAN_STATUS_COUNTS		36
#define ELM_CAN_DISPLAY_DLC_OFF				37
#define ELM_CAN_DISPLAY_DLC_ON				38
#define ELM_CAN_SEND_RTR_MESSAGE			39
#define ELM_CAN_USE_OF_DLC_OFF				40
#define ELM_CAN_USE_OF_DLC_ON				41

#define ELM_CMD_SIZE						42
#define ELM_SEND_OBD						43

#define ELM_PROTOCOL_AUTO					"0"
#define ELM_PROTOCOL_J1850_PWM				"1"
#define ELM_PROTOCOL_J1850_VPW				"2"
#define ELM_PROTOCOL_ISO9141_2				"3"
#define ELM_PROTOCOL_KWP_SLOW				"4"
#define ELM_PROTOCOL_KWP_FAST				"5"
#define ELM_PROTOCOL_CAN_11BIT_500KB		"6"
#define ELM_PROTOCOL_CAN_29BIT_500KB		"7"
#define ELM_PROTOCOL_CAN_11BIT_250KB		"8"
#define ELM_PROTOCOL_CAN_29BIT_250KB		"9"
#define ELM_PROTOCOL_J1939_CAN_29BIT_250KB	"A"
#define ELM_PROTOCOL_USER1_CAN_11BIT_125KB	"B"
#define ELM_PROTOCOL_USER2_CAN_11BIT_50KB	"C"


//============================================================================
//				GLOBAL VARIABLES
//============================================================================

extern char* Elm_Cmds[ELM_CMD_SIZE];
extern unsigned char ErrorNoReadyCounter;
extern unsigned char ErrorNoDataCounter;
extern unsigned short ErrorNullCounterCpy,ErrorNullCounterCat,ErrorNullCounter1,ErrorNullCounter2,ErrorNullCounter3,ErrorNullCounter4,ErrorNullCounter5,ErrorNullCounter6,ErrorNullCounter7;
extern unsigned char ErrorCompareCounter;
extern unsigned short ErrorATRV;
extern unsigned short ErrorATI;

extern volatile unsigned char Elm_GlobalRequestCmd;
extern volatile unsigned char Elm_GlobalRequestMode;
extern volatile unsigned char Elm_GlobalRequestPID;

extern char UartTxBuffer[10];
extern char* pToCmd;
extern char* pToData;
extern char* pToResp;
extern volatile unsigned char  CmdLen, DataLen, RespLen;


//============================================================================
//				FUNCTION PROTOTYPES
//============================================================================

extern void Elm_Init(void);
extern void Elm_SendATCmd(unsigned char pCmd);
extern void Elm_SendATCmdPlusString(unsigned char pCmd, char* pString);
extern void Elm_SendOBDCmd(unsigned char pCmd, unsigned char pMode, unsigned pPID);
extern void Elm_SendOBDString(char* pString);
extern void Elm_AddHexDigit(unsigned char pDigit);
extern void Elm_AddHexNumber(unsigned char pNumber);
extern unsigned char Elm_ReadHexDigit(char* pPointer,unsigned char pIndex);
extern void Elm_SearchData(void);
extern void Elm_AnalyseATCmd(void);


