// KS0108.h

//============================================================================
//				DEFINES
//============================================================================

#define _IFDEF_FAST_LCD					// LCDWrite is a little bit faster, because the "GLCD_ReadStatus()->Busy-Loop" is ignored by setting this #define
#define GLCD_Delay(pTimes) 				do{ for(i=0; i<pTimes; i++) __asm__ __volatile__ ("nop"); } while(0)

#define KS0108_SCREEN_WIDTH				128
#define KS0108_SCREEN_HEIGHT			64
#define DISPLAY_SET_Y       			0x40
#define DISPLAY_SET_X       			0xB8
#define DISPLAY_START_LINE  			0xC0
#define DISPLAY_ON_CMD		 			0x3E
#define ON								0x01
#define OFF								0x00
#define DISPLAY_STATUS_BUSY				0x80

#define OPTIMIZED_IMAGES_MARKER			0xEE

#define SCAN_SCREEN_WIDTH_X_MAX			108
#define SCAN_SCREEN_HEIGHT_Y_MAX		6		

#define GLCD_SetInvert(pInvert) 		LCDInvert = pInvert
#define GLCD_SetFontBold(pBold) 		Font_Bold = pBold
#define GLCD_SetFontSOffset(pOffset)	FontS_Offset = pOffset

// Bold
#define NORMAL							0
#define BOLD							1

// Scroll
#define SCROLL_UP						1
#define SCROLL_DOWN						2
#define SCROLL_LEFT						3
#define SCROLL_RIGHT					4

// Number Format
#define GLCD_WriteNumberZero_UNUM8(pX,pY,pAddr,pDigits,pKomma,pFont,pDir)	GLCD_WriteNumberZero_Format(pX,pY,pAddr,_8BIT,pDigits,pKomma,pFont,pDir);
#define GLCD_WriteNumberZero_SNUM8(pX,pY,pAddr,pDigits,pKomma,pFont,pDir)	GLCD_WriteNumberZero_SFormat(pX,pY,pAddr,_8BIT,pDigits,pKomma,pFont,pDir);
#define GLCD_WriteNumberZero_UNUM16(pX,pY,pAddr,pDigits,pKomma,pFont,pDir)	GLCD_WriteNumberZero_Format(pX,pY,(unsigned char*)pAddr,_16BIT,pDigits,pKomma,pFont,pDir);
#define GLCD_WriteNumberZero_SNUM16(pX,pY,pAddr,pDigits,pKomma,pFont,pDir)	GLCD_WriteNumberZero_SFormat(pX,pY,(signed char*)pAddr,_16BIT,pDigits,pKomma,pFont,pDir);



//============================================================================
//				GLOBAL VARIABLES
//============================================================================

extern volatile unsigned char LCDInvert;
extern volatile unsigned char Font_Bold;
extern volatile unsigned char FontS_Offset;
extern          char StringBuffer[33],NumberBuffer[33];


//============================================================================
//				FUNCTIONS PROTOTYPES
//============================================================================

// User LCD Functions
extern void GLCD_Initalize(void);
extern void GLCD_FillScreen(unsigned char);
extern void GLCD_GoTo(unsigned char, unsigned char);
extern void GLCD_SetPixel(unsigned char, unsigned char, unsigned char);
extern void GLCD_WriteData(unsigned char dataToWrite);

// Small 5x4 Font Functions
extern void GLCD_WriteCharS(unsigned char pCharToWrite);
extern void GLCD_WriteStringS(char* pStringToWrite);
extern void GLCD_WriteStringS_P(const char* pStringToWrite);
extern void GLCD_WriteNumberS(signed short pNumber);
extern void GLCD_WriteNumberS_Long(signed long pNumber);

// Medium 7 Font Functions
extern void GLCD_WriteCharM(unsigned char pCharToWrite);
extern void GLCD_WriteStringM(char* pStringToWrite);
extern void GLCD_WriteStringM_P(const char* pStringToWrite);
extern void GLCD_WriteNumberM(signed short pNumber);

// Font "Zero Twos" for Mitsubishi
extern void GLCD_WriteDigitZero(unsigned char pDigit,unsigned char pFont);
extern void GLCD_WriteNumberZero(unsigned short pNumber,unsigned char pKomma,unsigned char pFont);
extern void GLCD_WriteNumberZero_Format(unsigned char pXCursor,unsigned char pYCursor,unsigned char* pNumberAddr,unsigned char pBit,unsigned char pDigits,unsigned char pKomma,unsigned char pFont,unsigned char pDirection);
extern void GLCD_WriteNumberZero_SFormat(unsigned char pXCursor,unsigned char pYCursor,signed char* pNumberAddr,unsigned char pBit,unsigned char pDigits,unsigned char pKomma,unsigned char pFont,unsigned char pDirection);

// LCD Images Functions
extern void GLCD_DrawImage(const unsigned char*, unsigned char, unsigned char, unsigned char, unsigned char);
extern void GLCD_DrawAni(unsigned short*, unsigned char, unsigned char, unsigned char, unsigned char, unsigned char, unsigned short);
extern void GLCD_DrawSimpleLine(unsigned char,unsigned char,unsigned char,unsigned char, unsigned char);
extern void GLCD_DrawLine(unsigned char x1,unsigned char y1,unsigned char x2,unsigned char y2,unsigned char color,unsigned char pBold);

// Scroll + Scan Functions
#ifdef CFG_SCROLL_SCREEN_ENABLE
	extern void GLCD_ScrollScreen(unsigned char pDirection,unsigned char pCursorX,unsigned char pPageY,unsigned char pWidthX,unsigned char pHeightY,unsigned char pLines,unsigned short pWait);
	extern void GLCD_ScanScreen(unsigned char pCursorX,unsigned char pPageY,unsigned char pWidthX,unsigned char pHeightY);
#endif


