
//============================================================================
//				HEADERS
//============================================================================

#include <avr/pgmspace.h>
#include "Led.h"


//============================================================================
//				DEFINITIONS
//============================================================================


//============================================================================
//				GLOBAL VARIABLES
//============================================================================

unsigned char LedTogglePeriod,LedToggleStyle;
volatile unsigned char Led_LCDBrightness = 1;
volatile unsigned char Led_Color;

const unsigned char PROGMEM Led_ColorMix[COLOR_NUMBERS * COLOR_BITS] = {
	16,16,16,16,	//  0 Red
	16, 0, 0, 0,	//  1 Red-Dark
	16,16,16,48,	//  2 Red-Orange
	16,16,16,32,	//  3 Orange
	16,16,48,32,	//  4 Orange-Yellow
	48,48,48,48,	//  5 Yellow-Full
	16,16,32,32,	//  6 Yellow
	48, 0, 0, 0,	//  7 Yellow-Dark
	48,48,48,112,	//  8 Yellow-Light
	16,48,32,32,	//  9 Yellow-Grellow
	16,32,32,32,	// 10 Grellow
	48,32,32,32,	// 11 Grellow-Green
	32,32,32,32,	// 12 Green
	32, 0, 0, 0,	// 13 Green-Dark
	32,32,32,112,	// 14 Green-Light
	32,32,32,96,	// 15 Green-Gryan
	32,32,32,64,	// 16 Gryan
	32,32,96,64,	// 17 Gryan-Cyan
	96,96,96,96,	// 18 Cyan-Full
	32,32,64,64,	// 19 Cyan
	96, 0, 0, 0,	// 20 Cyan-Dark
	96,96,96,112,	// 21 Cyan-Light
	32,96,64,64,	// 22 Cyan-Bluan
	32,64,64,64,	// 23 Bluan
	96,64,64,64,	// 24 Bluan-Blue
	64,64,64,64,	// 25 Blue
	64, 0, 0, 0,	// 26 Blue-Dark
	64,64,64,112,	// 27 Blue-Light
	64,64,64,80,	// 28 Blue-Blila
	64,64,64,16,	// 29 Blila
	64,64,80,16,	// 30 Blila-Lila
	80,80,80,80,	// 31 Lila-Full
	64,64,16,16,	// 32 Lila
	64,80,16,16,	// 33 Lila-Pink
	80, 0, 0, 0,	// 34 Lila-Dark
	80,80,80,112,	// 35 Lila-Light
	64,16,16,16,	// 36 Pink
	80,16,16,16,	// 37 Pink-Red
	16,16,16,112,	// 38 Red-Light
	112,112,112,112,// 39 White
	 0, 0, 0,112,	// 40 White dark
};


//============================================================================
//				FUNCTION PROTOTYPES
//============================================================================

void Led_RefreshToggle(void);
void Led_SetToggle(unsigned char pPeriod, unsigned char pStyle);
void Led_RefreshLCDBacklight(void);
void Led_RefreshRGBLedColor(void);


//============================================================================
//				FUNCTIONS
//============================================================================

void Led_RefreshRGBLedColor(void)
{
	static unsigned char ColorCounter=0;
	const unsigned char* ColorAddr;

	ColorCounter++;
	if(ColorCounter == COLOR_BITS) ColorCounter -= COLOR_BITS;

	ColorAddr = Led_ColorMix + Led_Color*COLOR_BITS + ColorCounter;

	LED_PORT = ( LED_PORT & ~0x70 ) | ( pgm_read_byte(ColorAddr) & 0x70 );
}


void Led_RefreshToggle(void)
{
	static unsigned char ToggleCounter;

	if(LedTogglePeriod)
	{
		ToggleCounter++;

		// Rectangle Style of Toggling
		if(LedToggleStyle == RECT)
		{
			if(ToggleCounter & (1<<(LedTogglePeriod-1)))
			{
				Led_SetRGBLedColor(OFF);
			}
			else
			{
				Led_SetRGBLedColor(Led_Color);
			}
		}
		// Peak Style of Toggling
		else if(LedToggleStyle == PEAK)
		{
			if(ToggleCounter % (1<<LedTogglePeriod))
			{
				Led_SetRGBLedColor(OFF);
			}
			else
			{
				Led_SetRGBLedColor(Led_Color);
			}
		}
	}
}

// pColor = R,G,Y; pPeriod = 0(32ms), 1(64ms), 2(128ms), 3(256ms), 4(512ms), 5(1024ms), 6(2048ms), 7(4096ms)
void Led_SetToggle(unsigned char pPeriod, unsigned char pStyle)
{
	LedTogglePeriod = pPeriod & 0x07;	// only lowest 3 bits
	LedToggleStyle  = pStyle;
}


// Refresh LCD-Backlight
void Led_RefreshLCDBacklight(void)
{
	static unsigned char LCDBrightnessCounter;

	LCDBrightnessCounter++;
	
	if(LCDBrightnessCounter >= LCD_MAX_BRIGHTNESS)
	{
		LCDBrightnessCounter = 0;
	}		
	
	if(LCDBrightnessCounter == 0 && Led_LCDBrightness)
	{
		LED_PORT |= LED_LCD;
	}		
	if(LCDBrightnessCounter == Led_LCDBrightness)
	{
		LED_PORT &= ~ LED_LCD;
	}		
}

