// Menu.c

//============================================================================
//				HEADERS
//============================================================================

#include <avr/io.h>
#include <stdlib.h>
#include <inttypes.h>
#include <avr/eeprom.h>
#include <avr/pgmspace.h>

#include "EvoBox.h"
#include "Led.h"
#include "Buzzer.h"
#include "Buttons.h"
#include "LCD_KS0108.h"
#include "Images.h"
#include "Sensors.h"
#include "Menu.h"
#include "OBD.h"
#include "Options.h"
#include "Eeprom.h"


//============================================================================
//				DEFINITIONS
//============================================================================


//============================================================================
//				GLOBAL VARIABLES
//============================================================================

const char PROGMEM MainMenuText[][20]={
	"Digital-Anzeige    ",
	"Analog-Anzeige     ",
	"Sensorwerte        ",
	"Oszilloskop        ",
	"Fahrstatistiken    ",
	"Sprint 0..100      ",
	"Diagnose           ",
	"Optionen           "
};


//============================================================================
//				FUNCTION PROTOTYPES
//============================================================================

void LCDStartTheme(void);
unsigned char MainMenu(void);
void Menu_WriteEeprom(unsigned char pMenuPoint);


//============================================================================
//				START THEME FUNCTIONS
//============================================================================

// LCD-Modul Start-Theme
void LCDStartTheme(void)
{
	#ifdef CFG_START_THEME_ENABLE
	unsigned char i;
	#endif

	GLCD_FillScreen(0x00);

	#ifdef CFG_START_THEME_ENABLE
	for(i=0; i<2; i++)
	{
		GLCD_DrawAni(Ani_StartTheme,12,30,0,72,8,100);
	}
	#endif
}


//============================================================================
//				MENU
//============================================================================

unsigned char MainMenu(void)
{
	#ifdef CFG_MAIN_MENU_ENABLE

	static signed char MainMenuPoint;
	unsigned char i;
	

	OBD_MenuUpdate(OBD_CYCLE_MAIN_MENU);

	// Clear LCD
	GLCD_SetInvert(0);
	GLCD_FillScreen(0x00);

	// Draw Headline
	GLCD_GoTo(0,0);
	for(i=0; i<128; i++) GLCD_WriteData(0x7F);

	GLCD_SetInvert(1);
	GLCD_SetFontSOffset(1);
	GLCD_GoTo(24,0);
	GLCD_WriteStringS("EVO-BOX HAUPTMENUE");
	GLCD_SetInvert(0);

	GLCD_DrawSimpleLine(0,7,127,7,0);


	// Draw Icons
	GLCD_DrawImage(MenuIcon_GaugesDigital	, 0,1,32,3);
	GLCD_DrawImage(MenuIcon_GaugesAnalog	,32,1,32,3);
	GLCD_DrawImage(MenuIcon_Sensors			,64,1,32,3);
	GLCD_DrawImage(MenuIcon_Graphs			,96,1,32,3);
	GLCD_DrawImage(MenuIcon_Statistics		, 0,4,32,3);
	GLCD_DrawImage(MenuIcon_Sprint			,32,4,32,3);
	GLCD_DrawImage(MenuIcon_Diagnose		,64,4,32,3);
	GLCD_DrawImage(MenuIcon_Options			,96,4,32,3);
	
	while(1)
	{
		// Draw new Icon
		GLCD_SetInvert(1);

		     if(MainMenuPoint == 0) GLCD_DrawImage(MenuIcon_GaugesDigital	, 0,1,32,3);
		else if(MainMenuPoint == 1) GLCD_DrawImage(MenuIcon_GaugesAnalog	,32,1,32,3);
		else if(MainMenuPoint == 2) GLCD_DrawImage(MenuIcon_Sensors			,64,1,32,3);
		else if(MainMenuPoint == 3) GLCD_DrawImage(MenuIcon_Graphs			,96,1,32,3);
		else if(MainMenuPoint == 4) GLCD_DrawImage(MenuIcon_Statistics		, 0,4,32,3);
		else if(MainMenuPoint == 5) GLCD_DrawImage(MenuIcon_Sprint			,32,4,32,3);
		else if(MainMenuPoint == 6) GLCD_DrawImage(MenuIcon_Diagnose		,64,4,32,3);
		else if(MainMenuPoint == 7) GLCD_DrawImage(MenuIcon_Options			,96,4,32,3);

		GLCD_SetInvert(0);


		// Write additional Info
		GLCD_GoTo(0,7);
		GLCD_SetFontBold(BOLD);
		GLCD_WriteStringM_P(MainMenuText[MainMenuPoint & 0x07]);

		WaitInMs(100);		

		do
		{
			ExecuteISR();
		}
		while( !(MENU_UP || MENU_DOWN || MENU_LEFT || MENU_RIGHT || MENU_ENTER) );

		// Erase old Icon
		GLCD_SetInvert(0);
		
		     if(MainMenuPoint == 0) GLCD_DrawImage(MenuIcon_GaugesDigital	, 0,1,32,3);
		else if(MainMenuPoint == 1) GLCD_DrawImage(MenuIcon_GaugesAnalog	,32,1,32,3);
		else if(MainMenuPoint == 2) GLCD_DrawImage(MenuIcon_Sensors			,64,1,32,3);
		else if(MainMenuPoint == 3) GLCD_DrawImage(MenuIcon_Graphs			,96,1,32,3);
		else if(MainMenuPoint == 4) GLCD_DrawImage(MenuIcon_Statistics		, 0,4,32,3);
		else if(MainMenuPoint == 5) GLCD_DrawImage(MenuIcon_Sprint			,32,4,32,3);
		else if(MainMenuPoint == 6) GLCD_DrawImage(MenuIcon_Diagnose		,64,4,32,3);
		else if(MainMenuPoint == 7) GLCD_DrawImage(MenuIcon_Options			,96,4,32,3);

		// Check Buttons
		if(MENU_UP)
		{
			MainMenuPoint -= 4;
			if(MainMenuPoint < 0) MainMenuPoint += 4;
		}
		else if(MENU_DOWN)
		{
			MainMenuPoint += 4;
			if(MainMenuPoint > 7) MainMenuPoint -= 4;
		}
		else if(MENU_LEFT)
		{
			MainMenuPoint--;
			if(MainMenuPoint < 0) MainMenuPoint = 7;
		}
		else if(MENU_RIGHT)
		{
			MainMenuPoint++;
			if(MainMenuPoint > 7) MainMenuPoint = 0;
		}
		else if(MENU_ENTER)
		{
			     if(MainMenuPoint == MAIN_MENU_GO_GAUGES_DIGITAL)	return GAUGES_DIGITAL;
			else if(MainMenuPoint == MAIN_MENU_GO_GAUGES_ANALOG)	return GAUGES_ANALOG;
			else if(MainMenuPoint == MAIN_MENU_GO_SENSORS)			return SENSORS;
			else if(MainMenuPoint == MAIN_MENU_GO_GRAPHS)			return GRAPHS;
			else if(MainMenuPoint == MAIN_MENU_GO_STATISTICS)		return STATISTICS;
			else if(MainMenuPoint == MAIN_MENU_GO_SPRINT)			return SPRINT;
			else if(MainMenuPoint == MAIN_MENU_GO_DIAGNOSE)			return DIAGNOSE;
			else if(MainMenuPoint == MAIN_MENU_GO_OPTIONS)			return OPTIONS;
		}
	}

	#endif

	return MAIN_MENU;
}


void Menu_WriteEeprom(unsigned char pMenuPoint)
{
	if((MenuPointStatus != 0) && (MenuPointStatus != OPTIONS_TIME))
	{
		Eeprom_WriteChar(EEP_ADR_MENU_POINT_STATUS,pMenuPoint);
	}
}

