// Menu.c

//============================================================================
//				HEADERS
//============================================================================

#include <avr/io.h>
#include <stdlib.h>
#include <string.h>
#include <inttypes.h>
#include <math.h>
#include <avr/eeprom.h>
#include <avr/pgmspace.h>

#include "EvoBox.h"
#include "Led.h"
#include "Buzzer.h"
#include "Buttons.h"
#include "LCD_KS0108.h"
#include "Fonts.h"
#include "Images.h"
#include "Menu.h"
#include "Mitsubishi.h"
#include "Sensors.h"
#include "Elm327.h"
#include "Stack.h"
#include "OBD.h"
#include "uart.h"
#include "Eeprom.h"
#include "Stack.h"


//============================================================================
//				DEFINITIONS
//============================================================================


//============================================================================
//				GLOBAL VARIABLES
//============================================================================

unsigned char GaugesDigitalVariant,GaugesAnalogVariant;

const float   SinusTable[45] = {
	0.000,0.035,0.070,0.105,0.139,0.174,0.208,0.242,0.276,0.309,
	0.342,0.375,0.407,0.438,0.469,0.500,0.530,0.559,0.588,0.616,
	0.643,0.669,0.695,0.719,0.743,0.766,0.788,0.809,0.829,0.848,
	0.866,0.883,0.899,0.914,0.927,0.940,0.951,0.961,0.970,0.978,
	0.985,0.990,0.995,0.998,0.999
};

const char PROGMEM GraphText[][18]={
	"BATTERIE [V]     ",
	"DREHZAHL [1/MIN] ",
	"GESCHWIND.[KM/H] ",
	"MOTORLAST [%]    ",
	"DROSSELKLAPPE [%]",
	"ZUENDWINKEL [`OT]",
	"MOTORTEMP.[`C]   ",
	"ANSAUGTEMP.[`C]  ",
	"ANSAUGDRUCK[MBAR]",
	"DURCHSATZ [G/S]  ",
	"O2-SENSOR 1 [%]  ",
	"O2-SENSOR 2 [%]  ",
	"FUELTRIM KURZ [%]",
	"FUELTRIM LANG [%]",
	"VERBRAUCH MOM.[L]",
	"VERBRAUCH AVG.[L]",
};


//============================================================================
//				FUNCTION PROTOTYPES
//============================================================================

unsigned char GaugesDigital(void);
unsigned char GaugesAnalog(void);
unsigned char Sensors(void);
unsigned char Graphs(void);
unsigned char Statistics(void);
unsigned char Sprint(void);
unsigned char Diagnose(void);


//============================================================================
//				FUNCTIONS
//============================================================================

// Sinus
float Sinus(signed short pAlpha)
{
	float SinusValue;

	if(pAlpha > 360) pAlpha -= 360;
	if(pAlpha <   0) pAlpha += 360;

	     if(pAlpha <  90) SinusValue = SinusTable[pAlpha/2];
	else if(pAlpha < 180) SinusValue = 1 - SinusTable[(pAlpha-90)/2];
	else if(pAlpha < 270) SinusValue = -1 * SinusTable[(pAlpha-180)/2];
	else                  SinusValue = SinusTable[(pAlpha-270)/2] - 1;

	return SinusValue;
}

// --- RACE MODES ------------------------------------------------------------
unsigned char GaugesDigital(void)
{
	#ifdef CFG_GAUGES_DIGITAL_ENABLE	

	OBD_MenuUpdate(OBD_CYCLE_GAUGES_DIGITAL);
	Menu_WriteEeprom(GAUGES_DIGITAL);

	// Make non-floating values visible
	Sensor_Time[MIN][1]--;
	Sensor_Date[DAY][1]--;
	Sensor_Coolant_Temp[1]--,Sensor_AirIntake_Temp[1]--;
	Sensor_MotorRpm[1]--,Sensor_Speed[1]--;
	Sensor_MotorLoad[1]--,SensorCalc_Verbrauch[1]--,SensorCalc_VerbrauchAvg[1]--;
	Sensor_NumberDTCs[1]--;

	// Clear LCD
	GLCD_DrawImage(GaugesDigital_Frame,0,0,128,8);

	while(1)
	{
		// INVERT ON
		GLCD_SetInvert(1);

		// Uhrzeit
		if(Sensor_Time[MIN][0] != Sensor_Time[MIN][1])
		{
			GLCD_GoTo(0,0);
			GLCD_SetFontSOffset(1);
			if(Sensor_Time[HOUR][0] < 10) GLCD_WriteCharS('0');
			GLCD_WriteNumberS(Sensor_Time[HOUR][0]);
			GLCD_WriteCharS(':');
			if(Sensor_Time[MIN][0] < 10) GLCD_WriteCharS('0');
			GLCD_WriteNumberS(Sensor_Time[MIN][0]);
			GLCD_DrawSimpleLine(0,7,19,7,1);

			Sensor_Time[MIN][1] = Sensor_Time[MIN][0];
		}

		// Datum
		if(Sensor_Date[DAY][0] != Sensor_Date[DAY][1])
		{
			GLCD_GoTo(87,0);
			GLCD_SetFontSOffset(1);
			if(Sensor_Date[DAY][0] < 10) GLCD_WriteCharS('0');
			GLCD_WriteNumberS(Sensor_Date[DAY][0]);
			GLCD_WriteCharS('.');
			if(Sensor_Date[MONTH][0] < 10) GLCD_WriteCharS('0');
			GLCD_WriteNumberS(Sensor_Date[MONTH][0]);
			GLCD_WriteStringS(".20");
			if(Sensor_Date[YEAR][0] < 10) GLCD_WriteCharS('0');
			GLCD_WriteNumberS(Sensor_Date[YEAR][0]);
			GLCD_DrawSimpleLine(87,7,126,7,1);

			Sensor_Date[DAY][1] = Sensor_Date[DAY][0];
		}

		// Fehlereintrge
		if(Sensor_NumberDTCs[0] != Sensor_NumberDTCs[1])
		{
			GLCD_GoTo(51,3);
			GLCD_SetFontSOffset(1);
			GLCD_WriteNumberS(Sensor_NumberDTCs[0]);
			GLCD_DrawSimpleLine(51,31,54,31,1);

			Sensor_NumberDTCs[1] = Sensor_NumberDTCs[0];
		}

		// Temperatur: Motor, Auen
		if(Sensor_Coolant_Temp[0] != Sensor_Coolant_Temp[1])
		{
			GLCD_SetFontSOffset(2);
			GLCD_GoTo(11,7);
			if(Sensor_Coolant_Temp[0] >= 0)
			{
				if(Sensor_Coolant_Temp[0] < 10) GLCD_WriteCharS(' ');
				GLCD_WriteCharS('+');
			}
			else
			{
				if(Sensor_Coolant_Temp[0] > -10) GLCD_WriteCharS(' ');
			}
			GLCD_WriteNumberS(Sensor_Coolant_Temp[0]);
			GLCD_WriteStringS("`C ");
			if(Sensor_Coolant_Temp[0] >= 0 && Sensor_Coolant_Temp[0] < 10) GLCD_WriteCharS(' ');
			GLCD_DrawSimpleLine(11,56,38,56,1);

			Sensor_Coolant_Temp[1] = Sensor_Coolant_Temp[0];
		}

		if(Sensor_AirIntake_Temp[0] != Sensor_AirIntake_Temp[1])
		{
			GLCD_SetFontSOffset(2);
			GLCD_GoTo(107,7);
			if(Sensor_AirIntake_Temp[0] >= 0)
			{
				if(Sensor_AirIntake_Temp[0] < 10) GLCD_WriteCharS(' ');
				GLCD_WriteCharS('+');
			}
			else
			{
				if(Sensor_AirIntake_Temp[0] > 10) GLCD_WriteCharS(' ');
			}
			GLCD_WriteNumberS(Sensor_AirIntake_Temp[0]);
			GLCD_WriteStringS("`C");
			if(Sensor_AirIntake_Temp[0] >= 0 && Sensor_AirIntake_Temp[0] < 10) GLCD_WriteCharS(' ');
			GLCD_DrawSimpleLine(107,56,126,56,1);

			Sensor_AirIntake_Temp[1] = Sensor_AirIntake_Temp[0];
		}

		// INVERT OFF
		GLCD_SetInvert(0);

		// Status
		if(Do_FlagByte & DO_OBD_MIL_ON)
		{
			GLCD_DrawImage(Gauges_Status_MIL,31,1,24,2);
		}
		else
		{
			GLCD_DrawImage(Gauges_Status_OK,31,1,24,2);
		}

		// Drehzahl, Geschwindigkeit
		GLCD_WriteNumberZero_UNUM16(68,1,Sensor_MotorRpm		,4,NO_KOMMA,FONT_ZERO_14,RIGHT_ALIGN);
		GLCD_WriteNumberZero_UNUM8(57,4,Sensor_Speed			,3,NO_KOMMA,FONT_ZERO_22,RIGHT_ALIGN);

		// Last, Verbrauch
		GLCD_WriteNumberZero_UNUM8(1,4,Sensor_MotorLoad			,3,NO_KOMMA,FONT_ZERO_7BOLD,RIGHT_ALIGN);
		GLCD_WriteNumberZero_UNUM16(1,5,SensorCalc_Verbrauch	,3,KOMMA,FONT_ZERO_7,RIGHT_ALIGN);
		GLCD_WriteNumberZero_UNUM16(1,6,SensorCalc_VerbrauchAvg	,3,KOMMA,FONT_ZERO_7,RIGHT_ALIGN);


		Wait200us();
		ExecuteISR();
		if(MENU_UP || MENU_DOWN) break;
	}

	#endif

	return MAIN_MENU;
}

unsigned char GaugesAnalog(void)
{
	#ifdef CFG_GAUGES_ANALOG_ENABLE

	float Alpha_MotorRpm=0,Alpha_Verbrauch=0,Alpha_Speed=0;

	OBD_MenuUpdate(OBD_CYCLE_GAUGES_ANALOG);
	Menu_WriteEeprom(GAUGES_ANALOG);

	// Make non-floating values visible
	Sensor_Time[MIN][1]--;
	Sensor_Date[DAY][1]--;
	Sensor_MotorRpm[1]-=16;
	Sensor_Speed[1]--;
	SensorCalc_Verbrauch[1]--;
	Sensor_AirIntake_Temp[1]--;

	// Clear LCD
	GLCD_DrawImage(GaugesAnalog_Frame,0,0,128,8);


	while(1)
	{
		// Uhrzeit
		if(Sensor_Time[MIN][0] != Sensor_Time[MIN][1])
		{
			GLCD_SetInvert(1);
			GLCD_GoTo(0,0);
			GLCD_SetFontSOffset(1);
			if(Sensor_Time[HOUR][0] < 10) GLCD_WriteCharS('0');
			GLCD_WriteNumberS(Sensor_Time[HOUR][0]);
			GLCD_WriteCharS(':');
			if(Sensor_Time[MIN][0] < 10) GLCD_WriteCharS('0');
			GLCD_WriteNumberS(Sensor_Time[MIN][0]);
			GLCD_DrawSimpleLine(0,7,19,7,1);

			Sensor_Time[MIN][1] = Sensor_Time[MIN][0];
		}

		// Datum
		if(Sensor_Date[DAY][0] != Sensor_Date[DAY][1])
		{
			GLCD_SetInvert(1);
			GLCD_GoTo(87,0);
			GLCD_SetFontSOffset(1);
			if(Sensor_Date[DAY][0] < 10) GLCD_WriteCharS('0');
			GLCD_WriteNumberS(Sensor_Date[DAY][0]);
			GLCD_WriteCharS('.');
			if(Sensor_Date[MONTH][0] < 10) GLCD_WriteCharS('0');
			GLCD_WriteNumberS(Sensor_Date[MONTH][0]);
			GLCD_WriteStringS(".20");
			if(Sensor_Date[YEAR][0] < 10) GLCD_WriteCharS('0');
			GLCD_WriteNumberS(Sensor_Date[YEAR][0]);
			GLCD_DrawSimpleLine(89,7,126,7,1);

			Sensor_Date[DAY][1] = Sensor_Date[DAY][0];
		}

		// Temperatur
		GLCD_SetInvert(1);
		GLCD_WriteNumberZero_SNUM8(46,7,Sensor_AirIntake_Temp,3,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);
		if(Sensor_AirIntake_Temp[0] > 0)
		{
			GLCD_GoTo(47,7);
			GLCD_SetFontBold(0);
			if(Sensor_AirIntake_Temp[0] > 10)
			{
				GLCD_WriteCharM('+');
			}
			else
			{
				GLCD_WriteStringM(" +");
			}
		}

		// Status
		GLCD_SetInvert(1);
		if(Do_FlagByte & DO_OBD_MIL_ON)
		{
			GLCD_DrawImage(Gauges_Status_MIL,51,5,24,2);
		}
		else
		{
			GLCD_DrawImage(Gauges_Status_OK,51,5,24,2);
		}

		// Drehzahl
		if(fabs(Sensor_MotorRpm[0] - Sensor_MotorRpm[1]) >= 15)
		{
			GLCD_SetInvert(1);
			GLCD_GoTo(0,7);
			GLCD_SetFontSOffset(2);
			if(Sensor_MotorRpm[0] < 1000) GLCD_WriteCharS('0');
			if(Sensor_MotorRpm[0] < 100) GLCD_WriteCharS('0');
			if(Sensor_MotorRpm[0] < 10) GLCD_WriteCharS('0');
			GLCD_WriteNumberS(Sensor_MotorRpm[0]);

			GLCD_SetInvert(0);
			GLCD_DrawLine(16,36,16+11*cos(Alpha_MotorRpm),36+15*sin(Alpha_MotorRpm),0,NORMAL);
			Alpha_MotorRpm = AlphaInGrad(107) + Sensor_MotorRpm[0]/8000.0F * AlphaInGrad(180);
			GLCD_DrawLine(16,36,16+11*cos(Alpha_MotorRpm),36+15*sin(Alpha_MotorRpm),1,NORMAL);

			Sensor_MotorRpm[1] = Sensor_MotorRpm[0];
		}

		// Geschwindigkeit
		if(fabs(Sensor_Speed[0] - Sensor_Speed[1]) >= 1)
		{
			GLCD_SetInvert(0);
			GLCD_DrawLine(63+23*cos(Alpha_Speed),48+24*sin(Alpha_Speed),63+31*cos(Alpha_Speed),48+35*sin(Alpha_Speed),0,BOLD);
			Alpha_Speed = AlphaInGrad(165) + Sensor_Speed[0]/270.0F * AlphaInGrad(205);
			GLCD_DrawLine(63+23*cos(Alpha_Speed),48+24*sin(Alpha_Speed),63+31*cos(Alpha_Speed),48+35*sin(Alpha_Speed),1,BOLD);
			
			GLCD_SetInvert(1);
			GLCD_WriteNumberZero_UNUM8(50,4,Sensor_Speed,3,NO_KOMMA,FONT_ZERO_7BOLD,RIGHT_ALIGN);
		}


		// Verbrauch
		if(SensorCalc_Verbrauch[0] != SensorCalc_Verbrauch[1])
		{
			GLCD_SetInvert(1);
			GLCD_GoTo(111,7);
			GLCD_SetFontSOffset(2);
			if(SensorCalc_Verbrauch[0] < 100) GLCD_WriteCharS(' ');
			GLCD_WriteNumberS(SensorCalc_Verbrauch[0]/10);
			GLCD_WriteCharS('.');
			GLCD_WriteNumberS(SensorCalc_Verbrauch[0]%10);

			if(SensorCalc_Verbrauch[0] < 400)
			{
				GLCD_SetInvert(0);
				GLCD_DrawLine(111,36,111+12*cos(Alpha_Verbrauch),36+15*sin(Alpha_Verbrauch),0,NORMAL);
				Alpha_Verbrauch = AlphaInGrad(70) - SensorCalc_Verbrauch[0]/400.0F * AlphaInGrad(180);
				GLCD_DrawLine(111,36,111+12*cos(Alpha_Verbrauch),36+15*sin(Alpha_Verbrauch),1,NORMAL);
			}

			SensorCalc_Verbrauch[1] = SensorCalc_Verbrauch[0];
		}

		Wait200us();
		ExecuteISR();
		if(MENU_UP || MENU_DOWN) break;
	}

	#endif

	return MAIN_MENU;
}


// --- SENSOREN --------------------------------------------------------------
unsigned char Sensors(void)
{
	unsigned short ResetTime;

	#ifdef CFG_SENSORS_ENABLE
	Menu_WriteEeprom(SENSORS);


// --- Sensoren Motor ------------------------------------------------------------
LABEL_MOTOR:

	OBD_MenuUpdate(OBD_CYCLE_SENSORS_MOTOR);

	// Make non-floating values visible
	Sensor_MotorRpm[1]--,SensorCalc_MotorRpmMax[1]--;
	Sensor_MotorTorque[1]--,SensorCalc_MotorTorqueMax[1]--;
	Sensor_MotorPower[1]--,SensorCalc_MotorPowerMax[1]--;
	Sensor_MotorLoad[1]--,SensorCalc_MotorLoadMax[1]--;
	Sensor_Throttle[1]--;
	Sensor_AdvanceAngle[1]--;
	Sensor_Coolant_Temp[1]--,SensorCalc_Coolant_TempMin[1]--,SensorCalc_Coolant_TempMax[1]--;


	// Clear LCD
	GLCD_DrawImage(Sensors_Motor_Frame,0,0,128,8);

	// Reset time
	GLCD_SetInvert(1);
	GLCD_SetFontSOffset(1);
	ResetTime = Sensor_MotorResetTime/60;
	GLCD_GoTo(90,0);
	if(ResetTime < 10) GLCD_WriteCharS(' ');
	GLCD_WriteNumberS(ResetTime);

	ResetTime = Sensor_MotorResetTime%60;
	GLCD_GoTo(102,0);
	if(ResetTime < 10) GLCD_WriteCharS(' ');
	GLCD_WriteNumberS(ResetTime);
	GLCD_DrawSimpleLine(90,7,109,7,1);
	GLCD_SetInvert(0);

	while(1)
	{
		// Drehzahl
		GLCD_WriteNumberZero_UNUM16(42,1,Sensor_MotorRpm       		,4,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);
		GLCD_WriteNumberZero_UNUM16(83,1,SensorCalc_MotorRpmMax		,4,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);

		// Drehmoment
		GLCD_WriteNumberZero_UNUM16(58,2,Sensor_MotorTorque       	,3,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);
		GLCD_WriteNumberZero_UNUM16(91,2,SensorCalc_MotorTorqueMax	,3,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);

		// Leistung
		GLCD_WriteNumberZero_UNUM16(58,3,Sensor_MotorPower       	,3,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);
		GLCD_WriteNumberZero_UNUM16(91,3,SensorCalc_MotorPowerMax	,3,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);

		// Motorlast
		GLCD_WriteNumberZero_UNUM8(58,4,Sensor_MotorLoad       		,3,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);
		GLCD_WriteNumberZero_UNUM8(91,4,SensorCalc_MotorLoadMax		,3,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);

		// Drosselklappe
		GLCD_WriteNumberZero_UNUM8(91,5,Sensor_Throttle	      		,3,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);

		// Zndwinkel
		GLCD_WriteNumberZero_SNUM8(99,6,Sensor_AdvanceAngle      	,2,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);

		// Khler-Temperatur
		GLCD_WriteNumberZero_SNUM8(25,7,SensorCalc_Coolant_TempMin	,3,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);
		GLCD_WriteNumberZero_SNUM8(58,7,Sensor_Coolant_Temp       	,3,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);
		GLCD_WriteNumberZero_SNUM8(91,7,SensorCalc_Coolant_TempMax	,3,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);


		Wait200us();
		ExecuteISR();
		if(MENU_LEFT) goto LABEL_BATTERY;
		if(MENU_RIGHT) goto LABEL_LUFT;
		if(MENU_DOWN || MENU_UP) return MAIN_MENU;
	}


// --- Sensoren Luft ------------------------------------------------------------
LABEL_LUFT:

	OBD_MenuUpdate(OBD_CYCLE_SENSORS_LUFT);

	// Make non-floating values visible
	Sensor_AirIntake_Temp[1]--,SensorCalc_AirIntake_TempMin[1]--,SensorCalc_AirIntake_TempMax[1]--;
	Sensor_AirIntake_Press[1]--;
	Sensor_MassAirFlow[1]--;
	Sensor_O2Sensor1[1]--,SensorCalc_O2Sensor1Min[1]--,SensorCalc_O2Sensor1Min[1]--;
	Sensor_O2Sensor2[1]--,SensorCalc_O2Sensor2Min[1]--,SensorCalc_O2Sensor2Min[1]--;


	// Clear LCD
	GLCD_DrawImage(Sensors_Luft_Frame,0,0,128,8);

	// Reset time
	GLCD_SetInvert(1);
	GLCD_SetFontSOffset(1);
	ResetTime = Sensor_LuftResetTime/60;
	GLCD_GoTo(90,0);
	if(ResetTime < 10) GLCD_WriteCharS(' ');
	GLCD_WriteNumberS(ResetTime);

	ResetTime = Sensor_LuftResetTime%60;
	GLCD_GoTo(102,0);
	if(ResetTime < 10) GLCD_WriteCharS(' ');
	GLCD_WriteNumberS(ResetTime);
	GLCD_DrawSimpleLine(90,7,109,7,1);
	GLCD_SetInvert(0);

	while(1)
	{
		// Ansaugluft-Temperatur
		GLCD_WriteNumberZero_SNUM8(86,1,Sensor_AirIntake_Temp       ,3,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);
		GLCD_WriteNumberZero_SNUM8(53,2,SensorCalc_AirIntake_TempMin,3,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);
		GLCD_WriteNumberZero_SNUM8(86,2,SensorCalc_AirIntake_TempMax,3,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);

		// Ansaugluft-Druck
		GLCD_WriteNumberZero_UNUM16(78,3,Sensor_AirIntake_Press    	,4,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);

		// Luftdurchsatz
		GLCD_WriteNumberZero_UNUM16(78,4,Sensor_MassAirFlow	    	,4,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);

		// Lambda
		GLCD_WriteNumberZero_SNUM8(27,6,SensorCalc_O2Sensor1Min		,3,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);
		GLCD_WriteNumberZero_SNUM8(60,6,Sensor_O2Sensor1			,3,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);
		GLCD_WriteNumberZero_SNUM8(93,6,SensorCalc_O2Sensor1Min		,3,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);
		GLCD_WriteNumberZero_SNUM8(27,7,SensorCalc_O2Sensor2Min		,3,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);
		GLCD_WriteNumberZero_SNUM8(60,7,Sensor_O2Sensor2		   	,3,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);
		GLCD_WriteNumberZero_SNUM8(93,7,SensorCalc_O2Sensor2Min		,3,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);


		Wait200us();
		ExecuteISR();
		if(MENU_LEFT) goto LABEL_MOTOR;
		if(MENU_RIGHT) goto LABEL_BATTERY;
		if(MENU_DOWN || MENU_UP) return MAIN_MENU;
	}


// --- Sensoren Dynamik ------------------------------------------------------------
LABEL_BATTERY:

	OBD_MenuUpdate(OBD_CYCLE_SENSORS_BATTERY);

	// Make non-floating values visible
	SensorCalc_VoltageMax[1]--,Sensor_Voltage[1]--,SensorCalc_VoltageMin[1]--;
	SensorCalc_Voltage0RpmMax[1]--,Sensor_Voltage0Rpm[1]--,SensorCalc_Voltage0RpmMin[1]--;
	SensorCalc_MotorRpmMax[1]--;

	// Clear LCD
	GLCD_DrawImage(Sensors_Battery_Frame,0,0,128,8);

	// Reset time
	GLCD_SetInvert(1);
	GLCD_SetFontSOffset(1);
	ResetTime = Sensor_VoltageResetTime/60;
	GLCD_GoTo(90,0);
	if(ResetTime < 10) GLCD_WriteCharS(' ');
	GLCD_WriteNumberS(ResetTime);

	ResetTime = Sensor_VoltageResetTime%60;
	GLCD_GoTo(102,0);
	if(ResetTime < 10) GLCD_WriteCharS(' ');
	GLCD_WriteNumberS(ResetTime);
	GLCD_DrawSimpleLine(90,7,109,7,1);
	GLCD_SetInvert(0);

	while(1)
	{
		// Batterie-Spannung @ 0 Rpm: min, current, max
		GLCD_GoTo(21,1);
		GLCD_SetFontSOffset(1);
		if(SensorCalc_Voltage0RpmMax[0] < 100) GLCD_WriteCharS(' ');
		GLCD_WriteNumberS(SensorCalc_Voltage0RpmMax[0]/10);
		GLCD_GoTo(31,1);
		GLCD_WriteNumberS(SensorCalc_Voltage0RpmMax[0]%10);

		GLCD_GoTo(21,2);
		GLCD_SetInvert(1);
		GLCD_SetFontSOffset(2);
		if(Sensor_Voltage0Rpm[0] < 100) GLCD_WriteCharS(' ');
		GLCD_WriteNumberS(Sensor_Voltage0Rpm[0]/10);
		GLCD_GoTo(31,2);
		GLCD_WriteNumberS(Sensor_Voltage0Rpm[0]%10);

		GLCD_GoTo(21,3);
		GLCD_SetInvert(0);
		GLCD_SetFontSOffset(2);
		if(SensorCalc_Voltage0RpmMin[0] < 100) GLCD_WriteCharS(' ');
		GLCD_WriteNumberS(SensorCalc_Voltage0RpmMin[0]/10);
		GLCD_GoTo(31,3);
		GLCD_WriteNumberS(SensorCalc_Voltage0RpmMin[0]%10);
	

		// Batterie-Spannung @ X Rpm: min, current, max
		GLCD_GoTo(85,1);
		GLCD_SetFontSOffset(1);
		if(SensorCalc_VoltageMax[0] < 100) GLCD_WriteCharS(' ');
		GLCD_WriteNumberS(SensorCalc_VoltageMax[0]/10);
		GLCD_GoTo(95,1);
		GLCD_WriteNumberS(SensorCalc_VoltageMax[0]%10);

		GLCD_GoTo(85,2);
		GLCD_SetInvert(1);
		GLCD_SetFontSOffset(2);
		if(Sensor_Voltage[0] < 100) GLCD_WriteCharS(' ');
		GLCD_WriteNumberS(Sensor_Voltage[0]/10);
		GLCD_GoTo(95,2);
		GLCD_WriteNumberS(Sensor_Voltage[0]%10);

		GLCD_GoTo(85,3);
		GLCD_SetInvert(0);
		GLCD_SetFontSOffset(2);
		if(SensorCalc_VoltageMin[0] < 100) GLCD_WriteCharS(' ');
		GLCD_WriteNumberS(SensorCalc_VoltageMin[0]/10);
		GLCD_GoTo(95,3);
		GLCD_WriteNumberS(SensorCalc_VoltageMin[0]%10);

		// Drehzahl
		GLCD_WriteNumberZero_UNUM16(76,7,SensorCalc_MotorRpmMax		,4,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);


		Wait200us();
		ExecuteISR();
		if(MENU_LEFT) goto LABEL_LUFT;
		if(MENU_RIGHT) goto LABEL_MOTOR;
		if(MENU_DOWN || MENU_UP) return MAIN_MENU;
	}

	#endif

	return MAIN_MENU;
}


// --- GRAPHS --------------------------------------------------------------
unsigned char Graphs(void)
{
	#ifdef CFG_GRAPHS_ENABLE

	unsigned char i;
	unsigned char  GraphSpeed=0,GraphSpeedCounter=0;
	  signed char  GraphIndex=0;
	  signed short GraphValue[2]={0,0},GraphMin,GraphMax,GraphValueRaw=0;
	  signed short YAxisValue,YAxisMin,YAxisMax;
	unsigned char  XCounter;

	  signed short YAxisMinMax[32]={
	// Min		Max
		8,		16,		// ELM_READ_VOLTAGE
		0,		8000,	// PID_0C_ENGINE_RPM
		0,		270,	// PID_0D_VEHICLE_SPEED
		0,		100,	// PID_04_CALC_ENGINE_LOAD
		0,		100,	// PID_11_THROTTLE_POS
		-64,	64,		// PID_0E_TIMING_ADVANCE
		-20,	120,	// PID_05_ENGINE_COOLANT_TEMP
		-20,	80,		// PID_0F_INTAKE_TEMP
		0,		1500,	// PID_0B_INTAKE_PRESS
		0,		5000,	// PID_10_MASS_AIR_FLOW
		-100,	100,	// PID_14_O2_BANK1_SENSOR1
		-100,	100,	// PID_15_O2_BANK1_SENSOR2
		-100,	100,	// PID_06_FUEL_SHORT_TRIM_BANK1
		-100,	100,	// PID_07_FUEL_LONG_TRIM_BANK1
		0,		60,		// Verbrauch
		0,		30,		// VerbrauchAvg
	};

	unsigned char Graphs_Select[32]={
		MODE_F_SHOW_ELM_AT_DATA,	ELM_READ_VOLTAGE,			
		MODE_1_SHOW_CURRENT_DATA,	PID_0C_ENGINE_RPM,
		MODE_1_SHOW_CURRENT_DATA,	PID_0D_VEHICLE_SPEED,
		MODE_1_SHOW_CURRENT_DATA,	PID_04_CALC_ENGINE_LOAD,
		MODE_1_SHOW_CURRENT_DATA,	PID_11_THROTTLE_POS,
		MODE_1_SHOW_CURRENT_DATA,	PID_0E_TIMING_ADVANCE,
		MODE_1_SHOW_CURRENT_DATA,	PID_05_ENGINE_COOLANT_TEMP,
		MODE_1_SHOW_CURRENT_DATA,	PID_0F_INTAKE_TEMP,
		MODE_1_SHOW_CURRENT_DATA,	PID_0B_INTAKE_PRESS,
		MODE_1_SHOW_CURRENT_DATA,	PID_10_MASS_AIR_FLOW,
		MODE_1_SHOW_CURRENT_DATA,	PID_14_O2_BANK1_SENSOR1,
		MODE_1_SHOW_CURRENT_DATA,	PID_15_O2_BANK1_SENSOR2,
		MODE_1_SHOW_CURRENT_DATA,	PID_06_FUEL_SHORT_TRIM_BANK1,
		MODE_1_SHOW_CURRENT_DATA,	PID_07_FUEL_LONG_TRIM_BANK1,
		MODE_1_SHOW_CURRENT_DATA,	PID_10_MASS_AIR_FLOW,
		MODE_1_SHOW_CURRENT_DATA,	PID_10_MASS_AIR_FLOW,
	};

	OBD_MenuUpdate(OBD_CYCLE_GRAPHS);
	Menu_WriteEeprom(GRAPHS);

	while(1)
	{
		// Clear LCD
		GLCD_DrawImage(Graphs_Frame,0,0,128,8);

		// Set right OBD-Cycle
		OBD_CycleCheck[OBD_CYCLE_GRAPHS][2] = Graphs_Select[2*GraphIndex];
		OBD_CycleCheck[OBD_CYCLE_GRAPHS][3] = Graphs_Select[2*GraphIndex + 1];

		if(GraphIndex == 14 || GraphIndex == 15)
		{
			OBD_CycleCheck[OBD_CYCLE_GRAPHS][0] = 2;
		}
		else
		{
			OBD_CycleCheck[OBD_CYCLE_GRAPHS][0] = 1;
		}

		// Headline
		GLCD_SetInvert(1);
		GLCD_SetFontSOffset(1);
		GLCD_GoTo(40,0);
		GLCD_WriteStringS_P(GraphText[GraphIndex]);
		GLCD_DrawSimpleLine(40,7,120,7,1);

		// Axis
		YAxisMin = YAxisMinMax[2*GraphIndex];
		YAxisMax = YAxisMinMax[2*GraphIndex + 1];

		for(i=0; i<7; i++)
		{
			YAxisValue = YAxisMin + (signed long)((YAxisMax - YAxisMin)*(signed long)(i))/6 + 0.5;
			GLCD_SetInvert(0);
			GLCD_SetFontSOffset(0);
			GLCD_GoTo(0,7-i);
			if(YAxisValue >= 0)
			{
				if(YAxisValue < 1000) GLCD_WriteCharS(' ');
				if(YAxisValue < 100) GLCD_WriteCharS(' ');
				if(YAxisValue < 10) GLCD_WriteCharS(' ');
			}
			else
			{
				if(YAxisValue > -100) GLCD_WriteCharS(' ');
				if(YAxisValue > -10) GLCD_WriteCharS(' ');
			}
			GLCD_WriteNumberS(YAxisValue);
		}

		if(GraphIndex == 0 || GraphIndex == 14 || GraphIndex == 15)
		{
			YAxisMin *= 10;
			YAxisMax *= 10;
		}

		// Max values
		GraphMin = YAxisMax;
		GraphMax = YAxisMin;
	

		WaitInMs(200);

		XCounter = 20;

		// Print SpeedDots
		for(i=0; i<GRAPH_SPEED_MAX; i++)
		{
			if(i <= (GRAPH_SPEED_MAX-1-GraphSpeed))
			{
				GLCD_SetPixel(i*3,62,1);
				GLCD_SetPixel(i*3,63,1);
				GLCD_SetPixel(i*3+1,62,1);
				GLCD_SetPixel(i*3+1,63,1);
			}
			else
			{
				GLCD_SetPixel(i*3,62,0);
				GLCD_SetPixel(i*3,63,0);
				GLCD_SetPixel(i*3+1,62,0);
				GLCD_SetPixel(i*3+1,63,0);
			}
		}

		// Print graphs
		while(1)
		{
			if(Do_FlagByte & DO_OBD_NEW_CYCLIC_CHECK)
			{
				Do_FlagByte &= ~DO_OBD_NEW_CYCLIC_CHECK;

				// Select right value
				     if(GraphIndex ==  0) GraphValueRaw += (signed short)(Sensor_Voltage[0]);
				else if(GraphIndex ==  1) GraphValueRaw += (signed short)(Sensor_MotorRpm[0]);
				else if(GraphIndex ==  2) GraphValueRaw += (signed short)(Sensor_Speed[0]);
				else if(GraphIndex ==  3) GraphValueRaw += (signed short)(Sensor_MotorLoad[0]);
				else if(GraphIndex ==  4) GraphValueRaw += (signed short)(Sensor_Throttle[0]);
				else if(GraphIndex ==  5) GraphValueRaw += (signed short)(Sensor_AdvanceAngle[0]);
				else if(GraphIndex ==  6) GraphValueRaw += (signed short)(Sensor_Coolant_Temp[0]);
				else if(GraphIndex ==  7) GraphValueRaw += (signed short)(Sensor_AirIntake_Temp[0]);
				else if(GraphIndex ==  8) GraphValueRaw += (signed short)(Sensor_AirIntake_Press[0]);
				else if(GraphIndex ==  9) GraphValueRaw += (signed short)(Sensor_MassAirFlow[0]);
				else if(GraphIndex == 10) GraphValueRaw += (signed short)(Sensor_O2Sensor1[0]);
				else if(GraphIndex == 11) GraphValueRaw += (signed short)(Sensor_O2Sensor2[0]);
				else if(GraphIndex == 12) GraphValueRaw += (signed short)(Sensor_FuelShortTrim[0]);
				else if(GraphIndex == 13) GraphValueRaw += (signed short)(Sensor_FuelLongTrim[0]);
				else if(GraphIndex == 14) GraphValueRaw += (signed short)(SensorCalc_Verbrauch[0]);
				else if(GraphIndex == 15) GraphValueRaw += (signed short)(SensorCalc_VerbrauchAvg[0]);
				

				if(GraphSpeedCounter >= GraphSpeed)
				{
					// Mean value
					GraphValue[0] = GraphValueRaw/(float)(GraphSpeed+1);


					if(XCounter < 128)
					{
						// Plot Pixel
						if((GraphValue[0] >= YAxisMin) && (GraphValue[0] <= YAxisMax))
						{
							if(XCounter > 22) GLCD_DrawLine(XCounter, 55 - (signed long)((GraphValue[1] - YAxisMin)*47L)/(YAxisMax - YAxisMin), XCounter+1, 55 - (signed long)((GraphValue[0] - YAxisMin)*47L)/(YAxisMax - YAxisMin), 1, 0);
						
						}
					}
					else
					{
						// Scroll screen
						#ifdef CFG_SCROLL_SCREEN_ENABLE
						GLCD_ScrollScreen(SCROLL_LEFT,20,1,108,6,1,0);
						#endif

						// Plot Pixel
						if((GraphValue[0] >= YAxisMin) && (GraphValue[0] <= YAxisMax))
						{
							if(XCounter > 22) GLCD_DrawLine(126, 55 - (GraphValue[1] - YAxisMin)*47L/(YAxisMax - YAxisMin), 127, 55 - (GraphValue[0] - YAxisMin)*47L/(YAxisMax - YAxisMin), 1, 0);
						}
					}

					// Current value
					GLCD_SetFontSOffset(3);
					GLCD_GoTo(22,7);
					GLCD_WriteNumberS(GraphValue[0]);
					GLCD_WriteCharS(' ');
					GLCD_DrawSimpleLine(21,56,41,56,1);
			
					// Min value
					if(GraphValue[0] < GraphMin)
					{
						GraphMin = GraphValue[0];
						GLCD_GoTo(65,7);
						GLCD_WriteNumberS(GraphMin);
						GLCD_WriteCharS(' ');
						GLCD_DrawSimpleLine(64,56,84,56,1);
					}
					// Max value
					if(GraphValue[0] > GraphMax) 
					{
						GraphMax = GraphValue[0];
						GLCD_GoTo(108,7);
						GLCD_WriteNumberS(GraphMax);
						GLCD_WriteCharS(' ');
						GLCD_DrawSimpleLine(107,56,125,56,1);
					}

					if(XCounter < 128) XCounter++;

					GraphValue[1] = GraphValue[0];

					GraphValueRaw = 0;
					GraphSpeedCounter = 0;
				}
				else
				{
					GraphSpeedCounter++;
				}
			}
			
	
			Wait200us();
			ExecuteISR();
			if(MENU_LEFT)
			{
				GraphIndex--;
				if(GraphIndex == -1) GraphIndex = 15;
				break;
			}
			if(MENU_RIGHT)
			{
				GraphIndex++;
				if(GraphIndex == 16) GraphIndex = 0;
				break;
			}

			if(MENU_DOWN)
			{
				GraphSpeed++;
				if(GraphSpeed == GRAPH_SPEED_MAX) GraphSpeed = 0;

				// Start new mean values
				GraphValueRaw = 0;
				GraphSpeedCounter = 0;
			}

			// Print SpeedDots
			if(MENU_LEFT || MENU_RIGHT || MENU_DOWN)
			{
				for(i=0; i<GRAPH_SPEED_MAX; i++)
				{
					if(i <= (GRAPH_SPEED_MAX-1-GraphSpeed))
					{
						GLCD_SetPixel(i*3,62,1);
						GLCD_SetPixel(i*3,63,1);
						GLCD_SetPixel(i*3+1,62,1);
						GLCD_SetPixel(i*3+1,63,1);
					}
					else
					{
						GLCD_SetPixel(i*3,62,0);
						GLCD_SetPixel(i*3,63,0);
						GLCD_SetPixel(i*3+1,62,0);
						GLCD_SetPixel(i*3+1,63,0);
					}
				}
			}

			if(MENU_UP) return MAIN_MENU;
		}

	}

	#endif
	
	return MAIN_MENU;
}


// --- FAHR-STATISTIK --------------------------------------------------------------
unsigned char Statistics(void)
{
	#ifdef CFG_STATISTICS_ENABLE

	signed char Fahrstat_Counter = Fahrstat_Index;
	unsigned char SearchEntryCounter;

	OBD_MenuUpdate(OBD_CYCLE_STATISTICS);
	Menu_WriteEeprom(STATISTICS);

	// Read Data from EEPROM
	Sensor_Statistic_ReadEeprom();

	while(1)
	{
		// Clear LCD
		GLCD_DrawImage(Fahrstatistik_Frame,0,0,128,8);

		GLCD_GoTo(70,0);
		GLCD_SetFontSOffset(1);
		GLCD_SetInvert(1);
		if(Fahrstat_Counter == Fahrstat_Index)
		{
			GLCD_WriteStringS("AKTUELL");
		}
		else
		{
			if(Fahrstat_Date[Fahrstat_Counter][DAY][0] < 10) GLCD_WriteCharS('0');
			GLCD_WriteNumberS(Fahrstat_Date[Fahrstat_Counter][DAY][0]);
			GLCD_WriteCharS('.');
			if(Fahrstat_Date[Fahrstat_Counter][MONTH][0] < 10) GLCD_WriteCharS('0');
			GLCD_WriteNumberS(Fahrstat_Date[Fahrstat_Counter][MONTH][0]);
			GLCD_WriteStringS(".20");
			if(Fahrstat_Date[Fahrstat_Counter][YEAR][0] < 10) GLCD_WriteCharS('0');
			GLCD_WriteNumberS(Fahrstat_Date[Fahrstat_Counter][YEAR][0]);
		}

		GLCD_DrawSimpleLine(70,7,109,7,1);
		GLCD_SetInvert(0);

		if(Fahrstat_Counter == Fahrstat_Index)
		{
			// Make non-floating values visible
			SensorCalc_MotorRpmMax[1]--;
			SensorCalc_MotorTorqueMax[1]--;
			SensorCalc_MotorPowerMax[1]--;
			SensorCalc_SpeedMax[1]--;
			SensorCalc_SpeedAvg[1]--;
			SensorCalc_VerbrauchAvg[1]--;
			Sensor_MotorRuntime[1]--;
		}
		else
		{
			// Make non-floating values visible
			Fahrstat_MotorRpmMax[Fahrstat_Counter][1]--;
			Fahrstat_MotorLoadMax[Fahrstat_Counter][1]--;
			Fahrstat_MotorPowerMax[Fahrstat_Counter][1]--;
			Fahrstat_SpeedMax[Fahrstat_Counter][1]--;
			Fahrstat_SpeedAvg[Fahrstat_Counter][1]--;
			Fahrstat_VerbrauchAvg[Fahrstat_Counter][1]--;
			Fahrstat_MotorRuntime[Fahrstat_Counter][1]--;
		}

		while(1)
		{
			if(Fahrstat_Counter == Fahrstat_Index)
			{
				GLCD_WriteNumberZero_UNUM16(77,1,SensorCalc_MotorRpmMax		,4,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);
				GLCD_WriteNumberZero_UNUM16(85,2,SensorCalc_MotorLoadMax	,3,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);
				GLCD_WriteNumberZero_UNUM16(85,3,SensorCalc_MotorPowerMax	,3,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);
				GLCD_WriteNumberZero_UNUM8(85,4,SensorCalc_SpeedMax			,3,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);
				GLCD_WriteNumberZero_UNUM8(85,5,SensorCalc_SpeedAvg			,3,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);
				GLCD_WriteNumberZero_UNUM16(82,6,SensorCalc_VerbrauchAvg	,3,KOMMA,FONT_ZERO_7,RIGHT_ALIGN);
				GLCD_WriteNumberZero_UNUM16(85,7,Sensor_MotorRuntime		,3,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);
			}
			else
			{
				GLCD_WriteNumberZero_UNUM16(77,1,Fahrstat_MotorRpmMax[Fahrstat_Counter]		,4,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);
				GLCD_WriteNumberZero_UNUM8(85,2,Fahrstat_MotorLoadMax[Fahrstat_Counter]		,3,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);
				GLCD_WriteNumberZero_UNUM16(85,3,Fahrstat_MotorPowerMax[Fahrstat_Counter]	,3,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);
				GLCD_WriteNumberZero_UNUM8(85,4,Fahrstat_SpeedMax[Fahrstat_Counter]			,3,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);
				GLCD_WriteNumberZero_UNUM8(85,5,Fahrstat_SpeedAvg[Fahrstat_Counter]			,3,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);
				GLCD_WriteNumberZero_UNUM16(82,6,Fahrstat_VerbrauchAvg[Fahrstat_Counter]	,3,KOMMA,FONT_ZERO_7,RIGHT_ALIGN);
				GLCD_WriteNumberZero_UNUM16(85,7,Fahrstat_MotorRuntime[Fahrstat_Counter]	,3,NO_KOMMA,FONT_ZERO_7,RIGHT_ALIGN);
			}

			Wait200us();
			ExecuteISR();
			if(MENU_LEFT)
			{
				SearchEntryCounter = 0;
				do
				{
					Fahrstat_Counter--;
					if(Fahrstat_Counter == -1) Fahrstat_Counter = STATISTIC_MEMORY_SIZE-1;
					SearchEntryCounter++;
				}
				while(Fahrstat_Date[Fahrstat_Counter][YEAR][0] == 0 && SearchEntryCounter < STATISTIC_MEMORY_SIZE);
				break;
			}
			if(MENU_RIGHT)
			{
				SearchEntryCounter = 0;
				do
				{
					Fahrstat_Counter++;
					if(Fahrstat_Counter == STATISTIC_MEMORY_SIZE) Fahrstat_Counter = 0;
					SearchEntryCounter++;
				}
				while(Fahrstat_Date[Fahrstat_Counter][YEAR][0] == 0 && SearchEntryCounter < STATISTIC_MEMORY_SIZE);
				break;
			}
			if(MENU_DOWN || MENU_UP) return MAIN_MENU;
		}
	}

	#endif

	return MAIN_MENU;
}


// --- SPRINT --------------------------------------------------------------
unsigned char Sprint(void)
{
	#ifdef CFG_SPRINT_ENABLE
	
	unsigned char i;

	OBD_MenuUpdate(OBD_CYCLE_SPRINT);
	Menu_WriteEeprom(SPRINT);

	Sensor_Speed[1]--;

	// Clear LCD
	GLCD_DrawImage(Sprint_Frame,0,0,128,8);


	while(1)
	{
		GLCD_WriteNumberZero_UNUM8(1,5,Sensor_Speed					,3,NO_KOMMA,FONT_ZERO_22,LEFT_ALIGN);

		GLCD_GoTo(5,1);
		GLCD_SetFontBold(0);
		GLCD_WriteStringM("Rec: ");
		GLCD_WriteNumberM(Sprint_TimeInS_Record[0]/10);
		GLCD_WriteCharM('.');
		GLCD_WriteNumberM(Sprint_TimeInS_Record[0]%10);
		GLCD_WriteCharM('s');

		// Check Speed = 0
		if(Sensor_Speed[0] == 0)
		{
			GLCD_DrawImage(Sprint_Ready,104,1,24,2);
			while(1)
			{
				//if(Sensor_Speed[0] != 0) break;

				Wait200us();
				ExecuteISR();
				if(MENU_DOWN || MENU_UP) return MAIN_MENU;

				// Acknownledge Start
				if(MENU_RIGHT)
				{
					Sprint_TimeInS[0] = 0;
					Sprint_TimeInS[1]--;
					GLCD_SetInvert(1);
					GLCD_WriteNumberZero_UNUM8(73,5,Sprint_TimeInS	,2,KOMMA,FONT_ZERO_22,RIGHT_ALIGN);
					GLCD_SetInvert(0);

					// Wait Speed is changing
					while(Sensor_Speed[0] == 0)
					{
						Wait200us();
						ExecuteISR();
						if(MENU_DOWN || MENU_UP) return MAIN_MENU;
					}

					Do_FlagByte |= DO_SPRINT_RUN_START;

					GLCD_DrawImage(Sprint_Run,104,1,24,2);

					// Time is running...
					while(1)
					{
						// Display Time 
						if(Do_FlagByte & DO_SPRINT_NEW_100MS)
						{
							Do_FlagByte &= ~DO_SPRINT_NEW_100MS;

							Sprint_TimeInS[0]++;
							GLCD_SetInvert(1);
							GLCD_WriteNumberZero_UNUM8(73,5,Sprint_TimeInS	,2,KOMMA,FONT_ZERO_22,RIGHT_ALIGN);
							GLCD_SetInvert(0);
						
							GLCD_GoTo(3+Sprint_TimeInS[0],1);
							GLCD_WriteData(0xFF);

							if(Sprint_TimeInS[0] >= 99)
							{
								Do_FlagByte &= ~DO_SPRINT_RUN_START;
								break;
							}
						}

						// Display Speed
						GLCD_WriteNumberZero_UNUM8(1,5,Sensor_Speed			,3,NO_KOMMA,FONT_ZERO_22,LEFT_ALIGN);
						if(Sensor_Speed[0] >= 100)
						{
							Do_FlagByte &= ~DO_SPRINT_RUN_START;
							break;
						}

						Wait200us();
						ExecuteISR();
						if(MENU_DOWN || MENU_UP) return MAIN_MENU;
					}

					// Finish
					GLCD_DrawImage(Sprint_Finish,104,1,24,2);

					if(Sprint_TimeInS[0] < Sprint_TimeInS_Record[0])
					{
						Sprint_TimeInS_Record[0] = Sprint_TimeInS[0];
						Eeprom_WriteChar(EEP_ADR_SPRINT_RECORD,Sprint_TimeInS_Record[0]);
					}				

					while(1)
					{
						Wait200us();
						ExecuteISR();
						if(MENU_DOWN || MENU_UP) return MAIN_MENU;

						// Acknownledge Start
						if(MENU_RIGHT)
						{
							break;
						}
					}

					// Clear Balken
					GLCD_GoTo(3,1);
					for(i=0; i<99; i++) GLCD_WriteData(0x00);
				} 
			}
		}
		else
		{
			GLCD_DrawImage(Sprint_Wait,104,1,24,2);
		}


		Wait200us();
		ExecuteISR();
		if(MENU_DOWN || MENU_UP) return MAIN_MENU;

//		WaitInMs(1000);
//		Sensor_Speed[0] = 0;
	}

	#endif

	return MAIN_MENU;
}


// --- DIAGNOSE-------------------------------------------------------------
unsigned char Diagnose(void)
{
	#ifdef CFG_DIAGNOSE_ENABLE

	OBD_MenuUpdate(OBD_CYCLE_DIAGNOSE);
	Menu_WriteEeprom(DIAGNOSE);

	//	Elm_SendATCmdPlusString(ELM_PROGPAR_SET_ENABLE," 0C ON");
	//	WaitInMs(1000);
	//	Elm_SearchData();
	//	WaitInMs(1000);
	//	Elm_SendATCmdPlusString(ELM_PROGPAR_SET_VALUE, " 0C SV D0");	// D0 for 19200 Baud; 68 for 38400 Baud
	//	WaitInMs(1000);
	//	Elm_SearchData();
	//	while(1);

	//	Elm_SendATCmdPlusString(ELM_PROGPAR_SET_ENABLE," 03 ON");
	//	WaitInMs(1000);
	//	Elm_SearchData();
	//	Elm_SendATCmdPlusString(ELM_PROGPAR_SET_VALUE, " 03 SV 19");	// 32 for 205ms NODATA-Timeout; 19 for 102ms NODATA-Timeout
	//	WaitInMs(1000);
	//	Elm_SearchData();

	GLCD_FillScreen(0x00);
	GLCD_SetFontBold(0);

	GLCD_GoTo(0,0);
	GLCD_WriteStringM("JT:");
	GLCD_GoTo(0,1);
	GLCD_WriteStringM("WD:");
	GLCD_GoTo(0,2);
	GLCD_WriteStringM("BO:");
	GLCD_GoTo(0,3);
	GLCD_WriteStringM("PO:");
	GLCD_GoTo(0,4);
	GLCD_WriteStringM("EX:");
	GLCD_GoTo(0,5);
	GLCD_WriteStringM("BI:");
	GLCD_GoTo(0,7);
	GLCD_WriteStringM("RE:");

	GLCD_GoTo(40,0);
	GLCD_WriteStringM("Spd:");
	GLCD_GoTo(40,1);
	GLCD_WriteStringM("Rpm");
	GLCD_GoTo(40,2);
	GLCD_WriteStringM("ETp:");
	GLCD_GoTo(40,3);
	GLCD_WriteStringM("ITp:");
	GLCD_GoTo(40,4);
	GLCD_WriteStringM("Fue:");
	GLCD_GoTo(40,5);
	GLCD_WriteStringM("DTC:");
	GLCD_GoTo(40,6);
	GLCD_WriteStringM("Vlt:");
	GLCD_GoTo(40,7);
	GLCD_WriteStringM("CYC:");


	while(1)
	{
		GLCD_GoTo(15,0);
		GLCD_WriteNumberM(ResetErrorJTAG);
		GLCD_WriteCharM(' ');
		GLCD_GoTo(15,1);
		GLCD_WriteNumberM(ResetErrorWatchdog);
		GLCD_WriteCharM(' ');
		GLCD_GoTo(15,2);
		GLCD_WriteNumberM(ResetErrorBrownout);
		GLCD_WriteCharM(' ');
		GLCD_GoTo(15,3);
		GLCD_WriteNumberM(ResetErrorPoweron);
		GLCD_WriteCharM(' ');
		GLCD_GoTo(15,4);
		GLCD_WriteNumberM(ResetErrorExtern);
		GLCD_WriteCharM(' ');
		GLCD_GoTo(15,5);
		GLCD_WriteNumberM(ResetErrorBadInterrupt);
		GLCD_WriteCharM(' ');
		GLCD_GoTo(0,6);
		GLCD_WriteNumberM(ErrorNullCounterCpy);
		GLCD_WriteCharM('!');
		GLCD_WriteNumberM(ErrorNullCounterCat);
		GLCD_WriteCharM('!');
		GLCD_WriteNumberM(ErrorNullCounter1);
		GLCD_WriteCharM('|');
		GLCD_WriteNumberM(ErrorNullCounter2);
		GLCD_WriteCharM('|');
		GLCD_WriteNumberM(ErrorNullCounter3);
		GLCD_WriteCharM('|');
		GLCD_WriteNumberM(ErrorNullCounter4);
		GLCD_WriteCharM('|');
		GLCD_WriteNumberM(ErrorNullCounter5);
		GLCD_WriteCharM('|');
		GLCD_WriteNumberM(ErrorNullCounter6);
		GLCD_WriteCharM('|');
		GLCD_WriteNumberM(ErrorNullCounter7);
		GLCD_WriteCharM(' ');
		GLCD_GoTo(15,7);
		GLCD_WriteNumberM(ErrorCounter);
		GLCD_WriteCharM(' ');

		GLCD_GoTo(65,0);
		GLCD_WriteNumberM(Sensor_Speed[0]);
		GLCD_WriteCharM(' ');
		GLCD_GoTo(65,1);
		GLCD_WriteNumberM(Sensor_MotorRpm[0]);
		GLCD_WriteCharM(' ');
		GLCD_GoTo(65,2);
		GLCD_WriteNumberM(Sensor_Coolant_Temp[0]);
		GLCD_WriteCharM(' ');
		GLCD_GoTo(65,3);
		GLCD_WriteNumberM(Sensor_AirIntake_Temp[0]);
		GLCD_WriteCharM(' ');
		GLCD_GoTo(65,4);
		GLCD_WriteNumberM(Sensor_FuelShortTrim[0]);
		GLCD_WriteCharM(' ');
		GLCD_GoTo(65,5);
		GLCD_WriteNumberM(Sensor_NumberDTCs[0]);
		GLCD_WriteCharM(' ');
		GLCD_GoTo(65,6);
		GLCD_WriteNumberM(Sensor_Voltage[0]);
		GLCD_WriteCharM(' ');
		GLCD_GoTo(65,7);
		GLCD_WriteNumberM(OBD_CycleCompleteCounter);
		GLCD_WriteCharM(' ');

		GLCD_GoTo(95,0);
		GLCD_WriteNumberM(OBD_CycleCompleteCounter - ErrorSpeed);
		GLCD_WriteCharM(' ');
		GLCD_GoTo(95,1);
		GLCD_WriteNumberM(OBD_CycleCompleteCounter - ErrorRpm);
		GLCD_WriteCharM(' ');
		GLCD_GoTo(95,2);
		GLCD_WriteNumberM(OBD_CycleCompleteCounter - ErrorETemp);
		GLCD_WriteCharM(' ');
		GLCD_GoTo(95,3);
		GLCD_WriteNumberM(OBD_CycleCompleteCounter - ErrorITemp);
		GLCD_WriteCharM(' ');
		GLCD_GoTo(95,4);
		GLCD_WriteNumberM(OBD_CycleCompleteCounter - ErrorFuelS);
		GLCD_WriteCharM(' ');
		GLCD_GoTo(95,5);
		GLCD_WriteNumberM(OBD_CycleCompleteCounter - ErrorDTCs);
		GLCD_WriteCharM(' ');
		GLCD_GoTo(95,6);
		GLCD_WriteNumberM(OBD_CycleCompleteCounter - ErrorATRV);
		GLCD_WriteCharM(' ');
		GLCD_GoTo(95,7);
		GLCD_WriteNumberM(OBD_CycleCompleteCounter);
		GLCD_WriteCharM(' ');

		Wait200us();
		ExecuteISR();
		if(MENU_DOWN || MENU_UP) return MAIN_MENU;
	}

	#endif

	return MAIN_MENU;
}

