
// OBD.c

//============================================================================
//				HEADERS
//============================================================================

#include "EvoBox.h"
#include "Elm327.h"
#include "Sensors.h"
#include "OBD.h"
#include "LCD_KS0108.h"
#include "Menu.h"


//============================================================================
//				DEFINITIONS
//============================================================================


//============================================================================
//				GLOBAL VARIABLES
//============================================================================

volatile unsigned char OBD_MenuCycle;
volatile unsigned char OBD_CycleCounter;
unsigned short OBD_CycleCompleteCounter;

unsigned char OBD_CycleCheck[OBD_CYCLE_MENU_SIZE][OBD_CYCLE_CHECK_SIZE]=
{
	// OBD_CYCLE_MAIN_MENU
	{
		//PID_01_DTC_STATUS
		//PID_04_CALC_ENGINE_LOAD
		//PID_05_ENGINE_COOLANT_TEMP
		//PID_06_FUEL_SHORT_TRIM_BANK1
		//PID_07_FUEL_LONG_TRIM_BANK1
		//PID_0B_INTAKE_PRESS	
		//PID_0C_ENGINE_RPM
		//PID_0D_VEHICLE_SPEED
		//PID_0E_TIMING_ADVANCE
		//PID_0F_INTAKE_TEMP	
		//PID_10_MASS_AIR_FLOW
		//PID_11_THROTTLE_POS	
		//PID_14_O2_BANK1_SENSOR1	
		//PID_15_O2_BANK1_SENSOR2
		//ELM_READ_VOLTAGE
		1,							0,
		MODE_F_SHOW_ELM_AT_DATA,	ELM_READ_VOLTAGE,				//	V
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
	},

	// OBD_CYCLE_GAUGES_DIGITAL
	{
		//PID_06_FUEL_SHORT_TRIM_BANK1
		//PID_07_FUEL_LONG_TRIM_BANK1
		//PID_0B_INTAKE_PRESS	
		//PID_14_O2_BANK1_SENSOR1	
		//PID_15_O2_BANK1_SENSOR2
		//ELM_READ_VOLTAGE
		15,							0,								//	3 3 3 3		1 1	1	x in 4800ms
		MODE_1_SHOW_CURRENT_DATA,	PID_0C_ENGINE_RPM,				//	R
		MODE_1_SHOW_CURRENT_DATA,	PID_04_CALC_ENGINE_LOAD,		//	  L
		MODE_1_SHOW_CURRENT_DATA,	PID_0D_VEHICLE_SPEED,			//	    S
		MODE_1_SHOW_CURRENT_DATA,	PID_10_MASS_AIR_FLOW,			//		  M
		MODE_1_SHOW_CURRENT_DATA,	PID_05_ENGINE_COOLANT_TEMP,		//				E
		MODE_1_SHOW_CURRENT_DATA,	PID_0C_ENGINE_RPM,				//	R
		MODE_1_SHOW_CURRENT_DATA,	PID_04_CALC_ENGINE_LOAD,		//	  L
		MODE_1_SHOW_CURRENT_DATA,	PID_0D_VEHICLE_SPEED,			//	    S
		MODE_1_SHOW_CURRENT_DATA,	PID_10_MASS_AIR_FLOW,			//		  M
		MODE_1_SHOW_CURRENT_DATA,	PID_0F_INTAKE_TEMP,				//				  I
		MODE_1_SHOW_CURRENT_DATA,	PID_0C_ENGINE_RPM,				//	R
		MODE_1_SHOW_CURRENT_DATA,	PID_04_CALC_ENGINE_LOAD,		//	  L
		MODE_1_SHOW_CURRENT_DATA,	PID_0D_VEHICLE_SPEED,			//	    S
		MODE_1_SHOW_CURRENT_DATA,	PID_10_MASS_AIR_FLOW,			//		  M
		MODE_1_SHOW_CURRENT_DATA,	PID_01_DTC_STATUS,				//					D
		0,							0,
		0,							0,
		0,							0,
		0,							0,
	},
	
	// OBD_CYCLE_GAUGES_ANALOG
	{
		//PID_06_FUEL_SHORT_TRIM_BANK1
		//PID_07_FUEL_LONG_TRIM_BANK1
		//PID_0B_INTAKE_PRESS	
		//PID_14_O2_BANK1_SENSOR1	
		//PID_15_O2_BANK1_SENSOR2
		//ELM_READ_VOLTAGE
		12,							0,								//	3 3 3		1 1	1	x in 4800ms
		MODE_1_SHOW_CURRENT_DATA,	PID_0C_ENGINE_RPM,				//	R
		MODE_1_SHOW_CURRENT_DATA,	PID_0D_VEHICLE_SPEED,			//	  S
		MODE_1_SHOW_CURRENT_DATA,	PID_10_MASS_AIR_FLOW,			//		M
		MODE_1_SHOW_CURRENT_DATA,	PID_05_ENGINE_COOLANT_TEMP,		//				E
		MODE_1_SHOW_CURRENT_DATA,	PID_0C_ENGINE_RPM,				//	R
		MODE_1_SHOW_CURRENT_DATA,	PID_0D_VEHICLE_SPEED,			//	  S
		MODE_1_SHOW_CURRENT_DATA,	PID_10_MASS_AIR_FLOW,			//		M
		MODE_1_SHOW_CURRENT_DATA,	PID_0F_INTAKE_TEMP,				//				  I
		MODE_1_SHOW_CURRENT_DATA,	PID_0C_ENGINE_RPM,				//	R
		MODE_1_SHOW_CURRENT_DATA,	PID_0D_VEHICLE_SPEED,			//	  S
		MODE_1_SHOW_CURRENT_DATA,	PID_10_MASS_AIR_FLOW,			//		M
		MODE_1_SHOW_CURRENT_DATA,	PID_01_DTC_STATUS,				//					D
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
	},

	// OBD_CYCLE_SENSORS_MOTOR
	{
		19,							0,								//	3 3 3 3 3 3		1	x in 6080ms
		MODE_1_SHOW_CURRENT_DATA,	PID_0C_ENGINE_RPM,				//	R
		MODE_1_SHOW_CURRENT_DATA,	PID_04_CALC_ENGINE_LOAD,		//	  L
		MODE_1_SHOW_CURRENT_DATA,	PID_0D_VEHICLE_SPEED,			//		S
		MODE_1_SHOW_CURRENT_DATA,	PID_10_MASS_AIR_FLOW,			//		  M
		MODE_1_SHOW_CURRENT_DATA,	PID_11_THROTTLE_POS,			//			T
		MODE_1_SHOW_CURRENT_DATA,	PID_0E_TIMING_ADVANCE,			//			  A
		MODE_1_SHOW_CURRENT_DATA,	PID_05_ENGINE_COOLANT_TEMP,		//					E
		MODE_1_SHOW_CURRENT_DATA,	PID_0C_ENGINE_RPM,				//	R
		MODE_1_SHOW_CURRENT_DATA,	PID_04_CALC_ENGINE_LOAD,		//	  L
		MODE_1_SHOW_CURRENT_DATA,	PID_0D_VEHICLE_SPEED,			//		S
		MODE_1_SHOW_CURRENT_DATA,	PID_10_MASS_AIR_FLOW,			//		  M
		MODE_1_SHOW_CURRENT_DATA,	PID_11_THROTTLE_POS,			//			T
		MODE_1_SHOW_CURRENT_DATA,	PID_0E_TIMING_ADVANCE,			//			  A
		MODE_1_SHOW_CURRENT_DATA,	PID_0C_ENGINE_RPM,				//	R
		MODE_1_SHOW_CURRENT_DATA,	PID_04_CALC_ENGINE_LOAD,		//	  L
		MODE_1_SHOW_CURRENT_DATA,	PID_0D_VEHICLE_SPEED,			//		S
		MODE_1_SHOW_CURRENT_DATA,	PID_10_MASS_AIR_FLOW,			//		  M
		MODE_1_SHOW_CURRENT_DATA,	PID_11_THROTTLE_POS,			//			T
		MODE_1_SHOW_CURRENT_DATA,	PID_0E_TIMING_ADVANCE,			//			  A
	},

	// OBD_CYCLE_SENSORS_LUFT
	{
		17,							0,								//	4 4 4 4		1	x in 5440ms
		MODE_1_SHOW_CURRENT_DATA,	PID_10_MASS_AIR_FLOW,			//	M
		MODE_1_SHOW_CURRENT_DATA,	PID_14_O2_BANK1_SENSOR1,		//	  1
		MODE_1_SHOW_CURRENT_DATA,	PID_15_O2_BANK1_SENSOR2,		//		2
		MODE_1_SHOW_CURRENT_DATA,	PID_0B_INTAKE_PRESS,			//		  P
		MODE_1_SHOW_CURRENT_DATA,	PID_0F_INTAKE_TEMP,				//				T
		MODE_1_SHOW_CURRENT_DATA,	PID_10_MASS_AIR_FLOW,			//	M
		MODE_1_SHOW_CURRENT_DATA,	PID_14_O2_BANK1_SENSOR1,		//	  1
		MODE_1_SHOW_CURRENT_DATA,	PID_15_O2_BANK1_SENSOR2,		//		2
		MODE_1_SHOW_CURRENT_DATA,	PID_0B_INTAKE_PRESS,			//		  P
		MODE_1_SHOW_CURRENT_DATA,	PID_10_MASS_AIR_FLOW,			//	M
		MODE_1_SHOW_CURRENT_DATA,	PID_14_O2_BANK1_SENSOR1,		//	  1
		MODE_1_SHOW_CURRENT_DATA,	PID_15_O2_BANK1_SENSOR2,		//		2
		MODE_1_SHOW_CURRENT_DATA,	PID_0B_INTAKE_PRESS,			//		  P
		MODE_1_SHOW_CURRENT_DATA,	PID_10_MASS_AIR_FLOW,			//	M
		MODE_1_SHOW_CURRENT_DATA,	PID_14_O2_BANK1_SENSOR1,		//	  1
		MODE_1_SHOW_CURRENT_DATA,	PID_15_O2_BANK1_SENSOR2,		//		2
		MODE_1_SHOW_CURRENT_DATA,	PID_0B_INTAKE_PRESS,			//		  P
		0,							0,
		0,							0,
	},

	// OBD_CYCLE_SENSORS_BATTERY
	{
		2,							0,								//	1 1		x in 640ms
		MODE_1_SHOW_CURRENT_DATA,	PID_0C_ENGINE_RPM,				//	R
		MODE_F_SHOW_ELM_AT_DATA,	ELM_READ_VOLTAGE,				//	  V
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
	},

	// OBD_CYCLE_GRAPHS
	{
		1,							0,								//	1	x in 320ms
		MODE_F_SHOW_ELM_AT_DATA,	ELM_READ_VOLTAGE,				//	V
		MODE_1_SHOW_CURRENT_DATA,	PID_0D_VEHICLE_SPEED,			//		S
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
	},

	// STATISTICS
	{
		4,							0,								//	1 1 1 1		x in 1280ms
		MODE_1_SHOW_CURRENT_DATA,	PID_0C_ENGINE_RPM,				//	R
		MODE_1_SHOW_CURRENT_DATA,	PID_04_CALC_ENGINE_LOAD,		//	  L
		MODE_1_SHOW_CURRENT_DATA,	PID_0D_VEHICLE_SPEED,			//		S
		MODE_1_SHOW_CURRENT_DATA,	PID_10_MASS_AIR_FLOW,			//		  M
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
	},

	// SPRINT
	{
		1,							0,								//	1	x in 320ms
		MODE_1_SHOW_CURRENT_DATA,	PID_0D_VEHICLE_SPEED,			//	S
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
	},

	// DIAGNOSE
	{
		7,							0,								//	1 1 1 1	1 1 1	x in 2240ms
		MODE_1_SHOW_CURRENT_DATA,	PID_0D_VEHICLE_SPEED,			//	S
		MODE_1_SHOW_CURRENT_DATA,	PID_0C_ENGINE_RPM,				//	  R
		MODE_1_SHOW_CURRENT_DATA,	PID_05_ENGINE_COOLANT_TEMP,		//		E
		MODE_1_SHOW_CURRENT_DATA,	PID_0F_INTAKE_TEMP,				//		  I
		MODE_1_SHOW_CURRENT_DATA,	PID_06_FUEL_SHORT_TRIM_BANK1,	//			F
		MODE_1_SHOW_CURRENT_DATA,	PID_01_DTC_STATUS,				//			  D
		MODE_F_SHOW_ELM_AT_DATA,	ELM_READ_VOLTAGE,				//				V
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
	},

	// OPTIONS
	{
		1,							0,
		MODE_F_SHOW_ELM_AT_DATA,	ELM_READ_VOLTAGE,				//	V
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
		0,							0,
	},
};

unsigned short ErrorSpeed,ErrorRpm,ErrorETemp,ErrorITemp,ErrorFuelS,ErrorDTCs;


//============================================================================
//				FUNCTION PROTOTYPES
//============================================================================

void OBD_AnalyseOBDCmd(void);
void OBD_CyclicUpdate(void);
void OBD_MenuUpdate(unsigned char pMenuCycle);


//============================================================================
//				FUNCTIONS
//============================================================================

void OBD_AnalyseOBDCmd(void)
{
	unsigned char i;

	switch(Elm_GlobalRequestMode)
	{
		case MODE_1_SHOW_CURRENT_DATA:

			switch(Elm_GlobalRequestPID)
			{
				case PID_00_SUPPORTED_PIDS:
					if(Elm_ReadHexNumber(pToData,0) == 0xBE
					&& Elm_ReadHexNumber(pToData,2) == 0x3F
					&& Elm_ReadHexNumber(pToData,4) == 0xB8
					&& Elm_ReadHexNumber(pToData,6) == 0x10)
					{
						Do_FlagByte |= DO_OBD_CMD_OK;
					}
					break;

				case PID_01_DTC_STATUS:
					Sensor_NumberDTCs[0] = (Elm_ReadHexNumber(pToData,0) & 0x7F);
					if(Elm_ReadHexNumber(pToData,0) & 0x80)
					{
						Do_FlagByte |=  DO_OBD_MIL_ON;
					}
					else
					{
						Do_FlagByte &= ~DO_OBD_MIL_ON;
					}
					ErrorDTCs++;
					break;

				case PID_03_FUEL_SYSTEM_STATUS:
					break;

				case PID_04_CALC_ENGINE_LOAD:
					Sensor_MotorLoad[0] = (unsigned char)( (float)(Elm_ReadHexNumber(pToData,0)) / 2.55f + 0.5);
					if(Sensor_MotorLoad[0] > SensorCalc_MotorLoadMax[0]) SensorCalc_MotorLoadMax[0] = Sensor_MotorLoad[0];
					if(Sensor_MotorLoad[0] > Fahrstat_MotorLoadMax[Fahrstat_Index][0]) Fahrstat_MotorLoadMax[Fahrstat_Index][0] = Sensor_MotorLoad[0];
					break;

				case PID_05_ENGINE_COOLANT_TEMP:
					Sensor_Coolant_Temp[0] = Elm_ReadHexNumber(pToData,0) - 40;
					if(Sensor_Coolant_Temp[0] > SensorCalc_Coolant_TempMax[0]) SensorCalc_Coolant_TempMax[0] = Sensor_Coolant_Temp[0];
					if(Sensor_Coolant_Temp[0] < SensorCalc_Coolant_TempMin[0]) SensorCalc_Coolant_TempMin[0] = Sensor_Coolant_Temp[0];
					ErrorETemp++;
					break;

				case PID_06_FUEL_SHORT_TRIM_BANK1:
					Sensor_FuelShortTrim[0] = (unsigned char)( (float)(Elm_ReadHexNumber(pToData,0) - 128) / 1.28f);
					ErrorFuelS++;
					break;

				case PID_07_FUEL_LONG_TRIM_BANK1:
					Sensor_FuelLongTrim[0] = (unsigned char)( (float)(Elm_ReadHexNumber(pToData,0) - 128) / 1.28f);
					break;

				case PID_0B_INTAKE_PRESS:
					Sensor_AirIntake_Press[0] = (unsigned short)(Elm_ReadHexNumber(pToData,0)) * 10;
					break;

				case PID_0C_ENGINE_RPM:
					Sensor_MotorRpm[0] = ( (unsigned short)(256*(unsigned short)(Elm_ReadHexNumber(pToData,0)) + (unsigned short)(Elm_ReadHexNumber(pToData,2))) ) / 4;
					if(Sensor_MotorRpm[0] > SensorCalc_MotorRpmMax[0]) SensorCalc_MotorRpmMax[0] = Sensor_MotorRpm[0];
					if(Sensor_MotorRpm[0] > Fahrstat_MotorRpmMax[Fahrstat_Index][0]) Fahrstat_MotorRpmMax[Fahrstat_Index][0] = Sensor_MotorRpm[0];
					
					for(i=0; i<19; i++)
					{
						if(Sensor_MotorRpm[0] < SensorCalc_MotorTorque_Tab[0][i]) break;
					}
					// Drehmoment in Nm
					if(Sensor_MotorRpm[0] > 3500)
					{
						Sensor_MotorTorque[0] = SensorCalc_MotorTorque_Tab[1][i] + (SensorCalc_MotorTorque_Tab[0][i] - Sensor_MotorRpm[0]) * (SensorCalc_MotorTorque_Tab[1][i-1] - SensorCalc_MotorTorque_Tab[1][i]) / (SensorCalc_MotorTorque_Tab[0][i] - SensorCalc_MotorTorque_Tab[0][i-1]);
					}
					else
					{
						Sensor_MotorTorque[0] = SensorCalc_MotorTorque_Tab[1][i-1] + (Sensor_MotorRpm[0] - SensorCalc_MotorTorque_Tab[0][i-1]) * (SensorCalc_MotorTorque_Tab[1][i] - SensorCalc_MotorTorque_Tab[1][i-1]) / (SensorCalc_MotorTorque_Tab[0][i] - SensorCalc_MotorTorque_Tab[0][i-1]);
					}
					if(Sensor_MotorTorque[0] > SensorCalc_MotorTorqueMax[0]) SensorCalc_MotorTorqueMax[0] = Sensor_MotorTorque[0];
					if(Sensor_MotorTorque[0] > Fahrstat_MotorTorqueMax[Fahrstat_Index][0]) Fahrstat_MotorTorqueMax[Fahrstat_Index][0] = Sensor_MotorTorque[0];
					
					// Leistung in PS
					Sensor_MotorPower[0] = (unsigned short)( (float)(Sensor_MotorTorque[0]) * (float)(Sensor_MotorRpm[0]) / 7023.5f ); // /60  * 2 * 3.1416 / 1000 * 1.359622;
					if(Sensor_MotorPower[0] > SensorCalc_MotorPowerMax[0]) SensorCalc_MotorPowerMax[0] = Sensor_MotorPower[0];
					if(Sensor_MotorPower[0] > Fahrstat_MotorPowerMax[Fahrstat_Index][0]) Fahrstat_MotorPowerMax[Fahrstat_Index][0] = Sensor_MotorPower[0];
						
					ErrorRpm++;
					break;

				case PID_0D_VEHICLE_SPEED:

					// Geschwindigkeit
					Sensor_Speed[0] = Elm_ReadHexNumber(pToData,0);
					if(Sensor_Speed[0] > SensorCalc_SpeedMax[0]) SensorCalc_SpeedMax[0] = Sensor_Speed[0];
					if(Sensor_Speed[0] > Fahrstat_SpeedMax[Fahrstat_Index][0]) Fahrstat_SpeedMax[Fahrstat_Index][0] = Sensor_Speed[0];

					// Geschwindigkeit Durchschnitt
					Sensor_SpeedRaw = 1000000L * (unsigned long)(Sensor_Speed[0]);
					SensorCalc_SpeedAvgRaw = (unsigned long)( SensorCalc_SpeedAvgRaw + (Sensor_SpeedRaw/AVG_FILTER_SPEED) - (SensorCalc_SpeedAvgRaw/AVG_FILTER_SPEED) + 0.5f );
					SensorCalc_SpeedAvg[0] = SensorCalc_SpeedAvgRaw / 1000000L;

					// Geschwindigkeit Durchschnitt Statistik
					Fahrstat_SpeedRaw = 1000000L * (unsigned long)(Sensor_Speed[0]);
					Fahrstat_SpeedAvgRaw = (unsigned long)( Fahrstat_SpeedAvgRaw + (Fahrstat_SpeedRaw/AVG_FILTER_SPEED) - (Fahrstat_SpeedAvgRaw/AVG_FILTER_SPEED) + 0.5f );
					Fahrstat_SpeedAvg[Fahrstat_Index][0] = Fahrstat_SpeedAvgRaw / 1000000L;					

					ErrorSpeed++;
					break;

				case PID_0E_TIMING_ADVANCE:
					Sensor_AdvanceAngle[0] = (unsigned char)(Elm_ReadHexNumber(pToData,0))/2 - 64;
					break;

				case PID_0F_INTAKE_TEMP:
					Sensor_AirIntake_Temp[0] = Elm_ReadHexNumber(pToData,0) - 40;
					if(Sensor_AirIntake_Temp[0] > SensorCalc_AirIntake_TempMax[0]) SensorCalc_AirIntake_TempMax[0] = Sensor_AirIntake_Temp[0];
					if(Sensor_AirIntake_Temp[0] < SensorCalc_AirIntake_TempMin[0]) SensorCalc_AirIntake_TempMin[0] = Sensor_AirIntake_Temp[0];
					ErrorITemp++;
					break;

				case PID_10_MASS_AIR_FLOW:
					Sensor_MassAirFlow[0] = ( (unsigned short)(256*(unsigned short)(Elm_ReadHexNumber(pToData,0)) + (unsigned short)(Elm_ReadHexNumber(pToData,2))) + 0.5f);
					
					// Verbrauch berechnen
					if(Sensor_Speed[0])
					{
						SensorCalc_Verbrauch[0] = (unsigned short)( 0.331f * 10.0f * (float)(Sensor_MassAirFlow[0]) / (float)(Sensor_Speed[0]) + 0.5f );	// * 10 because of 1 comma digit
					}
					//Verbrauch schtzen ber Drehzahl und Last
					else
					{
						SensorCalc_Verbrauch[0] = FUEL_REF_VERBRAUCH * (float)(Sensor_MotorRpm[0])/FUEL_REF_RPM * (float)(Sensor_MotorLoad[0])/FUEL_REF_LOAD * 10.0f + 0.5f;	// * 10 because of 1 comma digit
					}
					if(SensorCalc_Verbrauch[0] > 999) SensorCalc_Verbrauch[0] = 999;

					// Verbrauch Durchschnitt
					SensorCalc_VerbrauchRaw = 1000000L * (unsigned long)(SensorCalc_Verbrauch[0]);
					SensorCalc_VerbrauchAvgRaw = (unsigned long)( SensorCalc_VerbrauchAvgRaw + (SensorCalc_VerbrauchRaw/AVG_FILTER_VERBRAUCH) - (SensorCalc_VerbrauchAvgRaw/AVG_FILTER_VERBRAUCH) + 0.5f );
					SensorCalc_VerbrauchAvg[0] = SensorCalc_VerbrauchAvgRaw / 1000000L;
					if(SensorCalc_VerbrauchAvg[0] > 999) SensorCalc_VerbrauchAvg[0] = 999;

					// Verbrauch Durchschnitt Statistik
					Fahrstat_VerbrauchRaw = 1000000L * (unsigned long)(SensorCalc_Verbrauch[0]);
					Fahrstat_VerbrauchAvgRaw = (unsigned long)( Fahrstat_VerbrauchAvgRaw + (Fahrstat_VerbrauchRaw/AVG_FILTER_VERBRAUCH) - (Fahrstat_VerbrauchAvgRaw/AVG_FILTER_VERBRAUCH) + 0.5f );
					Fahrstat_VerbrauchAvg[Fahrstat_Index][0] = Fahrstat_VerbrauchAvgRaw / 1000000L;
					if(Fahrstat_VerbrauchAvg[Fahrstat_Index][0] > 999) Fahrstat_VerbrauchAvg[Fahrstat_Index][0] = 999;

					break;

				case PID_11_THROTTLE_POS:
					Sensor_Throttle[0] = (unsigned char)( (float)(Elm_ReadHexNumber(pToData,0)) / 2.55f + 0.5);
					break;

				case PID_13_O2_SENSORS_PRESENT:
					break;

				case PID_14_O2_BANK1_SENSOR1:
					Sensor_O2Sensor1[0] = (unsigned char)( (float)(Elm_ReadHexNumber(pToData,2) - 128) * 10 / 1.28f);
					if(Sensor_O2Sensor1[0] > SensorCalc_O2Sensor1Max[0]) SensorCalc_O2Sensor1Max[0] = Sensor_O2Sensor1[0];
					if(Sensor_O2Sensor1[0] < SensorCalc_O2Sensor1Min[0]) SensorCalc_O2Sensor1Min[0] = Sensor_O2Sensor1[0];
					break;

				case PID_15_O2_BANK1_SENSOR2:
					Sensor_O2Sensor2[0] = (unsigned char)( (float)(Elm_ReadHexNumber(pToData,2) - 128) * 10 / 1.28f);
					if(Sensor_O2Sensor2[0] > SensorCalc_O2Sensor2Max[0]) SensorCalc_O2Sensor2Max[0] = Sensor_O2Sensor2[0];
					if(Sensor_O2Sensor2[0] < SensorCalc_O2Sensor2Min[0]) SensorCalc_O2Sensor2Min[0] = Sensor_O2Sensor2[0];
					break;

				case PID_1C_OBD_STANDARDS:
					break;

				default:
					break;
			}
			break;

		case MODE_2_SHOW_FREEZE_FRAME_DATA:
			break;

		case MODE_3_SHOW_DTC:
			break;

		case MODE_4_CLEAR_DTC_AND_DATA:
			break;

		case MODE_5_TEST_RESULT_O2_SENSOR:
			break;

		case MODE_6_TEST_RESULT_OTHER_COMP:
			break;

		case MODE_7_SHOW_PENDING_DTC:
			break;

		case MODE_8_SPECIAL_CONTROL_MODE:
			break;

		case MODE_9_REQUEST_VEHICLE_INFO:
			break;

		case MODE_A_PERMANENT_DTC:
			break;

		default :
			break;
	}

}


// --- CYCLIC DIAGNOSE------------------------------------------------------
void OBD_CyclicUpdate(void)
{
	if(OBD_CycleCheck[OBD_MenuCycle][0])
	{
		OBD_CycleCounter++;
		if((OBD_CycleCounter > OBD_CycleCheck[OBD_MenuCycle][0]) || (OBD_CycleCounter > OBD_CYCLE_CHECK_SIZE/2 - 1))
		{
			OBD_CycleCounter = 1;
			OBD_CycleCompleteCounter++;
		}

		if(OBD_CycleCheck[OBD_MenuCycle][2*OBD_CycleCounter] == MODE_F_SHOW_ELM_AT_DATA)
		{
			if(OBD_CycleCheck[OBD_MenuCycle][2*OBD_CycleCounter+1] == ELM_READ_VOLTAGE)
			{
				Elm_SendATCmd(ELM_READ_VOLTAGE);
			}
		}
		else
		{
			Elm_SendOBDCmd(ELM_SEND_OBD,OBD_CycleCheck[OBD_MenuCycle][2*OBD_CycleCounter],OBD_CycleCheck[OBD_MenuCycle][2*OBD_CycleCounter+1]);
		}

		Do_FlagByte |= DO_OBD_NEW_CYCLIC_CHECK;
	}
}

// --- CYCLIC DIAGNOSE------------------------------------------------------
void OBD_MenuUpdate(unsigned char pMenuCycle)
{
	OBD_CycleCounter = 0;
	OBD_MenuCycle = pMenuCycle;
}


