// Menu.c

//============================================================================
//				HEADERS
//============================================================================

#include <avr/io.h>
#include <stdlib.h>
#include <inttypes.h>
#include <avr/eeprom.h>

#include "EvoBox.h"
#include "Led.h"
#include "Buzzer.h"
#include "Buttons.h"
#include "LCD_KS0108.h"
#include "Fonts.h"
#include "Images.h"
#include "Menu.h"
#include "Mitsubishi.h"
#include "Sensors.h"
#include "Options.h"
#include "Elm327.h"
#include "OBD.h"
#include "Eeprom.h"


//============================================================================
//				DEFINITIONS
//============================================================================


//============================================================================
//				GLOBAL VARIABLES
//============================================================================

const char PROGMEM OptionsMenuText[][12]={
	"LICHT      ",
	"ANZEIGEN   ",
	"UHR+DATUM  ",
	"RESET MAX  ",
};

const char PROGMEM LightMenuText[][15]={
	"LCD-Helligkeit",
	"RGB-Farbe",
};

const char PROGMEM DisplaysMenuText[][17]={
	"Digital Variante",
	"Analog Variante",
	"OBD2-Delay",
	"Rem ",
};

const char PROGMEM TimeMenuText[][17]={
	"Zeit einstellen",
	"Datum einstellen",
};

const char PROGMEM ResetMenuText[][14]={
	"Motor",
	"Luft",
	"Batterie",
	"Sprint-Record",
	"Fahrstatistik",
};


//============================================================================
//				FUNCTION PROTOTYPES
//============================================================================

unsigned char Options(void);
void ShowMenuPointS_P(const char* pMenuText,unsigned char pMenuPoint,unsigned char pInvert,unsigned char pLines);
void ShowMenuPointM_P(const char* pMenuText,unsigned char pMenuPoint);


//============================================================================
//				OPTIONS
//============================================================================

unsigned char Options(void)
{
	#ifdef CFG_OPTIONS_ENABLE

	static signed char OptionsMenuPoint,LightMenuPoint,DisplaysMenuPoint,TimeMenuPoint,ResetMenuPoint;
	unsigned char TimeCursor,DateCursor,MenuPointCursor=0;
	const unsigned char TimeMax[2]={24,60};
	const unsigned char TimePos[2]={67,84};
	const unsigned char DateMax[3]={31,12,99};
	const unsigned char DatePos[3]={53,70,101};
	unsigned short EepromAdrOffset;
	unsigned char i;


	OBD_MenuUpdate(OBD_CYCLE_OPTIONS);
	Menu_WriteEeprom(OPTIONS);

	// Clear LCD
	GLCD_FillScreen(0x00);

	// Draw Headline
	GLCD_GoTo(0,0);
	for(i=0; i<128; i++) GLCD_WriteData(0x7F);

	GLCD_SetInvert(1);
	GLCD_SetFontSOffset(1);
	GLCD_GoTo(40,0);
	GLCD_WriteStringS("OPTIONEN");
	GLCD_SetInvert(0);

	GLCD_DrawSimpleLine(0,7,127,7,CLEAR);

	GLCD_DrawSimpleLine(OPTIONS_WIDTH,8,OPTIONS_WIDTH,63,FULL);


	// Write MenuPoints
	for(i=0; i<OPTIONS_MENU_SIZE; i++)
	{
		ShowMenuPointS_P(OptionsMenuText[i],i+1,NOT_INVERT,WITH_LINES);
	}	
	
	// Start at SetTime
	if(MenuPointStatus == OPTIONS_TIME) OptionsMenuPoint = OPTIONS_TIME;

	while(1)
	{
		// Write new MenuPoint
		ShowMenuPointS_P(OptionsMenuText[OptionsMenuPoint],OptionsMenuPoint+1,INVERT,WITH_LINES);

		WaitInMs(100);

		do
		{
			ExecuteISR();
		}
		while( !(MENU_UP || MENU_DOWN || MENU_LEFT || MENU_RIGHT) );

		// Erase old MenuPoint
		if(!MENU_RIGHT) ShowMenuPointS_P(OptionsMenuText[OptionsMenuPoint],OptionsMenuPoint+1,NOT_INVERT,WITH_LINES);

		// Check Buttons
		if(MENU_UP)
		{
			OptionsMenuPoint--;
			if(OptionsMenuPoint == -1) OptionsMenuPoint = OPTIONS_MENU_SIZE-1;
		}
		else if(MENU_DOWN)
		{
			OptionsMenuPoint++;
			if(OptionsMenuPoint == OPTIONS_MENU_SIZE) OptionsMenuPoint = 0;
		}
		else if(MENU_LEFT)
		{
			return MAIN_MENU;
		}
		else if(MENU_RIGHT)
		{

// --- Optionen LIGHT -------------------------------------------------------------------
			if(OptionsMenuPoint == OPTIONS_LIGHT)
			{
				// Write right MenuPoints
				for(i=0; i<OPTIONS_LIGHT_MENU_SIZE; i++)
				{
					ShowMenuPointM_P(LightMenuText[i],i+1);
				}

				GLCD_GoTo(116,1);
				GLCD_WriteNumberM(Led_LCDBrightness);

				GLCD_GoTo(99,2);
				GLCD_WriteNumberM(Led_Color+1);

				while(1)
				{
					// Mark new MenuPoint
					GLCD_GoTo(OPTIONS_WIDTH+OPTIONS_GAP,LightMenuPoint+1);
					GLCD_WriteCharM(FONT_M_ARROW);

					WaitInMs(100);

					do
					{
						ExecuteISR();
					}
					while( !(MENU_UP || MENU_DOWN || MENU_LEFT || MENU_RIGHT) );

					// Erase old MenuPoint
					GLCD_GoTo(OPTIONS_WIDTH+OPTIONS_GAP,LightMenuPoint+1);
					GLCD_WriteCharM(FONT_M_BLANKO);


					// Check Buttons
					if(MENU_UP)
					{
						LightMenuPoint--;
						if(LightMenuPoint == -1) LightMenuPoint = OPTIONS_LIGHT_MENU_SIZE-1;
					}
					else if(MENU_DOWN)
					{
						LightMenuPoint++;
						if(LightMenuPoint == OPTIONS_LIGHT_MENU_SIZE) LightMenuPoint = 0;
					}
					else if(MENU_LEFT)
					{
						break;
					}
					else if(MENU_RIGHT)
					{
			// --- Optionen LCD ---------------------------------------------------------
						if(LightMenuPoint == OPTIONS_LIGHT_LCD)
						{
							Led_LCDBrightness++;
							if(Led_LCDBrightness == LCD_MAX_BRIGHTNESS+1) Led_LCDBrightness = 0;
							
							GLCD_GoTo(116,1);
							GLCD_WriteNumberM(Led_LCDBrightness);
						}

			// --- Optionen RGB-LED -----------------------------------------------------
						else if(LightMenuPoint == OPTIONS_LIGHT_RGB_CHANGE)
						{
							Led_Color++;
							if(Led_Color == COLOR_NUMBERS) Led_Color = 0;

							GLCD_GoTo(99,2);
							GLCD_WriteNumberM(Led_Color+1);
							GLCD_WriteCharM(FONT_M_BLANKO);
						}
					}
				}
				// Store current LCD brightness
				Eeprom_WriteChar(EEP_ADR_LCD_BRIGHTNESS,Led_LCDBrightness);
				// Store current LED color
				Eeprom_WriteChar(EEP_ADR_LED_COLOR,Led_Color);
			}

// --- Optionen DISPLAYS ----------------------------------------------------------------
			else if(OptionsMenuPoint == OPTIONS_DISPLAYS)
			{
				// Write MenuPoints
				for(i=0; i<OPTIONS_DISPLAYS_MENU_SIZE; i++)
				{
					ShowMenuPointM_P(DisplaysMenuText[i],i+1);
				}

				GLCD_GoTo(121,1);
				GLCD_WriteNumberM(GaugesDigitalVariant+1);

				GLCD_GoTo(123,2);
				GLCD_WriteNumberM(GaugesAnalogVariant+1);

				GLCD_GoTo(110,3);
				GLCD_WriteNumberM(OBD2_Delay);

				while(1)
				{
					// Mark new MenuPoint
					GLCD_GoTo(OPTIONS_WIDTH+OPTIONS_GAP,DisplaysMenuPoint+1);
					GLCD_WriteCharM(FONT_M_ARROW);

					WaitInMs(100);

					do
					{
						ExecuteISR();
					}
					while( !(MENU_UP || MENU_DOWN || MENU_LEFT || MENU_RIGHT) );

					// Erase old MenuPoint
					GLCD_GoTo(OPTIONS_WIDTH+OPTIONS_GAP,DisplaysMenuPoint+1);
					GLCD_WriteCharM(FONT_M_BLANKO);


					// Check Buttons
					if(MENU_UP)
					{
						DisplaysMenuPoint--;
						if(DisplaysMenuPoint == -1) DisplaysMenuPoint = OPTIONS_DISPLAYS_MENU_SIZE-1;
					}
					else if(MENU_DOWN)
					{
						DisplaysMenuPoint++;
						if(DisplaysMenuPoint == OPTIONS_DISPLAYS_MENU_SIZE) DisplaysMenuPoint = 0;
					}
					else if(MENU_LEFT)
					{
						break;
					}
					else if(MENU_RIGHT)
					{
			// --- Optionen Digital -----------------------------------------------------
						if(DisplaysMenuPoint == OPTIONS_DISPLAYS_GAUGES_DIGITAL)
						{
							GaugesDigitalVariant++;
							if(GaugesDigitalVariant == 2) GaugesDigitalVariant = 0;

							GLCD_GoTo(121,DisplaysMenuPoint+1);
							GLCD_WriteNumberM(GaugesDigitalVariant+1);
						}

			// --- Optionen Analog ------------------------------------------------------
						else if(DisplaysMenuPoint == OPTIONS_DISPLAYS_GAUGES_ANALOG)
						{
							GaugesAnalogVariant++;
							if(GaugesAnalogVariant == 2) GaugesAnalogVariant = 0;
							
							GLCD_GoTo(123,DisplaysMenuPoint+1);
							GLCD_WriteNumberM(GaugesAnalogVariant+1);
						}
			// --- OBD2-Delay -----------------------------------------------------------
						else if(DisplaysMenuPoint == OPTIONS_DISPLAYS_OBD2_DELAY)
						{
							OBD2_Delay++;
							if(OBD2_Delay > 10) OBD2_Delay = 0;
							
							GLCD_GoTo(110,DisplaysMenuPoint+1);
							GLCD_WriteNumberM(OBD2_Delay);
						}
			// --- MenuPoint ------------------------------------------------------------
						else if(DisplaysMenuPoint == OPTIONS_DISPLAYS_MENU_POINT)
						{
							MenuPointCursor++;
							if(MenuPointCursor >= 3) MenuPointCursor = 0;
							
							GLCD_GoTo(76,DisplaysMenuPoint+1);
							     if(MenuPointCursor == 0) GLCD_WriteStringM("MainMenu  ");
							else if(MenuPointCursor == 1) GLCD_WriteStringM("LastMenu  ");
							else if(MenuPointCursor == 2) GLCD_WriteStringM("SetTime   ");
			    
							// Store in Eeprom
							if(MenuPointCursor == 0)
							{
							 	MenuPointStatus = 0;
								Eeprom_WriteChar(EEP_ADR_MENU_POINT_STATUS,	0);
							}
							else if(MenuPointCursor == 2)
							{
								MenuPointStatus = OPTIONS_TIME;
								Eeprom_WriteChar(EEP_ADR_MENU_POINT_STATUS,	OPTIONS_TIME);
							}
						}
					}
				}
				// Store Delay to EEPROM
				Eeprom_WriteChar(EEP_ADR_OBD2_DELAY, OBD2_Delay);

			}

// --- Optionen TIME+DATE ---------------------------------------------------------------
			else if(OptionsMenuPoint == OPTIONS_TIME)
			{
				// Write MenuPoints
				for(i=0; i<OPTIONS_TIME_MENU_SIZE; i++)
				{
					ShowMenuPointM_P(TimeMenuText[i],i+1);
				}

				GLCD_SetFontBold(BOLD);
				GLCD_GoTo(TimePos[0],4);
				if(Sensor_Time[HOUR][0] < 10) GLCD_WriteCharM('0');
				GLCD_WriteNumberM(Sensor_Time[HOUR][0]);
				GLCD_WriteCharM(':');
				if(Sensor_Time[MIN][0] < 10) GLCD_WriteCharM('0');
				GLCD_WriteNumberM(Sensor_Time[MIN][0]);

				GLCD_GoTo(DatePos[0],6);
				if(Sensor_Date[DAY][0] < 10) GLCD_WriteCharM('0');
				GLCD_WriteNumberM(Sensor_Date[DAY][0]);
				GLCD_WriteCharM('.');
				if(Sensor_Date[MONTH][0] < 10) GLCD_WriteCharM('0');
				GLCD_WriteNumberM(Sensor_Date[MONTH][0]);
				GLCD_WriteCharM('.');
				GLCD_WriteStringM("20");
				if(Sensor_Date[YEAR][0] < 10) GLCD_WriteCharM('0');
				GLCD_WriteNumberM(Sensor_Date[YEAR][0]);


				while(1)
				{
					// Mark new MenuPoint
					GLCD_SetFontBold(NORMAL);
					GLCD_GoTo(OPTIONS_WIDTH+OPTIONS_GAP,TimeMenuPoint+1);
					GLCD_WriteCharM(FONT_M_ARROW);

					WaitInMs(100);

					do
					{
						ExecuteISR();
					}
					while( !(MENU_UP || MENU_DOWN || MENU_LEFT || MENU_RIGHT) );

					// Erase old MenuPoint
					GLCD_GoTo(OPTIONS_WIDTH+OPTIONS_GAP,TimeMenuPoint+1);
					GLCD_WriteCharM(FONT_M_BLANKO);


					// Check Buttons
					if(MENU_UP)
					{
						TimeMenuPoint--;
						if(TimeMenuPoint == -1) TimeMenuPoint = OPTIONS_TIME_MENU_SIZE-1;
					}
					else if(MENU_DOWN)
					{
						TimeMenuPoint++;
						if(TimeMenuPoint == OPTIONS_TIME_MENU_SIZE) TimeMenuPoint = 0;
					}
					else if(MENU_LEFT)
					{
						break;
					}
					else if(MENU_RIGHT)
					{
			// --- Optionen Time --------------------------------------------------------
						if(TimeMenuPoint == OPTIONS_TIME_SET_TIME)
						{
							TimeCursor = 0;

							GLCD_SetFontBold(BOLD);

							while(1)
							{
								// Write new Time-Value
								GLCD_SetInvert(1);

								GLCD_GoTo(TimePos[TimeCursor],4);
								if(Sensor_Time[TimeCursor][0] < 10) GLCD_WriteCharM('0');
								GLCD_WriteNumberM(Sensor_Time[TimeCursor][0]);
								

								WaitInMs(100);

								do
								{
									ExecuteISR();
								}
								while( !(MENU_UP || MENU_DOWN || MENU_LEFT || MENU_RIGHT) );

								// Erase old Time-Value
								GLCD_SetInvert(0);

								GLCD_GoTo(TimePos[TimeCursor],4);
								if(Sensor_Time[TimeCursor][0] < 10) GLCD_WriteCharM('0');
								GLCD_WriteNumberM(Sensor_Time[TimeCursor][0]);


								// Check Buttons
								if(MENU_UP)
								{
									Sensor_Time[TimeCursor][0]++;
									if(Sensor_Time[TimeCursor][0] >= TimeMax[TimeCursor]) Sensor_Time[TimeCursor][0] = 0;
								}
								else if(MENU_DOWN)
								{
									Sensor_Time[TimeCursor][0]--;
									if(Sensor_Time[TimeCursor][0] == -1) Sensor_Time[TimeCursor][0] = TimeMax[TimeCursor]-1;
								}
								else if(MENU_LEFT)
								{
									break;
								}
								else if(MENU_RIGHT)
								{
									TimeCursor++;
									if(TimeCursor == 2) TimeCursor = 0;
								}
							}
							// Store current time
							Eeprom_WriteChar(EEP_ADR_SENSOR_TIME_HOUR,Sensor_Time[HOUR][0]);
							Eeprom_WriteChar(EEP_ADR_SENSOR_TIME_MIN,Sensor_Time[MIN][0]);
						}

			// --- Optionen Date --------------------------------------------------------
						else if(TimeMenuPoint == OPTIONS_TIME_SET_DATE)
						{
							DateCursor = 0;

							GLCD_SetFontBold(BOLD);

							while(1)
							{
								// Write new Time-Value
								GLCD_SetInvert(1);

								GLCD_GoTo(DatePos[DateCursor],6);
								if(Sensor_Date[DateCursor][0] < 10) GLCD_WriteCharM('0');
								GLCD_WriteNumberM(Sensor_Date[DateCursor][0]);
								

								WaitInMs(100);

								do
								{
									ExecuteISR();
								}
								while( !(MENU_UP || MENU_DOWN || MENU_LEFT || MENU_RIGHT) );

								// Erase old Time-Value
								GLCD_SetInvert(0);

								GLCD_GoTo(DatePos[DateCursor],6);
								if(Sensor_Date[DateCursor][0] < 10) GLCD_WriteCharM('0');
								GLCD_WriteNumberM(Sensor_Date[DateCursor][0]);


								// Check Buttons
								if(MENU_UP)
								{
									Sensor_Date[DateCursor][0]++;
									if(Sensor_Date[DateCursor][0] >= DateMax[DateCursor]) Sensor_Date[DateCursor][0] = 0;
								}
								else if(MENU_DOWN)
								{
									Sensor_Date[DateCursor][0]--;
									if(Sensor_Date[DateCursor][0] == -1) Sensor_Date[DateCursor][0] = DateMax[DateCursor];
								}
								else if(MENU_LEFT)
								{
									break;
								}
								else if(MENU_RIGHT)
								{
									DateCursor++;
									if(DateCursor == 3) DateCursor = 0;
								}
							}	
							// Store current date
							Eeprom_WriteChar(EEP_ADR_SENSOR_DATE_DAY,Sensor_Date[DAY][0]);
							Eeprom_WriteChar(EEP_ADR_SENSOR_DATE_MONTH,Sensor_Date[MONTH][0]);
							Eeprom_WriteChar(EEP_ADR_SENSOR_DATE_YEAR,Sensor_Date[YEAR][0]);
						}
					}
				}
			}

// --- Optionen RESETS ------------------------------------------------------------------
			else if(OptionsMenuPoint == OPTIONS_RESET)
			{
				// Write MenuPoints
				for(i=0; i<OPTIONS_RESET_MENU_SIZE; i++)
				{
					ShowMenuPointM_P(ResetMenuText[i],i+1);
				}

				while(1)
				{
					// Mark new MenuPoint
					GLCD_SetFontBold(NORMAL);
					GLCD_GoTo(OPTIONS_WIDTH+OPTIONS_GAP,ResetMenuPoint+1);
					GLCD_WriteCharM(FONT_M_ARROW);

					WaitInMs(100);

					do
					{
						ExecuteISR();
					}
					while( !(MENU_UP || MENU_DOWN || MENU_LEFT || MENU_RIGHT) );

					// Erase old MenuPoint
					GLCD_GoTo(OPTIONS_WIDTH+OPTIONS_GAP,ResetMenuPoint+1);
					GLCD_WriteCharM(FONT_M_BLANKO);


					// Check Buttons
					if(MENU_UP)
					{
						ResetMenuPoint--;
						if(ResetMenuPoint == -1) ResetMenuPoint = OPTIONS_RESET_MENU_SIZE-1;
					}
					else if(MENU_DOWN)
					{
						ResetMenuPoint++;
						if(ResetMenuPoint == OPTIONS_RESET_MENU_SIZE) ResetMenuPoint = 0;
					}
					else if(MENU_LEFT)
					{
						break;
					}
					else if(MENU_RIGHT)
					{
/*						if(ResetMenuPoint == OPTIONS_RESET_VERBRAUCH)
						{
							SensorCalc_VerbrauchAvg[0] = 0;
							Eeprom_WriteNumber(EEP_ADR_VERBRAUCH_AVG		,SensorCalc_VerbrauchAvg[0]);


						}
*/						if(ResetMenuPoint == OPTIONS_RESET_MOTOR)
						{
							Sensor_MotorResetTime = 0;
							SensorCalc_MotorRpmMax[0] = 0;
							SensorCalc_MotorTorqueMax[0] = 0;
							SensorCalc_MotorPowerMax[0] = 0;
							SensorCalc_MotorLoadMax[0] = 0;
							SensorCalc_Coolant_TempMin[0] = 127;
							SensorCalc_Coolant_TempMax[0] = -128;
							SensorCalc_MotorRuntimeTotal[0] = 0;
							Eeprom_WriteNumber(EEP_ADR_MOTOR_RESETTIME		,Sensor_MotorResetTime);
							Eeprom_WriteNumber(EEP_ADR_MOTOR_RPM_MAX		,SensorCalc_MotorRpmMax[0]);
							Eeprom_WriteNumber(EEP_ADR_MOTOR_TORQUE_MAX		,SensorCalc_MotorTorqueMax[0]);
							Eeprom_WriteNumber(EEP_ADR_MOTOR_POWER_MAX		,SensorCalc_MotorPowerMax[0]);
							Eeprom_WriteChar(EEP_ADR_MOTOR_LOAD_MAX			,SensorCalc_MotorLoadMax[0]);
							Eeprom_WriteChar(EEP_ADR_COOLANT_TEMP_MIN		,SensorCalc_Coolant_TempMin[0]);
							Eeprom_WriteChar(EEP_ADR_COOLANT_TEMP_MAX		,SensorCalc_Coolant_TempMax[0]);
							Eeprom_WriteNumber(EEP_ADR_MOTOR_RUNTIME_TOTAL	,SensorCalc_MotorRuntimeTotal[0]);
						}
						else if(ResetMenuPoint == OPTIONS_RESET_LUFT)
						{
							Sensor_LuftResetTime = 0;
							SensorCalc_AirIntake_TempMin[0] = 127;
							SensorCalc_AirIntake_TempMax[0] = -128;
 							SensorCalc_O2Sensor1Min[0]=127;
							SensorCalc_O2Sensor1Max[0]=-128;
 							SensorCalc_O2Sensor2Min[0]=127;
							SensorCalc_O2Sensor2Max[0]=-128;
							Eeprom_WriteNumber(EEP_ADR_LUFT_RESETTIME		,Sensor_LuftResetTime);
							Eeprom_WriteChar(EEP_ADR_AIRINTAKE_TEMP_MIN		,SensorCalc_AirIntake_TempMin[0]);
							Eeprom_WriteChar(EEP_ADR_AIRINTAKE_TEMP_MAX		,SensorCalc_AirIntake_TempMax[0]);
							Eeprom_WriteChar(EEP_ADR_O2SENSOR1_MIN			,SensorCalc_O2Sensor1Min[0]);
							Eeprom_WriteChar(EEP_ADR_O2SENSOR1_MAX			,SensorCalc_O2Sensor1Max[0]);
							Eeprom_WriteChar(EEP_ADR_O2SENSOR2_MIN			,SensorCalc_O2Sensor2Min[0]);
							Eeprom_WriteChar(EEP_ADR_O2SENSOR2_MAX			,SensorCalc_O2Sensor2Max[0]);
						}
						else if(ResetMenuPoint == OPTIONS_RESET_BATTERY)
						{
							Sensor_VoltageResetTime = 0;
							SensorCalc_VoltageMin[0] = 255;
							SensorCalc_VoltageMax[0] = 0;
							SensorCalc_Voltage0RpmMin[0] = 255;
							SensorCalc_Voltage0RpmMax[0] = 0;
							Eeprom_WriteNumber(EEP_ADR_VOLTAGE_RESETTIME	,Sensor_VoltageResetTime);
							Eeprom_WriteChar(EEP_ADR_VOLTAGE_MIN			,SensorCalc_VoltageMin[0]);
							Eeprom_WriteChar(EEP_ADR_VOLTAGE_MAX			,SensorCalc_VoltageMax[0]);
							Eeprom_WriteChar(EEP_ADR_VOLTAGE0RPM_MIN		,SensorCalc_Voltage0RpmMin[0]);
							Eeprom_WriteChar(EEP_ADR_VOLTAGE0RPM_MAX		,SensorCalc_Voltage0RpmMax[0]);
						}
						else if(ResetMenuPoint == OPTIONS_RESET_SPRINT_RECORD)
						{
							Sprint_TimeInS_Record[0] = 99;
							Eeprom_WriteChar(EEP_ADR_SPRINT_RECORD			,Sprint_TimeInS_Record[0]);
						}
						else if(ResetMenuPoint == OPTIONS_RESET_STATISTICS)
						{
							Fahrstat_Index = 0;
							Eeprom_WriteChar(EEP_ADR_STATISTIK_START		,Fahrstat_Index);
							for(i=0; i<STATISTIC_MEMORY_SIZE; i++)
							{
								EepromAdrOffset = EEP_ADR_STATISTIK_START - i*0x0010;
								Fahrstat_Date[i][DAY][0] = 0;							
								Fahrstat_Date[i][MONTH][0] = 0;
								Fahrstat_Date[i][YEAR][0] = 0;							
								Fahrstat_MotorRpmMax[i][0] = 0;
								Fahrstat_MotorLoadMax[i][0] = 0;
								Fahrstat_MotorPowerMax[i][0] = 0;
								Fahrstat_SpeedMax[i][0] = 0;
								Fahrstat_SpeedAvg[i][0] = 0;
								Fahrstat_VerbrauchAvg[i][0] = 0;
								Fahrstat_MotorRuntime[i][0] = 0;

								Eeprom_WriteChar(EepromAdrOffset + EEP_OFFSET_STATISTIK_DATE_DAY		,Fahrstat_Date[i][DAY][0]);
								Eeprom_WriteChar(EepromAdrOffset + EEP_OFFSET_STATISTIK_DATE_MONTH		,Fahrstat_Date[i][MONTH][0]);
								Eeprom_WriteChar(EepromAdrOffset + EEP_OFFSET_STATISTIK_DATE_YEAR		,Fahrstat_Date[i][YEAR][0]);
								Eeprom_WriteNumber(EepromAdrOffset + EEP_OFFSET_STATISTIK_RPM_MAX		,Fahrstat_MotorRpmMax[i][0]);
								Eeprom_WriteChar(EepromAdrOffset + EEP_OFFSET_STATISTIK_LOAD_MAX		,Fahrstat_MotorLoadMax[i][0]);
								Eeprom_WriteNumber(EepromAdrOffset + EEP_OFFSET_STATISTIK_POWER_MAX		,Fahrstat_MotorPowerMax[i][0]);
								Eeprom_WriteChar(EepromAdrOffset + EEP_OFFSET_STATISTIK_SPEED_MAX		,Fahrstat_SpeedMax[i][0]);
								Eeprom_WriteChar(EepromAdrOffset + EEP_OFFSET_STATISTIK_SPEED_AVG		,Fahrstat_SpeedAvg[i][0]);
								Eeprom_WriteNumber(EepromAdrOffset + EEP_OFFSET_STATISTIK_VERBRAUCH_AVG	,Fahrstat_VerbrauchAvg[i][0]);
								Eeprom_WriteNumber(EepromAdrOffset + EEP_OFFSET_STATISTIK_RUNTIME		,Fahrstat_MotorRuntime[i][0]);
							}
						}

						// Write OK-Haken
						GLCD_SetFontBold(BOLD);
						GLCD_GoTo(120,ResetMenuPoint+1);
						GLCD_WriteCharM(FONT_MBOLD_HAKEN);
						WaitInMs(1000);
						GLCD_GoTo(120,ResetMenuPoint+1);
						GLCD_WriteCharM(FONT_MBOLD_BLANKO_HAKEN);
					}
				}
			}

			// Delete right MenuPoint area
			GLCD_DrawImage(Options_ClearScreen,48,1,80,7);
		}
	}

	#endif

	return MAIN_MENU;
}


// Show Options SMALL 5x4
void ShowMenuPointS_P(const char* pMenuText,unsigned char pMenuPoint,unsigned char pInvert,unsigned char pLines)
{	
	#ifdef CFG_OPTIONS_ENABLE

	GLCD_DrawSimpleLine(0,8*pMenuPoint+8,OPTIONS_WIDTH-1,8*pMenuPoint+8,pInvert*0xFF);
		
	if(pInvert == 1) LCDInvert = 1;
	GLCD_SetFontSOffset(2);
	GLCD_GoTo(0,pMenuPoint);
	GLCD_WriteStringS_P(pMenuText);
	LCDInvert = 0;

	if(pLines)
	{	
		GLCD_DrawSimpleLine(0,8*pMenuPoint,OPTIONS_WIDTH-1,8*pMenuPoint,0xFF);
	}
	
	#endif		
}

// Show Options MEDIUM 7xX
void ShowMenuPointM_P(const char* pMenuText,unsigned char pMenuPoint)
{	
	#ifdef CFG_OPTIONS_ENABLE

	GLCD_SetFontBold(NORMAL);
	GLCD_GoTo(OPTIONS_WIDTH+OPTIONS_GAP+5,pMenuPoint);
	GLCD_WriteStringM_P(pMenuText);

	#endif
}
