// Menu.c

//============================================================================
//				HEADERS
//============================================================================

#include "LCD_KS0108.h"
#include "Sensors.h"
#include "Eeprom.h"
#include "Led.h"
#include "EvoBox.h"


//============================================================================
//				DEFINITIONS
//============================================================================


//============================================================================
//				GLOBAL VARIABLES
//============================================================================

/****************************** SENSOR VALUES ******************************/
// first value in array is the current measurement
// second value in array is the last one in previous measurement


// MotorRuntime
unsigned short Sensor_MotorRuntime[2]={0,0},		SensorCalc_MotorRuntimeTotal[2];

// Motor
unsigned short Sensor_MotorRpm[2]={0,0},			SensorCalc_MotorRpmMax[2]={0,0};
unsigned short SensorCalc_MotorRpm_ShiftFlash[2][6] = { {13750,13250,12750,12750,12750,20000},
														{    0, 3000, 6250, 6250, 6250, 6250} };													  														
unsigned short Sensor_MotorTorque[2]={0,0},			SensorCalc_MotorTorqueMax[2]={0,0};
const unsigned short SensorCalc_MotorTorque_Tab[2][19] = { {0,500,1000,1500,2000,2500,3000,3500,4000,4500,5000,5500,6000,6500,7000,7500,8000,8500,9000},
													       {0,115, 143, 178, 234, 305, 343, 355, 349, 348, 350, 338, 323, 303, 277, 257, 237, 217, 197} };
unsigned short Sensor_MotorPower[2]={0,0},			SensorCalc_MotorPowerMax[2]={0,0};
unsigned char  Sensor_MotorLoad[2]={0,0},			SensorCalc_MotorLoadMax[2]={0,0};
unsigned char  Sensor_Throttle[2]={0,0};
  signed char  Sensor_AdvanceAngle[2]={0,0};
  signed char  Sensor_Coolant_Temp[2]={0,0},		SensorCalc_Coolant_TempMin[2]={127,0},SensorCalc_Coolant_TempMax[2]={-128,0};
unsigned short Sensor_MotorResetTime;

// Luft
  signed char  Sensor_AirIntake_Temp[2]={0,0},		SensorCalc_AirIntake_TempMin[2]={127,0},SensorCalc_AirIntake_TempMax[2]={-128,0};
unsigned short Sensor_AirIntake_Press[2]={0,0};
unsigned short Sensor_MassAirFlow[2]={0,0};
  signed char  Sensor_O2Sensor1[2]={0,0},			SensorCalc_O2Sensor1Min[2]={127,0},SensorCalc_O2Sensor1Max[2]={-128,0};
  signed char  Sensor_O2Sensor2[2]={0,0},			SensorCalc_O2Sensor2Min[2]={127,0},SensorCalc_O2Sensor2Max[2]={-128,0};
unsigned short Sensor_LuftResetTime;

// Dynamik
unsigned char  Sensor_Speed[2]={0,0},				SensorCalc_SpeedMax[2]={0,0},SensorCalc_SpeedAvg[2]={0,0};
unsigned long  Sensor_SpeedRaw,						SensorCalc_SpeedAvgRaw;
unsigned char  Sprint_TimeInS[2]={0,0},				Sprint_TimeInS_Record[2]={0,0};

// Batterie
unsigned char  Sensor_Voltage[2]={0,0},				SensorCalc_VoltageMin[2]={255,0},SensorCalc_VoltageMax[2]={0,0};
unsigned char  Sensor_Voltage0Rpm[2]={0,0},			SensorCalc_Voltage0RpmMin[2]={255,0},SensorCalc_Voltage0RpmMax[2]={0,0};
unsigned short Sensor_VoltageResetTime;

// Verbrauch & Tank
  signed short Sensor_FuelShortTrim[2]={0,0};
  signed short Sensor_FuelLongTrim[2]={0,0};
unsigned short SensorCalc_Verbrauch[2]={0,0},		SensorCalc_VerbrauchAvg[2]={0,0};
unsigned long  SensorCalc_VerbrauchRaw,				SensorCalc_VerbrauchAvgRaw;

// Uhrzeit, Datum
  signed char  Sensor_Date[3][2]={{1,0},{1,0},{10,0}};
  signed char  Sensor_Time[3][2]={{12,0},{0,0},{0,0}};

// Diagnose
unsigned char  Sensor_NumberDTCs[2]={0,0};

// Fahrstatistik
unsigned char  Fahrstat_Index;
unsigned char  Fahrstat_Date[STATISTIC_MEMORY_SIZE][3][2];
unsigned short Fahrstat_MotorRpmMax[STATISTIC_MEMORY_SIZE][2];
unsigned short Fahrstat_MotorTorqueMax[STATISTIC_MEMORY_SIZE][2];
unsigned char  Fahrstat_MotorLoadMax[STATISTIC_MEMORY_SIZE][2];
unsigned short Fahrstat_MotorPowerMax[STATISTIC_MEMORY_SIZE][2];
unsigned char  Fahrstat_SpeedMax[STATISTIC_MEMORY_SIZE][2],Fahrstat_SpeedAvg[STATISTIC_MEMORY_SIZE][2];
unsigned long  Fahrstat_SpeedRaw,Fahrstat_SpeedAvgRaw;
unsigned short Fahrstat_VerbrauchAvg[STATISTIC_MEMORY_SIZE][2];
unsigned long  Fahrstat_VerbrauchRaw,Fahrstat_VerbrauchAvgRaw;
unsigned short Fahrstat_MotorRuntime[STATISTIC_MEMORY_SIZE][2];


//============================================================================
//				FUNCTION PROTOTYPES
//============================================================================

void Sensor_Statistic_ReadEeprom(void);
void Sensor_Statistic_WriteEeprom(void);
void Sensor_MaxValues_ReadEeprom(void);
void Sensor_MaxValues_WriteEeprom(void);


//============================================================================
//				SENSORS
//============================================================================

void Sensor_Statistic_ReadEeprom(void)
{
	unsigned char i;
	unsigned short EepromAdrOffset;

	for(i=0; i<STATISTIC_MEMORY_SIZE; i++)
	{
		EepromAdrOffset = EEP_ADR_STATISTIK_START - i*0x10;

		Eeprom_ReadChar(Fahrstat_Date[i][DAY][0]		,EepromAdrOffset + EEP_OFFSET_STATISTIK_DATE_DAY);
		Eeprom_ReadChar(Fahrstat_Date[i][MONTH][0]		,EepromAdrOffset + EEP_OFFSET_STATISTIK_DATE_MONTH);
		Eeprom_ReadChar(Fahrstat_Date[i][YEAR][0]		,EepromAdrOffset + EEP_OFFSET_STATISTIK_DATE_YEAR);
		Eeprom_ReadNumber(Fahrstat_MotorRpmMax[i][0]	,EepromAdrOffset + EEP_OFFSET_STATISTIK_RPM_MAX);
		Eeprom_ReadChar(Fahrstat_MotorLoadMax[i][0]		,EepromAdrOffset + EEP_OFFSET_STATISTIK_LOAD_MAX);
		Eeprom_ReadNumber(Fahrstat_MotorPowerMax[i][0]	,EepromAdrOffset + EEP_OFFSET_STATISTIK_POWER_MAX);
		Eeprom_ReadChar(Fahrstat_SpeedMax[i][0]			,EepromAdrOffset + EEP_OFFSET_STATISTIK_SPEED_MAX);
		Eeprom_ReadChar(Fahrstat_SpeedAvg[i][0]			,EepromAdrOffset + EEP_OFFSET_STATISTIK_SPEED_AVG);
		Eeprom_ReadNumber(Fahrstat_VerbrauchAvg[i][0]	,EepromAdrOffset + EEP_OFFSET_STATISTIK_VERBRAUCH_AVG);
		Eeprom_ReadNumber(Fahrstat_MotorRuntime[i][0]	,EepromAdrOffset + EEP_OFFSET_STATISTIK_RUNTIME);
	}
}

void Sensor_Statistic_WriteEeprom(void)
{
	static unsigned char Counter = 0;
	unsigned short EepromAdrOffset;

	EepromAdrOffset = EEP_ADR_STATISTIK_START - Fahrstat_Index*0x0010;

	if(Counter == 0)
	{
		Eeprom_WriteChar(EepromAdrOffset + EEP_OFFSET_STATISTIK_DATE_DAY		,Sensor_Date[DAY][0]);
		Eeprom_WriteChar(EepromAdrOffset + EEP_OFFSET_STATISTIK_DATE_MONTH		,Sensor_Date[MONTH][0]);
		Eeprom_WriteChar(EepromAdrOffset + EEP_OFFSET_STATISTIK_DATE_YEAR		,Sensor_Date[YEAR][0]);
	}
	else if(Counter == 1)
	{
		Eeprom_WriteNumber(EepromAdrOffset + EEP_OFFSET_STATISTIK_RPM_MAX		,Fahrstat_MotorRpmMax[Fahrstat_Index][0]);
		Eeprom_WriteChar(EepromAdrOffset + EEP_OFFSET_STATISTIK_LOAD_MAX		,Fahrstat_MotorLoadMax[Fahrstat_Index][0]);
	}
	else if(Counter == 2)
	{
		Eeprom_WriteNumber(EepromAdrOffset + EEP_OFFSET_STATISTIK_POWER_MAX		,Fahrstat_MotorPowerMax[Fahrstat_Index][0]);
		Eeprom_WriteChar(EepromAdrOffset + EEP_OFFSET_STATISTIK_SPEED_MAX		,Fahrstat_SpeedMax[Fahrstat_Index][0]);
		Eeprom_WriteChar(EepromAdrOffset + EEP_OFFSET_STATISTIK_SPEED_AVG		,Fahrstat_SpeedAvg[Fahrstat_Index][0]);
	}
	else if(Counter == 3)
	{
		Eeprom_WriteNumber(EepromAdrOffset + EEP_OFFSET_STATISTIK_VERBRAUCH_AVG	,Fahrstat_VerbrauchAvg[Fahrstat_Index][0]);
		Eeprom_WriteNumber(EepromAdrOffset + EEP_OFFSET_STATISTIK_RUNTIME		,Sensor_MotorRuntime[0]);
	}

	Counter++;
	if(Counter == 4) Counter = 1;
}

void Sensor_MaxValues_ReadEeprom(void)
{
	unsigned short ShortH,ShortL;

	Eeprom_ReadNumber(SensorCalc_MotorRuntimeTotal[0]	,EEP_ADR_MOTOR_RUNTIME_TOTAL);
	Eeprom_ReadNumber(SensorCalc_MotorRpmMax[0]			,EEP_ADR_MOTOR_RPM_MAX);
	Eeprom_ReadNumber(SensorCalc_MotorTorqueMax[0]		,EEP_ADR_MOTOR_TORQUE_MAX);
	Eeprom_ReadNumber(SensorCalc_MotorPowerMax[0]		,EEP_ADR_MOTOR_POWER_MAX);
	Eeprom_ReadChar(SensorCalc_MotorLoadMax[0]			,EEP_ADR_MOTOR_LOAD_MAX);
	Eeprom_ReadChar(SensorCalc_Coolant_TempMin[0]		,EEP_ADR_COOLANT_TEMP_MIN);
	Eeprom_ReadChar(SensorCalc_Coolant_TempMax[0]		,EEP_ADR_COOLANT_TEMP_MAX);
	Eeprom_ReadChar(SensorCalc_AirIntake_TempMin[0]		,EEP_ADR_AIRINTAKE_TEMP_MIN);
	Eeprom_ReadChar(SensorCalc_AirIntake_TempMax[0]		,EEP_ADR_AIRINTAKE_TEMP_MAX);
	Eeprom_ReadChar(SensorCalc_O2Sensor1Min[0]			,EEP_ADR_O2SENSOR1_MIN);
	Eeprom_ReadChar(SensorCalc_O2Sensor1Max[0]			,EEP_ADR_O2SENSOR1_MAX);
	Eeprom_ReadChar(SensorCalc_O2Sensor2Min[0]			,EEP_ADR_O2SENSOR2_MIN);
	Eeprom_ReadChar(SensorCalc_O2Sensor2Max[0]			,EEP_ADR_O2SENSOR2_MAX);
	Eeprom_ReadChar(SensorCalc_SpeedMax[0]				,EEP_ADR_SPEED_MAX);

	Eeprom_ReadNumber(ShortH							,EEP_ADR_SPEED_AVG_HIGH);
	Eeprom_ReadNumber(ShortL							,EEP_ADR_SPEED_AVG_LOW);
	SensorCalc_SpeedAvgRaw = 0x10000*ShortH + ShortL;

	Eeprom_ReadNumber(ShortH							,EEP_ADR_VERBRAUCH_AVG_HIGH);
	Eeprom_ReadNumber(ShortL							,EEP_ADR_VERBRAUCH_AVG_LOW);
	SensorCalc_VerbrauchAvgRaw = 0x10000*ShortH + ShortL;

	Eeprom_ReadChar(Sprint_TimeInS_Record[0]			,EEP_ADR_SPRINT_RECORD);

	Eeprom_ReadChar(SensorCalc_VoltageMin[0]			,EEP_ADR_VOLTAGE_MIN);
	Eeprom_ReadChar(SensorCalc_VoltageMax[0]			,EEP_ADR_VOLTAGE_MAX);
	Eeprom_ReadChar(SensorCalc_Voltage0RpmMin[0]		,EEP_ADR_VOLTAGE0RPM_MIN);
	Eeprom_ReadChar(SensorCalc_Voltage0RpmMax[0]		,EEP_ADR_VOLTAGE0RPM_MAX);

	Eeprom_ReadNumber(Sensor_MotorResetTime				,EEP_ADR_MOTOR_RESETTIME);
	Eeprom_ReadNumber(Sensor_LuftResetTime				,EEP_ADR_LUFT_RESETTIME);
	Eeprom_ReadNumber(Sensor_VoltageResetTime			,EEP_ADR_VOLTAGE_RESETTIME);

	Eeprom_ReadChar(Sensor_Date[DAY][0]					,EEP_ADR_SENSOR_DATE_DAY);
	Eeprom_ReadChar(Sensor_Date[MONTH][0]				,EEP_ADR_SENSOR_DATE_MONTH);
	Eeprom_ReadChar(Sensor_Date[YEAR][0]				,EEP_ADR_SENSOR_DATE_YEAR);
	Eeprom_ReadChar(Sensor_Time[HOUR][0]				,EEP_ADR_SENSOR_TIME_HOUR);
	Eeprom_ReadChar(Sensor_Time[MIN][0]					,EEP_ADR_SENSOR_TIME_MIN);

	Eeprom_ReadChar(OBD2_Delay							,EEP_ADR_OBD2_DELAY);
}

void Sensor_MaxValues_WriteEeprom(void)
{
	static unsigned char Counter = 0;

	if(Counter == 0)
	{
		Eeprom_WriteNumber(EEP_ADR_MOTOR_RUNTIME_TOTAL	,SensorCalc_MotorRuntimeTotal[0]);
		Eeprom_WriteNumber(EEP_ADR_MOTOR_RPM_MAX		,SensorCalc_MotorRpmMax[0]);
	}
	else if(Counter == 1)
	{
		Eeprom_WriteNumber(EEP_ADR_SPEED_AVG_HIGH		,(unsigned short)(SensorCalc_SpeedAvgRaw/65536));
		Eeprom_WriteNumber(EEP_ADR_SPEED_AVG_LOW		,(unsigned short)(SensorCalc_SpeedAvgRaw%65536));
		Eeprom_WriteNumber(EEP_ADR_VERBRAUCH_AVG_HIGH	,(unsigned short)(SensorCalc_VerbrauchAvgRaw/65536));
		Eeprom_WriteNumber(EEP_ADR_VERBRAUCH_AVG_LOW	,(unsigned short)(SensorCalc_VerbrauchAvgRaw%65536));
	}
	else if(Counter == 2)
	{
		Eeprom_WriteNumber(EEP_ADR_MOTOR_TORQUE_MAX		,SensorCalc_MotorTorqueMax[0]);
		Eeprom_WriteNumber(EEP_ADR_MOTOR_POWER_MAX		,SensorCalc_MotorPowerMax[0]);
	}
	else if(Counter == 3)
	{
		Eeprom_WriteChar(EEP_ADR_MOTOR_LOAD_MAX			,SensorCalc_MotorLoadMax[0]);
		Eeprom_WriteChar(EEP_ADR_COOLANT_TEMP_MIN		,SensorCalc_Coolant_TempMin[0]);
		Eeprom_WriteChar(EEP_ADR_COOLANT_TEMP_MAX		,SensorCalc_Coolant_TempMax[0]);
		Eeprom_WriteChar(EEP_ADR_AIRINTAKE_TEMP_MIN		,SensorCalc_AirIntake_TempMin[0]);
		Eeprom_WriteChar(EEP_ADR_AIRINTAKE_TEMP_MAX		,SensorCalc_AirIntake_TempMax[0]);
	}
	else if(Counter == 4)
	{
		Eeprom_WriteChar(EEP_ADR_O2SENSOR1_MIN			,SensorCalc_O2Sensor1Min[0]);
		Eeprom_WriteChar(EEP_ADR_O2SENSOR1_MAX			,SensorCalc_O2Sensor1Max[0]);
		Eeprom_WriteChar(EEP_ADR_O2SENSOR2_MIN			,SensorCalc_O2Sensor2Min[0]);
		Eeprom_WriteChar(EEP_ADR_O2SENSOR2_MAX			,SensorCalc_O2Sensor2Max[0]);

		Eeprom_WriteNumber(EEP_ADR_SPEED_MAX			,SensorCalc_SpeedMax[0]);
	}
	else if(Counter == 5)
	{
		Eeprom_WriteChar(EEP_ADR_VOLTAGE_MIN			,SensorCalc_VoltageMin[0]);
		Eeprom_WriteChar(EEP_ADR_VOLTAGE_MAX			,SensorCalc_VoltageMax[0]);
		Eeprom_WriteChar(EEP_ADR_VOLTAGE0RPM_MIN		,SensorCalc_Voltage0RpmMin[0]);
		Eeprom_WriteChar(EEP_ADR_VOLTAGE0RPM_MAX		,SensorCalc_Voltage0RpmMax[0]);
	}
	else if(Counter == 6)
	{
		Eeprom_WriteChar(EEP_ADR_SENSOR_DATE_DAY		,Sensor_Date[DAY][0]);
		Eeprom_WriteChar(EEP_ADR_SENSOR_DATE_MONTH		,Sensor_Date[MONTH][0]);
		Eeprom_WriteChar(EEP_ADR_SENSOR_DATE_YEAR		,Sensor_Date[YEAR][0]);
		Eeprom_WriteChar(EEP_ADR_SENSOR_TIME_HOUR		,Sensor_Time[HOUR][0]);
		Eeprom_WriteChar(EEP_ADR_SENSOR_TIME_MIN		,Sensor_Time[MIN][0]);
	}
	else if(Counter == 7)
	{
		Eeprom_WriteNumber(EEP_ADR_MOTOR_RESETTIME		,Sensor_MotorResetTime);
		Eeprom_WriteNumber(EEP_ADR_LUFT_RESETTIME		,Sensor_LuftResetTime);
		Eeprom_WriteNumber(EEP_ADR_VOLTAGE_RESETTIME	,Sensor_VoltageResetTime);
	}

	Counter++;
	if(Counter == 8) Counter = 0;
}
