
//============================================================================
//				INFOS
//============================================================================
/*
- WaitInMs angepasst
- Cmd-Lngenbeschrnkung auf 7 Zeichen
- bei 38400 Baud: Resets, bei 19200 Baud nicht
- Kondesnsator 100n/10n an RXD-Pin? eher nicht

*/


//============================================================================
//				TO-DO: Priority 1(high), 2(mid), 3(low)
//============================================================================
/*
- '?' als GetData abfangen
- Timings/Timout optimieren -> ELM_OBD_ADAPTIVE_TIMING_AUTO
- CAN probieren
- AT MA probieren
- Umlaute einfgen
- Search freigeben, wenn erste Bytes empfangen werden
- alle Images neu konvertieren
- lange InitZeit optimieren
- Elm_Init if-Checks ineinander verschachteln
- Adaptive Timing is enabled?
- Platine mit Gummi-Puffern dmpfen!
- Mode3 FEhlerspeicher
- Luftdurchsatz durch 100, Vorsicht bei Berechnung Verbrauch
- Voltage & Co. auch in anderen Menus messen (Gauges)
- OBD_MenuPoint mit Pointer anwhlen
- ExecuteISR() bringt C zum Absturz -> untersuchen, welcher Teil verantwortlich!
- 100nF zw. Vcc und GND bzw Reset und GND
- Pullup an Rx
- In SendCmd gleich SearchCmd bauen und ber switch-case abfragen
- Sensors: Fahrt: Speed, Avg, Verbrauch, Avg, Runtime, Runttime Total
- freie Flche in DigitalGauge (Gear)
- SprintRecord korrekt speichern
- Geile Tittenbilder
*/


//============================================================================
//				HEADERS
//============================================================================

#include <avr/io.h>
#include <stdlib.h>
#include <string.h>
#include <inttypes.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <avr/wdt.h>

#include "EvoBox.h"
#include "Led.h"
#include "Buttons.h"
#include "LCD_KS0108.h"
#include "Buzzer.h"
#include "Menu.h"
#include "Options.h"
#include "Mitsubishi.h"
#include "Sensors.h"
#include "Eeprom.h"
#include "Elm327.h"
#include "uart.h"
#include "OBD.h"


//============================================================================
//				DEFINITIONS
//============================================================================


//============================================================================
//				GLOBAL VARIABLES
//============================================================================

volatile  signed char  MenuPoint = MAIN_MENU,MenuPointStatus;
volatile unsigned short Tasks_FlagByte = 0;
volatile unsigned short Do_FlagByte = 0;
volatile unsigned short WaitInMsCounter;
unsigned char  ErrorCounter;
unsigned char  ResetErrorWatchdog,ResetErrorJTAG,ResetErrorPoweron,ResetErrorExtern,ResetErrorBrownout;
volatile unsigned char  ResetErrorBadInterrupt;
unsigned char  OBD2_Delay;


//============================================================================
//				FUNCTION PROTOTYPES
//============================================================================

void Init(void);
void PortsInit(void);
void Timer1Init(void);
void WaitInMs(unsigned short pLoop);
void Wait200us(void);
void ExecuteISR(void);


//============================================================================
//				FUNCTIONS
//============================================================================

// --- Init ------------------------------------------------------------------
void Init(void)
{
	unsigned short i;

	PortsInit();

	Timer1Init();

	WaitInMs(10);				// Wait a little time to startup LCD
	GLCD_Initalize();			// set LCD on
	
	//memset(StringBuffer,sizeof(StringBuffer),0);
	//memset(NumberBuffer,sizeof(NumberBuffer),0);
	
	for(i=0; i<33; i++)
	{
		StringBuffer[i] = 0;
		NumberBuffer[i] = 0;
	}

	uart_init(UART_BAUD_SELECT(19200,20000000L)); 
	UCSR0A |= U2X0;				// Atmega644 Datasheet page 190 Table 17-12

	sei();
}

// --- Init: Ports -----------------------------------------------------------
void PortsInit(void)
{
	DDRA  = 0b11111111;	// Pin0-7:LCD-Data(Output)
	PORTA = 0b00000000;

	DDRB  = 0b00011111;	// Pin0-4:LCD-Ctrl(Output)
	PORTB = 0b00000000;

	DDRC  = 0b11110010; // Pin4-6:RGB-Led(Output), Pin7:LCD-Backlight(Output), Pin1:Buzzer
	PORTC = 0b00000000; // Pin4-6:RGB-Led(Output)-Off, Pin7:LCD-Backlight(Output)-Off, Pin1:Buzzer-Off

	DDRD  = 0b00000110; // Pin4-7:Buttons(Input),Pin1:UART-TX
	PORTD = 0b01111000; // Pin4-7:PullUp
}

// --- Init: Timer1 ----------------------------------------------------------
void Timer1Init(void)
{
	// T=1ms @ 20MHz: 2500
	OCR1A = 2500;						// Timer1 Output Compare Register Bytes
	TIMSK1 |= (1<<OCIE1A);				// Timer1 Output CompareA Match Interrupt Enable
	TCCR1B |= (1<<CS11);				// Timer1 Start - Prescaler 8
}

// --- Wait ------------------------------------------------------------------
void WaitInMs(unsigned short pLoop)
{
	int i,j;

	for(i=0; i<pLoop; i++)
	{
		// 10MHz:2489 || 16MHz:2023 || 2MHz(intern):221
		for(j=0; j<2489; j++) __asm__ __volatile__ ("nop");
	}


//	WaitInMsCounter = pLoop;
//	Do_FlagByte |= DO_WAIT_IN_MS_LOOP;

//	while(Do_FlagByte & DO_WAIT_IN_MS_LOOP)
//	{
//		ExecuteISR();
//	}
}

void Wait200us(void)
{
	int j;
	for(j=0; j<500; j++) __asm__ __volatile__ ("nop");
}

// --- Execution of ISR-Tasks ------------------------------------------------
void ExecuteISR(void)
{
	static unsigned char  Counter4ms, Counter8ms, Counter64ms, MotorRuntime1s, Stat1s, Stat1sCompare = 10;

	// 1ms-Task
	if(Tasks_FlagByte & ISR_TASK_1MS_FLAG)
	{		
		Tasks_FlagByte &= ~ISR_TASK_1MS_FLAG;
//		Led_RefreshLCDBacklight();
//		Led_RefreshRGBLedColor();
		Buzzer_RefreshToggle();
	}	

	// 2ms-Task
	if(Tasks_FlagByte & ISR_TASK_2MS_FLAG)
	{	
		Tasks_FlagByte &= ~ISR_TASK_2MS_FLAG;
	}	

	// 4ms-Task
	if(Tasks_FlagByte & ISR_TASK_4MS_FLAG)
	{
		Tasks_FlagByte &= ~ISR_TASK_4MS_FLAG;
		Buttons_Check();

		Counter4ms++;

		// 25 * 4ms = 100ms
		if(Counter4ms == 25)		
		{				
			Counter4ms = 0;
			Tasks_FlagByte |= ISR_TASK_100MS_FLAG;
		}	
	}	

	// 8ms-Task
	if(Tasks_FlagByte & ISR_TASK_8MS_FLAG)
	{
		Tasks_FlagByte &= ~ISR_TASK_8MS_FLAG;
		Counter8ms++;

		// 1000ms-Task
		// 125 * 8ms = 1000ms = 1s
		if(Counter8ms == 125)		
		{				
			Counter8ms = 0;
			Tasks_FlagByte |= ISR_TASK_1000MS_FLAG;
		}	
	}	

	// 16ms-Task
	if(Tasks_FlagByte & ISR_TASK_16MS_FLAG)
	{
		Tasks_FlagByte &= ~ISR_TASK_16MS_FLAG;
	}	

	// 32ms-Task
	if(Tasks_FlagByte & ISR_TASK_32MS_FLAG)
	{
		Tasks_FlagByte &= ~ISR_TASK_32MS_FLAG;
		Led_RefreshToggle();
	}	

	// 64ms-Task
	if(Tasks_FlagByte & ISR_TASK_64MS_FLAG)
	{
		Tasks_FlagByte &= ~ISR_TASK_64MS_FLAG;
		
		Counter64ms++;
		#ifdef CFG_OBD_SIMULATION_ENABLE
		if(Counter64ms == 10)		// every 640ms because of "SEARCHING..." from ELM327
		#else
		if(Counter64ms == OBD2_Delay)		// 5 = every 320ms
		#endif
		{
			Counter64ms = 0;
			Elm_SearchData();		// Search the OBD 64ms before
			OBD_CyclicUpdate();		// Send new OBD
		}
	}	

	// 128ms-Task
	if(Tasks_FlagByte & ISR_TASK_128MS_FLAG)
	{
		Tasks_FlagByte &= ~ISR_TASK_128MS_FLAG;	
	}
	
	// 100ms-Task
	if(Tasks_FlagByte & ISR_TASK_100MS_FLAG)
	{
		Tasks_FlagByte &= ~ISR_TASK_100MS_FLAG;	

		// Sprint
		if(Do_FlagByte & DO_SPRINT_RUN_START)
		{
			Do_FlagByte |= DO_SPRINT_NEW_100MS;
		}
	}

	// 1000ms-Task
	if(Tasks_FlagByte & ISR_TASK_1000MS_FLAG)
	{
		Tasks_FlagByte &= ~ISR_TASK_1000MS_FLAG;	

		// Time
		Sensor_Time[SEC][0]++;
		if(Sensor_Time[SEC][0] == 60)
		{
			Sensor_Time[SEC][0] = 0;
			Sensor_Time[MIN][0]++;
			if(Sensor_Time[MIN][0] == 60)
			{
				Sensor_Time[MIN][0] = 0;
				Sensor_Time[HOUR][0]++;
				if(Sensor_Time[HOUR][0] == 24)
				{
					Sensor_Time[HOUR][0] = 0;
					Sensor_Date[DAY][0]++;
				}
			}

			// ResetTimes in Minutes
			Sensor_MotorResetTime++;
			Sensor_LuftResetTime++;
			Sensor_VoltageResetTime++;
		}

		// Motor runtime
		if(Sensor_MotorRpm[0] > 250)
		{
			MotorRuntime1s++;
			if(MotorRuntime1s == 60) 
			{
				MotorRuntime1s = 0;
				Sensor_MotorRuntime[0]++;
				SensorCalc_MotorRuntimeTotal[0]++;
			}
		}

		Stat1s++;

		// Store cyclically statistic data
		if(Stat1s == Stat1sCompare/2 || Stat1s == Stat1sCompare - 1)
		{
			Sensor_MaxValues_WriteEeprom();
		}

		// Store cyclically statistic data
		if(Stat1s == Stat1sCompare)
		{
			Stat1s = 0;
			Stat1sCompare += 10;
			if(Stat1sCompare > 180) Stat1sCompare = 180;
			Sensor_Statistic_WriteEeprom();
		}
	}	
}


//============================================================================
//				INTERRUPTS
//============================================================================

// --- Interrupt: Timer1 -----------------------------------------------------
ISR(TIMER1_COMPA_vect)
{
	static unsigned char ScheduleTaskCounter;

	cli();
	TCNT1 = 0;
	
	// Schedule Timer
	ScheduleTaskCounter++;
	if(ScheduleTaskCounter == 255) ScheduleTaskCounter = 0;
	
	// Tasks																			// every 1ms
											   Tasks_FlagByte |= ISR_TASK_1MS_FLAG;
	     if((ScheduleTaskCounter & 0x01) == 0) Tasks_FlagByte |= ISR_TASK_2MS_FLAG;		// every 2ms
	else if((ScheduleTaskCounter & 0x02) == 0) Tasks_FlagByte |= ISR_TASK_4MS_FLAG;		// every 4ms
	else if((ScheduleTaskCounter & 0x04) == 0) Tasks_FlagByte |= ISR_TASK_8MS_FLAG;		// every 8ms
	else if((ScheduleTaskCounter & 0x08) == 0) Tasks_FlagByte |= ISR_TASK_16MS_FLAG;	// every 16ms
	else if((ScheduleTaskCounter & 0x10) == 0) Tasks_FlagByte |= ISR_TASK_32MS_FLAG;	// every 32ms
	else if((ScheduleTaskCounter & 0x20) == 0) Tasks_FlagByte |= ISR_TASK_64MS_FLAG;	// every 64ms
	else if((ScheduleTaskCounter & 0x40) == 0) Tasks_FlagByte |= ISR_TASK_128MS_FLAG;	// every 128ms

	// WaitInMs
	if(WaitInMsCounter) WaitInMsCounter--;
	else Do_FlagByte &= ~DO_WAIT_IN_MS_LOOP;

	// LEDs
	Led_RefreshLCDBacklight();
	Led_RefreshRGBLedColor();

	sei();
}


ISR(BADISR_vect)
{
	Eeprom_WriteNumber(EEP_ADR_RESET_BAD_INT,ResetErrorBadInterrupt);
}


//============================================================================
//				MAIN()
//============================================================================

int main(void)
{
	unsigned char ResetSource,Temp;

	Init();

	Eeprom_ReadChar(Temp,EEP_ADR_LCD_BRIGHTNESS);
	Led_SetLCDBacklight(Temp);
	Led_RefreshLCDBacklight();
	Led_RefreshLCDBacklight();
	Led_RefreshLCDBacklight();
	Led_RefreshLCDBacklight();
	Led_RefreshLCDBacklight();

	Led_SetRGBLedColor(WHITE);
	Led_RefreshRGBLedColor();
	Led_SetToggle(0,PEAK);

	Buzzer_SetToggle(0);
	Buzzer_RefreshToggle();


	Eeprom_ReadNumber(ErrorCounter,EEP_ADR_ERROR_COUNTER);
	Eeprom_WriteNumber(EEP_ADR_ERROR_COUNTER,ErrorCounter+1);

	Eeprom_ReadNumber(ResetErrorJTAG,EEP_ADR_RESET_JTAG);
	Eeprom_ReadNumber(ResetErrorWatchdog,EEP_ADR_RESET_WATCHDOG);
	Eeprom_ReadNumber(ResetErrorBrownout,EEP_ADR_RESET_BROWNOUT);
	Eeprom_ReadNumber(ResetErrorPoweron,EEP_ADR_RESET_POWERON);
	Eeprom_ReadNumber(ResetErrorExtern,EEP_ADR_RESET_EXTERN);
	Eeprom_ReadNumber(ResetErrorBadInterrupt,EEP_ADR_RESET_BAD_INT);



	ResetSource = MCUSR;
	if((ResetSource & JTRF))
	{
		ResetSource &= ~JTRF;
		Eeprom_WriteNumber(EEP_ADR_RESET_JTAG,ResetErrorJTAG+1);
	}
	if((ResetSource & WDRF))
	{
		ResetSource &= ~WDRF;
		Eeprom_WriteNumber(EEP_ADR_RESET_WATCHDOG,ResetErrorWatchdog+1);
	}
	if((ResetSource & BORF))
	{
		ResetSource &= ~BORF;
		Eeprom_WriteNumber(EEP_ADR_RESET_BROWNOUT,ResetErrorBrownout+1);
	}
	if((ResetSource & EXTRF))
	{
		ResetSource &= ~EXTRF;
		Eeprom_WriteNumber(EEP_ADR_RESET_EXTERN,ResetErrorExtern+1);
	}
	if((ResetSource & PORF))
	{
		ResetSource &= ~PORF;
		Eeprom_WriteNumber(EEP_ADR_RESET_POWERON,ResetErrorPoweron+1);
	}
	MCUSR = ResetSource;


	LCDStartTheme();

	Elm_Init();


	// Index for Statistics
	Eeprom_ReadChar(Fahrstat_Index,EEP_ADR_STATISTIK_START);
	Fahrstat_Index++;
	if(Fahrstat_Index >= STATISTIC_MEMORY_SIZE) Fahrstat_Index = 0;
	Eeprom_WriteChar(EEP_ADR_STATISTIK_START,Fahrstat_Index);

	// LED
	Eeprom_ReadChar(Temp,EEP_ADR_LED_COLOR);
	Led_SetRGBLedColor(Temp);
	Led_RefreshRGBLedColor();

	// Read MaxValues from EEPROM
	Sensor_MaxValues_ReadEeprom();

	// MenuPoint
	Eeprom_ReadChar(MenuPointStatus,EEP_ADR_MENU_POINT_STATUS);
	if(MenuPointStatus == 0) MenuPoint = MAIN_MENU;
	else if(MenuPointStatus == OPTIONS_TIME) MenuPoint = OPTIONS;
	else MenuPoint = MenuPointStatus;

	while(1)
	{
		// Go to certain function and remember the return value for next Menu on LCD
		     if(MenuPoint == MAIN_MENU) 		MenuPoint = MainMenu();
		else if(MenuPoint == GAUGES_DIGITAL) 	MenuPoint = GaugesDigital();
		else if(MenuPoint == GAUGES_ANALOG) 	MenuPoint = GaugesAnalog();
		else if(MenuPoint == SENSORS) 			MenuPoint = Sensors();
		else if(MenuPoint == GRAPHS) 			MenuPoint = Graphs();
		else if(MenuPoint == STATISTICS) 		MenuPoint = Statistics();
		else if(MenuPoint == SPRINT) 			MenuPoint = Sprint();
		else if(MenuPoint == DIAGNOSE) 			MenuPoint = Diagnose();
		else if(MenuPoint == OPTIONS) 			MenuPoint = Options();
	}

	return 0;
}
